package cz.muni.fi.mail;


import cz.muni.fi.exception.EmailSendException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.mail.*;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import java.io.UnsupportedEncodingException;
import java.security.Security;
import java.util.Date;
import java.util.List;
import java.util.Properties;


/**
 * Used for sending email.
 * <p/>
 * These methods send email to only one recipient.
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 3.11.11
 */
@Service("emailSender")
public class EmailSender {

    @Value("#{evalinProps['mail.host']}")
    private String host = "default";

    @Value("#{evalinProps['mail.port']}")
    private String port = "465";

    @Value("#{evalinProps['mail.user']}")
    private String user = "default";

    @Value("#{evalinProps['mail.pass']}")
    private String pass = "default";


    // Get system properties
    private Properties properties = new Properties();

    protected final Log logger = LogFactory.getLog(getClass());

    private Session initializeSession() {
        logger.debug("Going to initialize mail session");
        Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());

        // Setup mail server
        properties.setProperty("mail.smtps.host", host);
        properties.setProperty("mail.smtps.port", port);

        properties.put("mail.smtps.socketFactory.port", port);
        properties.put("mail.smtps.socketFactory.class",
                "javax.net.ssl.SSLSocketFactory");
        properties.put("mail.smtps.socketFactory.fallback", "false");
        properties.setProperty("mail.transport.protocol", "smtps");
        properties.put("mail.smtps.ssl.protocols", "SSLv3");

        // Get the default Session object.
        Session session = Session.getDefaultInstance(properties,
                new javax.mail.Authenticator() {
                    protected PasswordAuthentication getPasswordAuthentication() {
                        return new PasswordAuthentication(user, pass);
                    }
                });
        logger.debug("Mail session initialized.");
        //  session.setDebug(true);
        return session;

    }

    /**
     * Sends simple (text/plain) email over SSL.
     *
     * @param email e-mail to send
     * @throws MessagingException when some problem happens
     */
    public void sendSimpleMail(Email email) throws EmailSendException {
        checkFieldsNotNull(email);
        logger.debug("Going to send plain text email");
        Session session = initializeSession();
        MimeMessage message = new MimeMessage(session);
        try {
            Transport tran = session.getTransport("smtps");
            tran.connect(user, pass);
            logger.debug("Connected to smtps server.");
            sendMessage(email, message, tran);
            logger.debug("Message sent successfully.");
            tran.close();
        } catch (UnsupportedEncodingException ex) {
            throw new EmailSendException(ex);
        } catch (MessagingException ex) {
            throw new EmailSendException(ex);
        }


    }

    public void sendSimpleMails(List<Email> emails) throws EmailSendException {
        for (Email email : emails) {
            checkFieldsNotNull(email);
        }
        logger.debug("Going to send plain text email");
        Session session = initializeSession();
        try {
            Transport tran = session.getTransport("smtps");
            tran.connect(user, pass);
            logger.debug("Connected to smtps server.");
            for (Email email : emails) {
                MimeMessage message = new MimeMessage(session);
                sendMessage(email, message, tran);
            }
            logger.debug("Message sent successfully.");
            tran.close();
        } catch (UnsupportedEncodingException ex) {
            throw new EmailSendException(ex);
        } catch (MessagingException ex) {
            throw new EmailSendException(ex);
        }


    }

    private void sendMessage(Email email, MimeMessage message, Transport tran) throws UnsupportedEncodingException, MessagingException {
        InternetAddress from = new InternetAddress(email.getFrom(), email.getFromName(), "utf-8");
        message.setFrom(from);
        message.setSubject(email.getSubject(), "utf-8");
        message.setText(email.getBody(), "utf-8");
        message.setRecipient(Message.RecipientType.TO, new InternetAddress(
                email.getTo()));
        tran.sendMessage(message, message.getAllRecipients());
    }

    private void checkFieldsNotNull(Email email) throws EmailSendException {
        if (email == null) {
            logger.debug("Email (object) is null");
            throw new EmailSendException("email (object) is null");
        }
        if (email.getFrom() == null) {
            logger.debug("From address is null");
            throw new EmailSendException("From address is null");
        }
        if (email.getTo() == null) {
            logger.debug("To address is null");
            throw new EmailSendException("To address is null");
        }
        if (email.getBody() == null) {
            logger.debug("Email body is null");
            throw new EmailSendException("Email body is null");
        }
    }

    @Deprecated
    public void sendHtmlEmail(Email email) throws MessagingException {
        logger.debug("Going to send html email.");
        Session session = initializeSession();
        MimeBodyPart messagePart = null;
        MimeMultipart multipart = null;
        MimeMessage message = new MimeMessage(session);
        message.setFrom(new InternetAddress(email.getFrom()));
        message.setSubject(email.getSubject(), "utf-8");

        messagePart = new MimeBodyPart();

        multipart = new MimeMultipart();
        multipart.addBodyPart(messagePart); // adding message part

        messagePart.setContent(email.getBody(), "text/html; charset=UTF-8");

        message.setContent(multipart);
        message.setSentDate(new Date());

        message.setRecipient(Message.RecipientType.TO, new InternetAddress(
                email.getTo()));

        Transport tran = session.getTransport("smtps");
        tran.connect(user, pass);
        logger.debug("Connected to smtps server.");
        tran.sendMessage(message, message.getAllRecipients());
        logger.debug("Message sent successfully.");
        tran.close();
    }

}
