package cz.muni.fi.mail;

import cz.muni.fi.domain.FormPair;
import cz.muni.fi.domain.Manager;
import cz.muni.fi.exception.EmailSendException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

/**
 * This class contains common logic for sending standalone mails and mass mails.
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 23.2.12
 */
@Service("mailCommons")
public class MailCommons {
    private static final String NEWLINE = System.getProperty("line.separator");

    private final EmailSender emailSender;

    @Autowired
    private ApplicationContext context;

    private final Log logger = LogFactory.getLog(getClass());

    @Autowired
    public MailCommons(EmailSender emailSender) {
        this.emailSender = emailSender;
    }

    public void sendMailAboutForm(FormPair pair, boolean manager, HttpServletRequest request, String myName, String myMail) throws EmailSendException {
        logger.debug("Going to send mail about pair " + pair);
        Email email = new Email(myMail);
        email.setFromName(myName);
        if (manager) {
            email.setTo(pair.getManagersForm().getOwner().getMail());
        } else {
            email.setTo(pair.getStudentsForm().getOwner().getMail());
        }
        logger.debug("Going to send email to: " + email.getTo() + " from: " + email.getFrom());
        email.setSubject(getMessage("mail.subject", (new String[]{
                manager ? pair.getStudentsForm().getOwner().getFullName()
                        : ((Manager) pair.getManagersForm().getOwner()).getOrganization().getName()}),
                request.getLocale()));

        StringBuffer url = request.getRequestURL();
        String uri = request.getRequestURI();
        int index = url.lastIndexOf(uri);
        String contextPath = request.getContextPath();
        url.delete(index, url.length());
        url.append(contextPath);
        url.append("/public/forms/");
        email.setBody(createEmailBody(url.toString(), pair, manager, request.getLocale(), myName).toString());
        emailSender.sendSimpleMail(email);
        logger.debug("Mail sent.");
    }

    public StringBuilder createEmailBody(String baseUrl, FormPair pair, boolean manager, Locale locale, String fromName) {
        StringBuilder sb = new StringBuilder();
        sb.append(getMessage("mail.key.hello", locale));
        sb.append(NEWLINE);
        sb.append(NEWLINE);
        if (manager) {
            sb.append(getMessage("mail.key.line1.student", new String[]{pair.getStudentsForm().getOwner().getFullName()}, locale));
        } else {
            sb.append(getMessage("mail.key.line1.manager", new String[]{
                    pair.getManagersForm().getOwner().getFullName(),
                    ((Manager) pair.getManagersForm().getOwner()).getOrganization().getName()
            }, locale));
        }
        sb.append(NEWLINE);
        sb.append(getMessage("mail.key.line2", locale));
        sb.append(NEWLINE);
        sb.append(getMessage("mail.key.line3", locale));
        sb.append(NEWLINE);
        sb.append(getMessage("mail.key.line4", locale));
        sb.append(NEWLINE);
        sb.append(NEWLINE);
        sb.append(getMessage("mail.key.line6", locale));
        sb.append(NEWLINE);
        sb.append(baseUrl);
        if (manager) {
            sb.append(pair.getManagersForm().getHash());
        } else {
            sb.append(pair.getStudentsForm().getHash());
        }
        sb.append(NEWLINE);
        sb.append(NEWLINE);

        sb.append(getMessage("mail.key.thanks", locale));
        sb.append(NEWLINE);
        sb.append(NEWLINE);

        sb.append(fromName);
        sb.append(NEWLINE);
        sb.append(getMessage("mail.key.signature", locale));
        return sb;
    }

    /**
     * Translates message from resource bundle
     *
     * @param key    message key
     * @param vals   values to fill in {}
     * @param locale in which locale the message should be
     * @return localized message
     */
    public String getMessage(String key, String[] vals, Locale locale) {
        return context.getMessage(key, vals, locale);
    }

    /**
     * Translates message without variables from resource bundle.
     *
     * @param key    message key
     * @param locale in which locale the message should be
     * @return localized message
     */
    public String getMessage(String key, Locale locale) {
        return getMessage(key, null, locale);
    }

    public void sendMailsAboutForm(List<FormPair> formPairs, HttpServletRequest request, String myName, String myMail) throws EmailSendException {
        logger.debug("Going to send mails about pairs :" + formPairs);
        List<Email> emails = new ArrayList<Email>();
        for (FormPair pair : formPairs) {
            Email emailForStudent = new Email(myMail);
            Email emailForManager = new Email(myMail);
            emailForManager.setFromName(myName);
            emailForStudent.setFromName(myName);
            emailForManager.setTo(pair.getManagersForm().getOwner().getMail());
            emailForStudent.setTo(pair.getStudentsForm().getOwner().getMail());
            logger.debug("Mail prepared to: " + emailForManager.getTo() + " from: " + emailForManager.getFrom());
            logger.debug("Mail prepared to: " + emailForStudent.getTo() + " from: " + emailForStudent.getFrom());

            emailForManager.setSubject(getMessage("mail.subject", (new String[]{
                    pair.getStudentsForm().getOwner().getFullName()}), request.getLocale()));
            emailForStudent.setSubject(getMessage("mail.subject", (new String[]{(
                    (Manager) pair.getManagersForm().getOwner()).getOrganization().getName()}), request.getLocale()));
            StringBuffer url = request.getRequestURL();
            String uri = request.getRequestURI();
            int index = url.lastIndexOf(uri);
            String contextPath = request.getContextPath();
            url.delete(index, url.length());
            url.append(contextPath);
            url.append("/public/forms/");
            emailForManager.setBody(createEmailBody(url.toString(), pair, true, request.getLocale(), myName).toString());
            emailForStudent.setBody(createEmailBody(url.toString(), pair, false, request.getLocale(), myName).toString());
            emails.add(emailForManager);
            emails.add(emailForStudent);
        }
        emailSender.sendSimpleMails(emails);
        logger.debug("Mails were sent.");
    }
}