package cz.muni.fi.service;

import cz.muni.fi.dao.*;
import cz.muni.fi.domain.*;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

/**
 * Manages features.
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 9.8.11
 */
@Service("featureManager")
public class FeatureManagerImpl implements FeatureManager {

    @Autowired
    private FeatureDao featureDao;

    @Autowired
    private FeatureRateDao featureRateDao;

    @Autowired
    private PeriodDao periodDao;

    @Autowired
    private OrganizationDao organizationDao;

    @Autowired
    private FormDao formDao;

    @Autowired
    private DisagreedFeatureRateDao disagreedFeatureRateDao;

    protected final Log logger = LogFactory.getLog(getClass());

    @Override
    @PreAuthorize("hasRole('coordinator')")
    public void addFeature(Feature feature) {
        if (feature == null) {
            throw new IllegalArgumentException("feature");
        }
        //if having parent it must be present
        if (feature.getParent() != null) {
            if (getFeatureById(feature.getParent().getId()) == null) {
                throw new IllegalArgumentException("feature parent not known");
            }
        }
        //no subfeatures must be present
        if (feature.getSubFeatures() != null && !feature.getSubFeatures().isEmpty()) {
            throw new IllegalArgumentException("can't add feature with some subfeatures");
        }

        featureDao.save(feature);
        if (feature.getParent() != null) {
            Feature parent = getFeatureById(feature.getParent().getId());
            parent.getSubFeatures().add(feature);
            featureDao.update(parent);
        }
        for (Period period : periodDao.getAllPeriods()) {
            for (Organization org : organizationDao.getAllOrganizations()) {
                FeatureRate rate = new FeatureRate();
                rate.setPeriod(period);
                rate.setSum(0);
                rate.setResponses(0);
                rate.setFeature(feature);
                rate.setOrganization(org);
                featureRateDao.save(rate);
            }
        }

    }

    @Override
    public Feature getFeatureById(Long id) {
        if (id == null) {
            throw new IllegalArgumentException("id");
        }
        return featureDao.findById(id);
    }

    @Override
    public List<Feature> getAllFeatures() {
        return featureDao.getAllFeatures();
    }

    @Override
    public List<Feature> getAllRootFeatures() {
        return featureDao.getAllRootFetures();
    }

    @Override
    public void updateFeature(Feature feature) {
        if (feature == null) {
            throw new IllegalArgumentException("feature");
        }
        if (feature.getId() == null) {
            throw new IllegalArgumentException("feature id");
        }
        Feature found = featureDao.findById(feature.getId());
        if (found == null) {
            throw new IllegalArgumentException("found feature is null");
        }

        if (feature.equals(feature.getParent())) {
            throw new IllegalArgumentException("can't set parent to itself");
        }
        if (notSame(found.getParent(), (feature.getParent()))) {
            Feature parent = found.getParent();
            Feature newParent = feature.getParent();
            if (parent != null) {
                parent.getSubFeatures().remove(feature);
                featureDao.update(parent);
            }
            if (newParent != null) {
                if (newParent.getSubFeatures() == null) {
                    newParent.setSubFeatures(new ArrayList<Feature>());
                }
                newParent.getSubFeatures().add(feature);
                featureDao.update(newParent);
            }
        }
        featureDao.update(feature);
    }

    private boolean notSame(Feature feature1, Feature feature2) {
        if (feature1 == null) {
            if (feature2 != null) {
                return true;
            } else {
                return false;
            }
        }
        return !feature1.equals(feature2);
    }

    @Override
    public void deleteFeature(Feature feature) {
        if (feature == null) {
            throw new IllegalArgumentException("feature");
        }
        if (feature.getId() == null) {
            throw new IllegalArgumentException("feature id");
        }
        Feature found = featureDao.findById(feature.getId());
        if (found == null) {
            throw new IllegalArgumentException("feature not found");
        }
        List<Period> periods = periodDao.getAllPeriods();
        for (Period period : periods) {
            for (FeatureRate fr : featureRateDao.findByFeature(found, period)) {
                featureRateDao.delete(fr);
            }
        }
        if (found.getParent() != null) {
            Feature parent = getFeatureById(found.getParent().getId());
            if (parent.getSubFeatures() != null) {
                parent.getSubFeatures().remove(found);
            } else {
                throw new IllegalArgumentException("parent does not have child as subfeature");
            }
            featureDao.update(parent);
        }
        for (Feature f : found.getSubFeatures()) {
            deleteFeature(f);
        }
        featureDao.delete(found);
    }

    @Override
    public boolean canFeatureBeDeleted(Feature featureP) {
        if (featureP == null) {
            throw new IllegalArgumentException("Feature is null");
        }
        Feature feature = getFeatureById(featureP.getId());
        if (feature == null) {
            return false;
        }
        List<Period> periods = periodDao.getAllPeriods();
        for (Period period : periods) {
            for (Organization org : organizationDao.getAllOrganizations()) {
                FeatureRate rate = featureRateDao.findByFeature(feature, period, org);
                if (rate == null) {
                    return false;
                }
                if (rate.getResponses() > 0) {
                    return false;
                }
            }
        }
        List<Form> forms = formDao.getAllForms();
        for (Form form : forms) {
            for (FeatureRating rating : form.getFeatureRatingList()) {
                if (rating.getFeature().equals(feature)) {
                    return false;
                }
            }
        }
        if (feature.getSubFeatures() != null) {
            for (Feature f : feature.getSubFeatures()) {
                if (!canFeatureBeDeleted(f)) {
                    return false;
                }
            }
        }
        return true;
    }

    @Override
    public List<DisagreedFeatureRate> getDisagreementList(Feature feature) {
        if (feature == null) {
            logger.info("Feature is null");
            throw new IllegalArgumentException("feature");
        }
        if (feature.getId() == null) {
            logger.info("feature id is null");
            throw new IllegalArgumentException("feature id");
        }
        if (featureDao.findById(feature.getId()) == null) {
            logger.info("feature not found");
            throw new IllegalArgumentException("feature not found");
        }
        return disagreedFeatureRateDao.getAllDisagreedFeatureRates(feature);
    }

}
