package cz.muni.fi.service;

import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.FeatureRate;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Period;

import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 15.8.11
 */
public interface FeatureRateManager {

    /**
     * Finds rate of given feature.
     * Aggregates result among all organizations.
     *
     * @param feature feature
     * @param period  period to get the result in
     * @return its rate
     */
    public FeatureRate getRateForFeature(Feature feature, Period period);

    /**
     * Finds rate of given feature.
     *
     * @param feature      feature
     * @param period       period to get the result in
     * @param organization organization to get the result for
     * @return its rate
     */
    public FeatureRate getRateForFeature(Feature feature, Period period, Organization organization);


    /**
     * Finds rate of given feature. (Global, for all periods and all organizations)
     * Aggregates result.
     *
     * @param feature feature
     * @return its rate
     */
    public FeatureRate getRateForFeature(Feature feature);

    /**
     * Finds rate of given feature. (Global, for all periods)
     * Aggregates result.
     *
     * @param feature      feature
     * @param organization organization to get the rate for
     * @return its rate
     */
    public FeatureRate getRateForFeature(Feature feature, Organization organization);


    /**
     * @return average rate from all features (global among periods and organizations)
     */
    public int getAverage();

    /**
     * Gets glogal average for specified organization.  (global among periods)
     *
     * @param organization organization to count the average for
     * @return global averagere for org.
     */
    public int getAverage(Organization organization);

    /**
     * Count average of all features¨for given period. (global among organizations)
     *
     * @param period given period
     * @return average
     */
    public int getAverage(Period period);

    /**
     * Count average of all features.
     *
     * @param period       given period
     * @param organization organiztaion to get the average for
     * @return average
     */
    public int getAverage(Period period, Organization organization);

    /**
     * @param negative if count negative values - e. g. improvement
     * @return sum of differences from average of the features which are lower then the average
     */
    public int getTotalImprovementCount(boolean negative);

    /**
     * @param negative     if count negative values - e. g. improvement
     * @param organization organization to get the improvement count for
     * @return sum of differences from average of the features which are lower then the average
     */
    public int getTotalImprovementCount(boolean negative, Organization organization);

    /**
     * @param negative if count negative values - e. g. improvement
     * @param period   given period
     * @return sum of differences from average of the features which are lower then the average
     */
    public int getTotalImprovementCount(boolean negative, Period period);

    /**
     * @param negative     if count negative values - e. g. improvement
     * @param period       given period
     * @param organization organization to count for
     * @return sum of differences from average of the features which are lower then the average
     */
    public int getTotalImprovementCount(boolean negative, Period period, Organization organization);

    /**
     * Aggregates feature rates by organizations for given period.
     *
     * @param period period of which needed the rates
     * @return all features rates
     */
    public List<FeatureRate> getAggregatedFeatureRates(Period period);


    /**
     * @return global (agregated) feature rates
     */
    public List<FeatureRate> getAggregatedFeatureRates();

    /**
     * Aggregates feature rates by periods for given organization.
     *
     * @param organization organization to count for
     * @return global (agregated) feature rates
     */
    public List<FeatureRate> getAggregatedFeatureRates(Organization organization);

    /**
     * @param organization organiztaon to get the rates for
     * @param period       period of which needed the rates
     * @return all features rates
     */
    public List<FeatureRate> getAllFeatureRates(Period period, Organization organization);


}
