package cz.muni.fi.service;

import cz.muni.fi.dao.*;
import cz.muni.fi.domain.*;
import cz.muni.fi.exception.DataIntegrityViolationException;
import cz.muni.fi.util.HashGenerator;
import cz.muni.fi.util.ThresholdProvider;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Manages forms and form pairs.
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 9.8.11
 */
@Service("formManager")
public class FormManagerImpl implements FormManager {

    protected final Log logger = LogFactory.getLog(getClass());

    /**
     * Form timeout. How long can be form reopened after once opened.
     * Amount is in seconds.
     */
    public static final int FORM_TIMEOUT_SECONDS = 3600;

    @Autowired
    private FormDao formDao;

    @Autowired
    private FeatureRatingDao featureRatingDao;

    @Autowired
    private FormPairDao formPairDao;

    @Autowired
    private FeatureRateDao featureRateDao;

    @Autowired
    private ParticipantDao participantDao;

    @Autowired
    private FeatureDao featureDao;

    @Autowired
    private PeriodDao periodDao;

    @Autowired
    private DisagreedFeatureRateDao disagreedFeatureRateDao;

    @Autowired
    private ThresholdProvider thresholdProvider;

    @Override
    public FormPair createNewPair(Participant student, Participant manager, List<Feature> features, Period period) {
        if (student == null) {
            logger.info("student is null");
            throw new IllegalArgumentException("student is null");
        }
        if (manager == null) {
            logger.info("manager is null");
            throw new IllegalArgumentException("manager is null");
        }
        if (features == null) {
            logger.info("features is null");
            throw new IllegalArgumentException("features is null");
        }

        if (period == null) {
            logger.info("period is null");
            throw new IllegalArgumentException("period is null");
        }

        if (period.getId() == null) {
            logger.info("period id is null");
            throw new IllegalArgumentException("period id is null");
        }
        Period p = periodDao.getById(period.getId());
        if (p == null) {
            logger.info("period not found");
            throw new IllegalArgumentException("period not found");
        }
        Participant part = participantDao.findByEmail(student.getMail());
        if (part == null) {
            logger.info("student not found");
            throw new IllegalArgumentException("student not found");
        }
        part = participantDao.findByEmail(manager.getMail());
        if (part == null) {
            logger.info("manager not found");
            throw new IllegalArgumentException("manager not found");
        }
        if (features.isEmpty()) {
            logger.info("Features list is empty.");
            throw new IllegalArgumentException("features list is empty");
        }
        FormPair pair = new FormPair();
        Form studentForm = new Form();
        Form managerForm = new Form();
        studentForm.setOwner(student);
        studentForm.setState("new");
        studentForm.setTimestamp(new Date());
        studentForm.setPeriod(period);
        managerForm.setState("new");
        managerForm.setTimestamp(new Date());
        managerForm.setOwner(manager);
        managerForm.setPeriod(period);
        pair.setState("new");
        formDao.save(studentForm);
        formDao.save(managerForm);
        studentForm.setHash(HashGenerator.computeHash(studentForm.getId()));
        managerForm.setHash(HashGenerator.computeHash(managerForm.getId()));
        List<FeatureRating> frlist = new ArrayList<FeatureRating>();
        List<FeatureRating> frlist2 = new ArrayList<FeatureRating>();
        int i = 0;
        for (Feature feature : features) {
            Feature found = featureDao.findById(feature.getId());
            if (found == null) {
                logger.info("feature not found");
                throw new IllegalArgumentException("feature not found");
            }
            FeatureRating fr = new FeatureRating();
            fr.setFeature(feature);
            fr.setValue(0);
            fr.setNumber(i);
            featureRatingDao.save(fr);
            frlist.add(fr);
            FeatureRating fr2 = new FeatureRating();
            fr2.setFeature(feature);
            fr2.setValue(0);
            fr2.setNumber(i);
            featureRatingDao.save(fr2);
            frlist2.add(fr2);
            i++;
        }
        studentForm.setFeatureRatingList(frlist);
        managerForm.setFeatureRatingList(frlist2);
        formDao.update(studentForm);
        formDao.update(managerForm);
        pair.setManagersForm(managerForm);
        pair.setStudentsForm(studentForm);
        formPairDao.save(pair);
        return pair;


    }

    @Override
    public void saveForm(Form form) {
        if (form == null) {
            throw new IllegalArgumentException("form");
        }
        formDao.update(form);
        FormPair pair = formPairDao.findByForm(form);
        Form otherForm = form.getOwner().getRole().equals("student") ? pair.getManagersForm() : pair.getStudentsForm();
        if ("new".equals(pair.getState())) {
            pair.setState(form.getOwner().getRole());
            form.setState("saved");
            form.setTimestamp(new Date());
            formDao.update(form);
        } else if ("student".equals(pair.getState()) || "manager".equals(pair.getState())) {
            for (FeatureRating rating : form.getFeatureRatingList()) {
                for (FeatureRating rating2 : otherForm.getFeatureRatingList()) {
                    if (rating.getFeature().equals(rating2.getFeature())) {
                        if (compareRatings(rating.getValue(), rating2.getValue())) {
                            FeatureRate rate = featureRateDao.findByFeature(rating.getFeature(), form.getPeriod(), ((Manager) pair.getManagersForm().getOwner()).getOrganization());
                            rate.setSum(rate.getSum() + ((rating.getValue() + rating2.getValue())*5));
                            rate.setResponses(rate.getResponses() + 1);
                            featureRateDao.update(rate);
                        } else {
                            DisagreedFeatureRate disagreedFeatureRate = new DisagreedFeatureRate();
                            disagreedFeatureRate.setFeature(rating.getFeature());
                            disagreedFeatureRate.setManager(pair.getManagersForm().getOwner());
                            disagreedFeatureRate.setStudent(pair.getStudentsForm().getOwner());
                            disagreedFeatureRateDao.save(disagreedFeatureRate);
                            logger.info("Ratings are different, ignoring.");
                        }
                    }
                }
            }
            form.setState("saved");
            form.setTimestamp(new Date());
            formDao.update(form);
            pair.setState("completed");
        } else {
            logger.error("Invalid form state.");
            throw new IllegalArgumentException("Invalid form state.");
        }
        formPairDao.update(pair);
    }

    @Override
    public FormPair findFormPair(Form form) {
        if (form == null) {
            throw new IllegalArgumentException("form");
        }
        return formPairDao.findByForm(form);
    }

    @Override
    public Form getFormByHash(String hash) {
        if (hash == null) {
            throw new IllegalArgumentException("hash");
        }
        return formDao.findByHash(hash);
    }

    @Override
    public List<FormPair> getAllFormPairs() {
        return formPairDao.getAllFormPairs();
    }

    @Override
    public List<FormPair> getAllFormPairs(Period period) {
        if (period == null) {
            throw new IllegalArgumentException("period");
        }
        return formPairDao.getAllFormPairs(period);
    }

    @Override
    public void deleteFormPair(FormPair pair) throws DataIntegrityViolationException {
        logger.debug("Going to delete form pair " + pair);
        if (pair == null) {
            logger.info("Pair is null, can't be deleted");
            throw new IllegalArgumentException("pair");
        }
        FormPair found = getFormPairById(pair.getId());
        if (!found.getState().equals("new")) {
            logger.warn("Pair is not new, can't be deleted");
            throw new DataIntegrityViolationException("Form is not new, can't be deleted");
        }
        if (!found.getManagersForm().getState().equals("new") || !found.getStudentsForm().getState().equals("new")) {
            logger.warn("Some of the forms in pair is not new, can't be deleted");
            throw new DataIntegrityViolationException("Form is not new, can't be deleted");
        }
        formPairDao.delete(pair);
        logger.debug("Successfully deleted");
    }

    @Override
    public void forceDeleteFormPair(FormPair pair) {
        logger.debug("Going to force delete form pair " + pair);
        if (pair == null) {
            logger.info("Pair is null, can't be deleted");
            throw new IllegalArgumentException("pair");
        }
        formPairDao.delete(pair);
        logger.debug("Successfully deleted");
    }

    @Override
    public FormPair getFormPairById(Long id) {
        return formPairDao.findById(id);
    }

    @Override
    public boolean canBeAnswered(Form formParam) throws DataIntegrityViolationException {
        Form form = formDao.findById(formParam.getId());
        if ("new".equals(form.getState())) {
            return true;
        }
        if ("opened".equals(form.getState())) {
            if (form.getTimestamp() == null) {
                throw new DataIntegrityViolationException("form with state opened doesnt have the open time set");
            }
            if ((System.currentTimeMillis() - form.getTimestamp().getTime()) > (FORM_TIMEOUT_SECONDS * 1000)) {
                logger.warn("Form has expired, can't be answered.");
                return false;
            } else {
                return true;
            }
        }
        return false;
    }

    @Override
    public void openForm(Form formParam) {
        logger.debug("Going to open form.");
        if (formParam == null) {
            logger.info("Form is null, aborting");
            throw new IllegalArgumentException("Form is null");
        }
        Form form = formDao.findById(formParam.getId());
        if (!"new".equals(form.getState())) {
            logger.info("Can't open form that can't be opened");
            throw new IllegalArgumentException("Cant open form that can't be opened");
        }
        form.setState("opened");
        form.setTimestamp(new Date());
        formDao.update(form);
    }

    @Override
    public void enableFormReopen(Form form) {
        logger.debug("Going to reopen form " + form);
        if (form == null) {
            logger.info("Form is null, aborting");
            throw new IllegalArgumentException("Form is null");
        }
        Form found = formDao.findById(form.getId());
        if (!"opened".equals(found.getState())) {
            logger.info("Can't reopen form that can't be reopened");
            throw new IllegalArgumentException("Cant reopen form that can't be reopened");
        }
        found.setState("new");
        found.setTimestamp(new Date());
        formDao.update(found);
    }

    @Override
    public Form getFormById(Long formId) {
        return formDao.findById(formId);
    }

    @Override
    public void setFormPairTimestamp(FormPair pair) {
        FormPair other = formPairDao.findById(pair.getId());
        other.setMailTimestamp(new Date());
        formPairDao.update(other);
    }

    @Override
    public List<FormPair> getAllFormPairs(Period period, Organization organization) {
        if (period == null) {
            if (organization == null) {
                return formPairDao.getAllFormPairs();
            } else {
                return formPairDao.getAllFormPairs(organization);
            }
        }
        if (organization == null) {
            return formPairDao.getAllFormPairs(period);
        }
        return formPairDao.getAllFormPairs(period, organization);

    }

    private boolean compareRatings(int rate1, int rate2) {
        return Math.abs(rate1 - rate2) < thresholdProvider.getThreshold();
    }

}
