package cz.muni.fi.service;

import cz.muni.fi.dao.FeatureDao;
import cz.muni.fi.dao.FeatureRateDao;
import cz.muni.fi.dao.OrganizationDao;
import cz.muni.fi.dao.PeriodDao;
import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.FeatureRate;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Period;
import cz.muni.fi.exception.AlreadyPresentException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 12.8.11
 */
@Service("organizationManager")
public class OrganizationManagerImpl implements OrganizationManager {

    protected final Log logger = LogFactory.getLog(getClass());

    @Autowired
    private OrganizationDao organizationDao;

    @Autowired
    private FeatureDao featureDao;

    @Autowired
    private PeriodDao periodDao;

    @Autowired
    private FeatureRateDao featureRateDao;

    @Override
    public void addOrganization(Organization organization) throws AlreadyPresentException {
        if (organization == null) {
            logger.info("organization is null, aborting.");
            throw new IllegalArgumentException("organization is null");
        }
        if (organization.getName() == null || organization.getName().isEmpty()) {
            logger.info("organization name is null or empty, aborting.");
            throw new IllegalArgumentException("organization name is null or empty");
        }
        if (organization.getIdNumber() == null) {
            logger.info("organization id number is null, aborting.");
            throw new IllegalArgumentException("organization id number is null");
        }
        try {
            organizationDao.save(organization);
        } catch (DataIntegrityViolationException ex) {
            logger.warn("organization " + organization + " already in db");
            throw new AlreadyPresentException("organization already in db", ex);
        }
        for (Period period : periodDao.getAllPeriods()) {
            for (Feature feature : featureDao.getAllFeatures()) {
                FeatureRate fr = new FeatureRate();
                fr.setFeature(feature);
                fr.setOrganization(organization);
                fr.setPeriod(period);
                featureRateDao.save(fr);
            }
        }
    }

    @Override
    public void deleteOrganization(Organization org) {
        if (org == null) {
            logger.info("null organiization, aborting");
            throw new IllegalArgumentException("organization");
        }
        if (!canBeDeleted(org)) {
            logger.warn("Organization can't be deleted.");
            throw new IllegalArgumentException("organization has some rating associated with it, it can't be deleted");
        }
        List<FeatureRate> rates = featureRateDao.getAllFeatureRates(org);
        for (FeatureRate rate : rates) {
            featureRateDao.delete(rate);
        }
        organizationDao.delete(org);
    }

    @Override
    public List<Organization> getAllActiveOrganizations() {
        return organizationDao.getAllActiveOrganizations();
    }


    @Override
    public List<Organization> getAllOrganizations() {
        return organizationDao.getAllOrganizations();
    }

    @Override
    public Organization getById(Long id) {
        if (id == null) {
            logger.info("Id is null, aborting");
            throw new IllegalArgumentException("id is null");
        }
        return organizationDao.findById(id);
    }

    @Override
    public void updateOrganization(Organization organization) throws AlreadyPresentException {
        if (organization == null) {
            logger.info("Organization id is null, can't be updated.");
            throw new IllegalArgumentException("id is null");
        }
        Organization other = getById(organization.getId());
        if (other == null) {
            logger.info("Organization not found, can't be updated.");
            throw new IllegalArgumentException("organization not found");
        }
        try {
            organizationDao.update(organization);
        } catch (DataIntegrityViolationException ex) {
            logger.warn("organization " + organization + " already in db");
            throw new AlreadyPresentException("organization already in db", ex);
        }
    }

    @Override
    public boolean canBeDeleted(Organization organization) {
        logger.debug("Checking whether organization can be deleted " + organization);
        if (organization == null) {
            logger.info("Organization is null, can't be deleted.");
            return false;
        }
        if (organization.getId() == null) {
            logger.info("Organizations id is null, can't be deleted");
            return false;
        }
        Organization found = organizationDao.findById(organization.getId());
        if (found == null) {
            logger.info("Organization not found, can't be deleted");
            return false;
        }
        for (Feature feature : featureDao.getAllFeatures()) {
            for (Period period : periodDao.getAllPeriods()) {
                FeatureRate fr = featureRateDao.findByFeature(feature, period, organization);
                if ((fr != null) && (fr.getResponses() > 0)) {
                    logger.debug("Organization cannot be deleted, it is used.");
                    return false;
                }
            }
        }
        logger.debug("Organization can be deleted.");
        return true;
    }

    @Override
    public Organization getByIdNumber(String s) {
        if (s == null) {
            logger.info("organization id number id null");
            throw new IllegalArgumentException("id number");
        }
        return organizationDao.findByIdNumber(s);
    }
}