package cz.muni.fi.service;

import cz.muni.fi.dao.DisagreedFeatureRateDao;
import cz.muni.fi.dao.OrganizationDao;
import cz.muni.fi.dao.ParticipantDao;
import cz.muni.fi.domain.*;
import cz.muni.fi.exception.AlreadyPresentException;
import cz.muni.fi.exception.DataIntegrityViolationException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 9.8.11
 */
@Service("participantManger")
public class ParticipantManagerImpl implements ParticipantManager {

    protected final Log logger = LogFactory.getLog(getClass());

    @Autowired
    private ParticipantDao participantDao;

    @Autowired
    private OrganizationDao organizationDao;

    @Autowired
    private DisagreedFeatureRateDao disagreedFeatureRateDao;

    @Override
    public List<Participant> getAllStudents() {
        logger.debug("Getting all students.");
        return participantDao.getAllStudents();
    }

    @Override
    public List<Participant> getAllManagers() {
        logger.debug("Getting all managers.");
        return participantDao.getAllManagers();
    }

    @Override
    public Participant getByEmail(String email) {
        if (email == null) {
            logger.info("Email is null.");
            throw new IllegalArgumentException("email");
        }
        return participantDao.findByEmail(email);
    }

    @Override
    public Participant getById(Long id) {
        logger.debug("Getting participant by id " + id);
        if (id == null) {
            logger.info("Participant id is null, aborting.");
            throw new IllegalArgumentException("id");
        }
        return participantDao.findById(id);
    }

    @Override
    public List<Participant> getAllManagersFor(Organization org) {
        if (org == null) {
            logger.info("Organization is null");
            throw new IllegalArgumentException("organization");
        }
        return participantDao.getAllManagersFor(org);
    }

    @Override
    public void addParticipant(Participant participant) throws AlreadyPresentException {
        checkParticipantConstraints(participant);
        try {
            participantDao.save(participant);
        } catch (org.springframework.dao.DataIntegrityViolationException ex) {
            logger.warn("participant " + participant + " already in db");
            throw new AlreadyPresentException("participant already in db", ex);
        }
    }

    @Override
    public void deleteParticipant(Participant participant) throws DataIntegrityViolationException {
        if (participant == null) {
            logger.info("null participant, aborting.");
            throw new IllegalArgumentException("participant");
        }
        if (participantDao.canBeDeleted(participant)) {
            participantDao.delete(participant);
        } else {
            logger.info("Participant has some forms, he can't be deleted.");
            throw new DataIntegrityViolationException("participant can't be deleted");
        }
    }

    @Override
    public void updateParticipant(Participant participant) throws AlreadyPresentException {
        checkParticipantConstraints(participant);
        Participant found = getById(participant.getId());
        if (found == null) {
            logger.info("Participant " + participant + "not found");
            throw new IllegalArgumentException("participant not found");
        }
        try {
            participantDao.update(participant);
        } catch (org.springframework.dao.DataIntegrityViolationException ex) {
            logger.warn("participant " + participant + " already in db");
            throw new AlreadyPresentException("participant already in db", ex);
        }

    }

    @Override
    public Participant getByUco(String studentUco) {
        if (studentUco == null) {
            logger.info("uco is null");
            throw new IllegalArgumentException("uco");
        }
        return participantDao.findByUco(studentUco);
    }

    private void checkParticipantConstraints(Participant participant) {
        if (participant == null) {
            logger.info("Participant is null.");
            throw new IllegalArgumentException("participant");
        }
        if (participant.getMail() == null || participant.getMail().isEmpty()) {
            logger.info("Participant mail is null or empty, aborting.");
            throw new IllegalArgumentException("participant - mail");
        }
        if (participant.getName() == null || participant.getName().isEmpty()) {
            logger.info("Participant name is null or empty, aborting.");
            throw new IllegalArgumentException("participant - name");
        }
        if (participant.getRole() == null) {
            logger.info("Participant role must not be null, aborting.");
            throw new IllegalArgumentException("participant - role");
        }
        if (!participant.getRole().equals("manager") && !participant.getRole().equals("student")) {
            logger.info("Participant role must be student/manager, aborting.");
            throw new IllegalArgumentException("participant - role");
        }
        if (!participant.getMail().matches(
                "^[\\w\\-]+(\\.[\\w\\-]+)*@([A-Za-z0-9-]+\\.)+[A-Za-z]{2,4}$")) {
            logger.info("Participant mail is not a valid email, aborting.");
            throw new IllegalArgumentException("mail must be a valid email.");
        }
        if (participant.getRole().equals("manager")) {
            if (!(participant instanceof Manager)) {
                logger.error("Manager must be the object Manager.");
                throw new IllegalArgumentException("cannot cast Manager to Manager");
            }
            if (((Manager) participant).getOrganization() == null) {
                logger.info("Manager must have the organization set.");
                throw new IllegalArgumentException("manager - organization is null");
            }
            if (((Manager) participant).getOrganization().getId() == null) {
                logger.info("Manager organization with null id.");
                throw new IllegalArgumentException("manager - unknown organization, null id");
            }
            if (organizationDao.findById(((Manager) participant).getOrganization().getId()) == null) {
                logger.info("Manager organization is not known.");
                throw new IllegalArgumentException("manager - unknown organization");
            }
        }
        if (participant.getRole().equals("student")) {
            if (!(participant instanceof Student)) {
                logger.error("Student must be the object Student.");
                throw new IllegalArgumentException("cannot cast Student to Student");
            }
            if (((Student) participant).getUco() == null) {
                logger.info("UCO must not be null.");
                throw new IllegalArgumentException("uco");
            }
        }
    }

    @Override
    public boolean canParticipantBeDeleted(Participant participant) {
        if (participant == null) {
            logger.info("participant is null");
            throw new IllegalArgumentException("participant is null");
        }
        return participantDao.canBeDeleted(participant);
    }

    @Override
    public List<DisagreedFeatureRate> getDisagreementList(Participant participant) {
        if (participant == null) {
            logger.info("participant is null");
            throw new IllegalArgumentException("participant");
        }
        if (participant.getId() == null) {
            logger.info("participant id is null");
            throw new IllegalArgumentException("participant id");
        }
        if (participantDao.findById(participant.getId()) == null) {
            logger.info("participant not found");
            throw new IllegalArgumentException("participant not found");
        }
        return disagreedFeatureRateDao.getAllDisagreedFeatureRates(participant);

    }
}
