package cz.muni.fi.service;

import cz.muni.fi.dao.*;
import cz.muni.fi.domain.*;
import cz.muni.fi.exception.DataIntegrityViolationException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 21.11.11
 */
@Service("periodManager")
public class PeriodManagerImpl implements PeriodManager {

    protected final Log logger = LogFactory.getLog(getClass());

    @Autowired
    private PeriodDao periodDao;

    @Autowired
    private FeatureDao featureDao;

    @Autowired
    private FeatureRateDao featureRateDao;

    @Autowired
    private UserDao userDao;

    @Autowired
    private FormDao formDao;

    @Autowired
    private OrganizationDao organizationDao;

    @Override
    public void addPeriod(Period period) {
        if (period == null) {
            throw new IllegalArgumentException("period");
        }
        period.setTimestamp(new Date());
        periodDao.save(period);
        List<Feature> features = featureDao.getAllFeatures();
        for (Feature feature : features) {
            for (Organization org : organizationDao.getAllOrganizations()) {
                FeatureRate rate = new FeatureRate();
                rate.setSum(0);
                rate.setPeriod(period);
                rate.setResponses(0);
                rate.setFeature(feature);
                rate.setOrganization(org);
                featureRateDao.save(rate);
            }
        }
    }

    @Override
    public Period updatePeriod(Period period) {
        if (period == null) {
            throw new IllegalArgumentException("period");
        }
        if (period.getId() == null) {
            throw new IllegalArgumentException("period id");
        }
        Period found = periodDao.getById(period.getId());
        if (found == null) {
            logger.info("cant update non present period " + period);
            throw new IllegalArgumentException("can't update non present period " + period);
        }
        return periodDao.update(period);
    }

    @Override
    public List<Period> getAllPeriods() {
        return periodDao.getAllPeriods();
    }

    @Override
    public Period getCurrentPeriod() throws DataIntegrityViolationException {
        return periodDao.getCurrrent();
    }

    @Override
    public void deletePeriod(Period period) {
        logger.debug("Going to delete period " + period);
        if (period == null) {
            logger.info("period is null");
            throw new IllegalArgumentException("period");
        }
        if (period.getId() == null) {
            logger.info("period id is null");
            throw new IllegalArgumentException("period id");
        }
        Period found = periodDao.getById(period.getId());
        if (found == null) {
            logger.info("period not found " + period);
            throw new IllegalArgumentException("period not found");
        }
        if (!canPeriodBeDeleted(period)) {
            logger.info("Period is used, can't be deleted.");
            throw new IllegalArgumentException("period is used");
        }

        List<FeatureRate> rates = featureRateDao.getAllFeatureRates(period);
        for (FeatureRate rate : rates) {
            featureRateDao.delete(rate);
        }
        List<User> usersUsing = userDao.getAllUsersWithSelectedPeriod(period);
        for (User user : usersUsing) {
            user.setSelectedPeriod(null);
            userDao.update(user);
        }
        periodDao.delete(period);
        logger.debug("Period successfully deleted.");
    }


    @Override
    public boolean canPeriodBeDeleted(Period period) {
        logger.debug("Checking whether period can be deleted " + period);
        if (period == null) {
            logger.info("Period is null.");
            throw new IllegalArgumentException("period");
        }
        if (period.getId() == null) {
            logger.info("Period id is null");
            throw new IllegalArgumentException("period id");
        }
        Period found = periodDao.getById(period.getId());
        if (found == null) {
            logger.info("Period not found, can't be deleted.");
            //non existent period can not be deleted
            return false;
        }
        boolean canBeDeleted = true;
        List<FeatureRate> rates = featureRateDao.getAllFeatureRates(period);
        for (FeatureRate rate : rates) {
            if (rate.getResponses() != 0) {
                logger.debug("Period has some rate associated with it: " + rate.getResponses());
                canBeDeleted = false;
            }
        }
        List<Form> forms = formDao.getAllForms(period);
        if (!forms.isEmpty()) {
            logger.debug("Period is in some forms.");
            canBeDeleted = false;
        }
        logger.debug("Result: " + canBeDeleted);
        return canBeDeleted;


    }

    @Override
    public int getNumberOfFormsForPeriod(Period period) {
        if (period == null) {
            logger.info("period is null");
            throw new IllegalArgumentException("period");
        }
        return formDao.getAllForms(period).size();
    }


    @Override
    public Period getPeriodById(Long id) {
        if (id == null) {
            logger.info("id is null");
            throw new IllegalArgumentException("id");
        }
        return periodDao.getById(id);
    }
}
