package cz.muni.fi.service;

import cz.muni.fi.dao.FeatureDao;
import cz.muni.fi.dao.FeatureRateDao;
import cz.muni.fi.dao.OrganizationDao;
import cz.muni.fi.dao.PeriodDao;
import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.FeatureRate;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Period;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 15.8.11
 */
@Service("plainFeatureRateManager")
public class PlainFeatureRateManagerImpl implements PlainFeatureRateManager {

    @Autowired
    private FeatureRateDao featureRateDao;

    @Autowired
    private PeriodDao periodDao;

    @Autowired
    private FeatureDao featureDao;

    @Autowired
    private OrganizationDao organizationDao;

    @Override
    public FeatureRate getRateForFeature(Feature feature) {
        if (feature == null) {
            throw new IllegalArgumentException("feature");
        }
        List<Period> allPeriods = periodDao.getAllPeriods();
        FeatureRate rate = new FeatureRate();
        rate.setFeature(feature);
        for (Period period : allPeriods) {

            List<FeatureRate> list = featureRateDao.findByFeature(feature, period);
            Map<Feature, FeatureRate> agg = aggregateByFeature(list);
            list.clear();
            for (Map.Entry<Feature, FeatureRate> pair : agg.entrySet()) {
                list.add(pair.getValue());
            }
            FeatureRate fr = null;
            if (list.size() != 1) {
                fr = null;
            } else {
                fr = list.get(0);
            }
            if (fr != null) {
                rate.setResponses(rate.getResponses() + fr.getResponses());
                rate.setSum(rate.getSum() + fr.getSum());
            }
        }
        return rate;
    }

    private Map<Organization, FeatureRate> aggregateByOrganization(List<FeatureRate> list) {
        Map<Organization, FeatureRate> agg = new HashMap<Organization, FeatureRate>();
        for (FeatureRate rate : list) {
            if (agg.get(rate.getOrganization()) == null) {
                agg.put(rate.getOrganization(), rate);
            } else {
                FeatureRate curr = agg.get(rate.getOrganization());
                curr.setResponses(curr.getResponses() + rate.getResponses());
                curr.setSum(curr.getSum() + rate.getSum());
                agg.put(rate.getOrganization(), curr);
            }
        }
        return agg;
    }

    private Map<Feature, FeatureRate> aggregateByFeature(List<FeatureRate> list) {
        Map<Feature, FeatureRate> agg = new HashMap<Feature, FeatureRate>();
        for (FeatureRate rate : list) {
            if (agg.get(rate.getFeature()) == null) {
                agg.put(rate.getFeature(), rate);
            } else {
                FeatureRate curr = agg.get(rate.getFeature());
                curr.setResponses(curr.getResponses() + rate.getResponses());
                curr.setSum(curr.getSum() + rate.getSum());
                agg.put(rate.getFeature(), curr);
            }
        }
        return agg;
    }

    private Map<Period, FeatureRate> aggregateByPeriod(List<FeatureRate> list) {
        Map<Period, FeatureRate> agg = new HashMap<Period, FeatureRate>();
        for (FeatureRate rate : list) {
            if (agg.get(rate.getPeriod()) == null) {
                agg.put(rate.getPeriod(), rate);
            } else {
                FeatureRate curr = agg.get(rate.getPeriod());
                curr.setResponses(curr.getResponses() + rate.getResponses());
                curr.setSum(curr.getSum() + rate.getSum());
                agg.put(rate.getPeriod(), curr);
            }
        }
        return agg;
    }

    @Override
    public FeatureRate getRateForFeature(Feature feature, Organization organization) {
        if (feature == null) {
            throw new IllegalArgumentException("feature");
        }
        if (organization == null) {
            throw new IllegalArgumentException("organization");
        }
        List<FeatureRate> rates = featureRateDao.findByFeature(feature, organization);
        FeatureRate fr = new FeatureRate();
        fr.setFeature(feature);
        fr.setOrganization(organization);
        if (rates.isEmpty()) {
            return null;
        }
        for (FeatureRate rate : rates) {
            fr.setResponses(fr.getResponses() + rate.getResponses());
            fr.setSum(fr.getSum() + rate.getSum());
        }
        return fr;
    }

    @Override
    public FeatureRate getRateForFeature(Feature feature, Period period) {
        if (feature == null) {
            throw new IllegalArgumentException("feature");
        }
        if (period == null) {
            throw new IllegalArgumentException("period");
        }
        List<FeatureRate> list = featureRateDao.findByFeature(feature, period);
        FeatureRate fr = new FeatureRate();
        fr.setFeature(feature);
        fr.setPeriod(period);
        if (list.isEmpty()) {
            return null;
        }
        for (FeatureRate rate : list) {
            fr.setResponses(fr.getResponses() + rate.getResponses());
            fr.setSum(fr.getSum() + rate.getSum());
        }
        return fr;

    }

    @Override
    public FeatureRate getRateForFeature(Feature feature, Period period, Organization organization) {
        if (feature == null) {
            throw new IllegalArgumentException("feature");
        }
        if (period == null) {
            throw new IllegalArgumentException("period");
        }
        if (organization == null) {
            throw new IllegalArgumentException("organization");
        }
        return featureRateDao.findByFeature(feature, period, organization);
    }

    @Override
    public int getAverage() {
        List<FeatureRate> allRates = getAggregatedFeatureRates();
        return countAverage(allRates);
    }

    /**
     * Counts average from rates.
     *
     * @param allRates rates to count average from
     * @return the average as int, zero if list was empty
     */
    public int countAverage(List<FeatureRate> allRates) {
        float sum = 0;
        int count = 0;
        for (FeatureRate rate : allRates) {
            //counting only answered stuff
            if (rate.getResponses() > 0) {
                count++;
                sum += rate.getValue();
            }
        }
        return count == 0 ? 0 : Math.round(sum / count);
    }

    @Override
    public int getAverage(Organization organization) {
        if (organization == null) {
            return getAverage();
        }
        List<FeatureRate> allRates = getAggregatedFeatureRates(organization);
        return countAverage(allRates);
    }

    @Override
    public int getAverage(Period period) {
        if (period == null) {
            return getAverage();
        }
        List<FeatureRate> allRates = getAggregatedFeatureRates(period);
        return countAverage(allRates);
    }

    @Override
    public int getAverage(Period period, Organization organization) {
        if (period == null) {
            return getAverage(organization);
        }
        if (organization == null) {
            return getAverage(period);
        }
        List<FeatureRate> allRates = getAllFeatureRates(period, organization);
        return countAverage(allRates);
    }

    @Override
    public int getTotalImprovementCount(boolean negative) {
        List<FeatureRate> allRates = getAggregatedFeatureRates();
        int average = getAverage();
        return countImprovement(negative, allRates, average);
    }

    public int countImprovement(boolean negative, List<FeatureRate> allRates, int average) {
        int sum = 0;
        for (FeatureRate rate : allRates) {
            //if negative, counting values less then average
            //if positive, counting values above average
            if (rate.getResponses() > 0) {
                if (negative == (rate.getValue() < average)) {
                    sum += Math.abs(rate.getValue() - average);
                }
            }
        }
        return sum;
    }

    @Override
    public int getTotalImprovementCount(boolean negative, Organization organization) {
        if (organization == null) {
            return getTotalImprovementCount(negative);
        }
        List<FeatureRate> allRates = getAggregatedFeatureRates(organization);
        int average = getAverage(organization);
        return countImprovement(negative, allRates, average);
    }

    @Override
    public List<FeatureRate> getAggregatedFeatureRates(Period period) {
        if (period == null) {
            throw new IllegalArgumentException("period");
        }
        if (period.getId() == null) {
            throw new IllegalArgumentException("period id");
        }
        Period found = periodDao.getById(period.getId());
        if (found == null) {
            throw new IllegalArgumentException("period not found");
        }
        List<FeatureRate> list = featureRateDao.getAllFeatureRates(period);
        Map<Feature, FeatureRate> map = aggregateByFeature(list);
        list.clear();
        for (Map.Entry<Feature, FeatureRate> pair : map.entrySet()) {
            list.add(pair.getValue());
        }
        return list;

    }

    @Override
    public List<FeatureRate> getAllFeatureRates(Period period, Organization organization) {
        if (period == null) {
            return getAggregatedFeatureRates(organization);
        }
        if (period.getId() == null) {
            throw new IllegalArgumentException("period id");
        }
        Period found = periodDao.getById(period.getId());
        if (found == null) {
            throw new IllegalArgumentException("period not found");
        }
        if (organization == null) {
            return getAggregatedFeatureRates(period);
        }
        return featureRateDao.getAllFeatureRates(period, organization);
    }

    @Override
    public List<FeatureRate> getAggregatedFeatureRates() {
        List<Feature> features = featureDao.getAllFeatures();
        List<FeatureRate> ret = new ArrayList<FeatureRate>();
        for (Feature feature : features) {
            FeatureRate rate = getRateForFeature(feature);
            ret.add(rate);
        }
        return ret;
    }

    @Override
    public List<FeatureRate> getAggregatedFeatureRates(Organization organization) {
        if (organization == null) {
            throw new IllegalArgumentException("organization");
        }
        if (organization.getId() == null) {
            throw new IllegalArgumentException("organization id");
        }
        Organization found = organizationDao.findById(organization.getId());
        if (found == null) {
            throw new IllegalArgumentException("organization not found");
        }
        List<FeatureRate> list = featureRateDao.getAllFeatureRates(organization);
        Map<Feature, FeatureRate> map = aggregateByFeature(list);
        list.clear();
        for (Map.Entry<Feature, FeatureRate> pair : map.entrySet()) {
            list.add(pair.getValue());
        }
        return list;
    }

    @Override
    public int getTotalImprovementCount(boolean negative, Period period) {
        if (period == null) {
            return getTotalImprovementCount(negative);
        }
        List<FeatureRate> allRates = getAggregatedFeatureRates(period);
        int average = getAverage(period);
        return countImprovement(negative, allRates, average);
    }

    @Override
    public int getTotalImprovementCount(boolean negative, Period period, Organization organization) {
        if (period == null) {
            return getTotalImprovementCount(negative, organization);
        }
        if (organization == null) {
            return getTotalImprovementCount(negative, period);
        }
        List<FeatureRate> allRates = getAllFeatureRates(period, organization);
        int average = getAverage(period, organization);
        return countImprovement(negative, allRates, average);
    }
}
