package cz.muni.fi.service;

import cz.muni.fi.dao.AuthorityDao;
import cz.muni.fi.dao.UserDao;
import cz.muni.fi.domain.Authority;
import cz.muni.fi.domain.Period;
import cz.muni.fi.domain.User;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 26.10.11
 */
@Service("userManager")
public class UserManagerImpl implements UserManager {

    protected final Log logger = LogFactory.getLog(getClass());

    @Autowired
    private UserDao userDao;

    @Autowired
    private AuthorityDao authorityDao;

    @Override
    public List<User> getAllUsers() {
        logger.debug("Getting all users.");
        List<User> list = userDao.getAllUsers();
        Collections.sort(list);
        return list;
    }

    @Override
    public User getById(String id) {
        if (id == null) {
            throw new IllegalArgumentException("id");
        }
        return userDao.findById(id);
    }

    @PreAuthorize("hasRole('admin')")
    public Authority getAuthorityById(Long id) {
        if (id == null) {
            throw new IllegalArgumentException("id");
        }
        return authorityDao.getAuthorityById(id);
    }

    @Override
    @PreAuthorize("hasRole('admin')")
    public void addUser(User user) {
        if (user == null) {
            logger.info("User is null, aborting.");
            throw new IllegalArgumentException("user");
        }
        if (user.getUserName() == null || user.getUserName().isEmpty()) {
            logger.info("Username is null or empty, aborting");
            throw new IllegalArgumentException("username null or empty");
        }
        if (user.getPassword() == null || user.getPassword().isEmpty()) {
            logger.info("Password is null or empty, aborting");
            throw new IllegalArgumentException("password null or empty");
        }

        userDao.save(user);
    }

    @Override
    @PreAuthorize("hasRole('admin')")
    public void deleteUser(User user) {
        if (user == null) {
            logger.info("User is null, aborting adding.");
            throw new IllegalArgumentException("user is null");
        }
        User found = userDao.findById(user.getUserName());
        if (found == null) {
            logger.info("User not found.");
            throw new IllegalArgumentException("user not found");
        }
        userDao.delete(user);
    }

    @Override
    @PreAuthorize("hasRole('admin')")
    public User updateUser(User user) {
        if (user == null) {
            logger.info("user is null");
            throw new IllegalArgumentException("user is null");
        }
        User found = userDao.findById(user.getUserName());
        if (found == null) {
            logger.info("user not found");
            throw new IllegalArgumentException("user not found");
        }
        return userDao.update(user);
    }

    @Override
    @PreAuthorize("hasRole('admin')")
    public User addUserAuthority(User user, Authority authority) {
        if (user == null) {
            logger.info("user is null");
            throw new IllegalArgumentException("user is null");
        }
        if (authority == null) {
            logger.info("authority is null");
            throw new IllegalArgumentException("authority is null");
        }
        if (!StringUtils.hasLength(authority.getRole())) {
            logger.info("role name must not be null or empty");
            throw new IllegalArgumentException("role name must not be null or empty");
        }
        User found = userDao.findById(user.getUserName());
        if (found == null) {
            logger.info("user not found");
            throw new IllegalArgumentException("user  not found");
        }
        for (Authority aut : found.getAuthorities()) {
            if (aut.getRole().equals(authority.getRole())) {
                logger.info("user already has this authority");
                throw new IllegalArgumentException("user already has this authority");
            }
        }
        found.getAuthorities().add(authority);
        return userDao.update(found);
    }

    @PreAuthorize("hasRole('admin')")
    public User removeUserAuthority(User user, Authority authority) {
        if (user == null) {
            logger.info("user is null");
            throw new IllegalArgumentException("user is null");
        }
        if (authority == null) {
            logger.info("authority is null");
            throw new IllegalArgumentException("authority is null");
        }
        User found = userDao.findById(user.getUserName());
        if (found == null) {
            logger.info("user not found");
            throw new IllegalArgumentException("user  not found");
        }
        if (!found.getAuthorities().contains(authority)) {
            logger.info("User does not have such authority.");
            throw new IllegalArgumentException("no such autority");
        }
        found.getAuthorities().remove(authority);
        User ret = userDao.update(found);
        authorityDao.delete(authority);
        return ret;
    }

    @Override
    public void changePassword(String username, String old, String newPass) {
        if (username == null) {
            throw new IllegalArgumentException("username is null");
        }
        if (old == null) {
            throw new IllegalArgumentException("old pwd is null");
        }
        if (newPass == null) {
            throw new IllegalArgumentException("new pwd is null");
        }
        User user = getById(username);
        if (user == null) {
            throw new IllegalArgumentException("no such user");
        }
        if (!user.getPassword().equals(old)) {
            throw new IllegalArgumentException("passwords do not match");
        }
        user.setPassword(newPass);
        updateUser(user);
    }

    @Override
    public void enable(User user) {
        User found = getById(user.getUserName());
        if (found.isEnabled()) {
            logger.info("User is already enabled.");
            throw new IllegalArgumentException("user is already enabled");
        }
        found.setEnabled(true);
        updateUser(found);
    }

    @Override
    public void disable(User user) {
        User found = getById(user.getUserName());
        if (!found.isEnabled()) {
            logger.info("User is already disabled.");
            throw new IllegalArgumentException("user is already disabled");
        }
        found.setEnabled(false);
        updateUser(found);
    }

    @Override
    public List<User> getAllUsersWithSelectedPeriod(Period period) {
        if (period == null) {
            throw new IllegalArgumentException("period");
        }
        return userDao.getAllUsersWithSelectedPeriod(period);
    }

    @Override
    public void changeSelectedPeriod(User user, Period period) {
        if (user == null) {
            logger.info("user is null");
            throw new IllegalArgumentException("user");
        }
        if (period == null) {
            logger.info("period is null");
            throw new IllegalArgumentException("period");
        }
        if (user.getUserName() == null) {
            logger.info("username is null");
            throw new IllegalArgumentException("username");
        }
        User found = getById(user.getUserName());
        if (found == null) {
            logger.info("User not found");
            throw new IllegalArgumentException("user not found");
        }
        found.setSelectedPeriod(period);
        updateUser(found);
    }

    public List<String> getAvailableRoles() {
        List<String> roles = new ArrayList<String>();
        roles.add("admin");
        roles.add("coordinator");
        return roles;
    }
}



