package cz.muni.fi.view;


import com.lowagie.text.Document;
import com.lowagie.text.Element;
import com.lowagie.text.PageSize;
import com.lowagie.text.Paragraph;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfTemplate;
import com.lowagie.text.pdf.PdfWriter;
import cz.muni.fi.util.FontMapperCP1250;
import cz.muni.fi.wrapper.FeatureRateWrapper;
import org.jfree.chart.ChartColor;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.Plot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.CategoryItemRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.general.DefaultPieDataset;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.web.servlet.view.document.AbstractPdfView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.text.DateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Map;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 19.10.11
 */
public class PdfReportView extends AbstractPdfView {

    private static final int ONE_PAGE_MAX = 6;
    @Autowired
    private ApplicationContext context;

    /**
     * Translates message from resource bundle
     *
     * @param key     message key
     * @param request servlet request (locale is taken from it)
     * @return localized message
     */
    private String getMessage(String key, HttpServletRequest request) {
        return context.getMessage(key, null, request.getLocale());
    }

    @Override
    protected void buildPdfDocument(Map modelSent, Document document,
                                    PdfWriter writer, HttpServletRequest request,
                                    HttpServletResponse response) throws Exception {

        Map<String, Object> model = (Map<String, Object>) modelSent.get("model");
        String year = (String) model.get("year");
        if (year == null) {
            year = getMessage("year.global", request);
        }
        String header = getMessage("rates", request) + " - " + year;
        Paragraph heading = new Paragraph(header, FontsProvider.titleFont());
        heading.setAlignment(Element.ALIGN_CENTER);
        document.add(heading);
        String org = ((model.get("organization") == null) ?
                getMessage("report.global", request) :
                (getMessage("report.organization", request) + " " + model.get("organization")));

        Paragraph subtitle = new Paragraph(org, FontsProvider.subTitleFont());
        subtitle.setAlignment(Element.ALIGN_CENTER);
        document.add(subtitle);
        String exported = getMessage("report.exported", request) + " " + getDateAsString(request.getLocale());

        Paragraph exp = new Paragraph(exported, FontsProvider.subTitleFont());
        exp.setAlignment(Element.ALIGN_CENTER);
        document.add(exp);


        float pageWidth = PageSize.A4.getWidth();
        float pageHeight = PageSize.A4.getHeight();
        float width = 0.7f * pageWidth;
        float height = 0.37f * pageHeight;
        float pieHeight = 0.8f * height;

        List<FeatureRateWrapper> barList = (List<FeatureRateWrapper>) model.get("totalRates");
        if (barList.size() > ONE_PAGE_MAX) {
            height = 0.75f * pageHeight;
            pieHeight = 0.8f * height;
        }
        String xAxis = getMessage("bars.x", request);
        String yAxis = getMessage("bars.y", request);
        String set = getMessage("bars.set", request);
        JFreeChart barChart = generateBarChart(barList, getMessage("rates.total", request), xAxis, yAxis, set);
        final List<FeatureRateWrapper> wrapperList = (List<FeatureRateWrapper>) model.get("rates");
        JFreeChart pieChart = generatePieChart(wrapperList, context.getMessage("rates.improvement", null, request.getLocale()));
        PdfContentByte contentByte = writer.getDirectContent();
        PdfTemplate templateBarChart = contentByte.createTemplate(width, height);
        PdfTemplate templatePieChart = contentByte.createTemplate(width, pieHeight);
        Graphics2D graphics2dBar = templateBarChart.createGraphics(width, height,
                new FontMapperCP1250());
        Graphics2D graphics2dPie = templatePieChart.createGraphics(width, pieHeight,
                new FontMapperCP1250());

        Rectangle2D rectangle2dBar = new Rectangle2D.Double(0, 0, width, height);
        Rectangle2D rectangle2dPie = new Rectangle2D.Double(0, 0, width, pieHeight);


        barChart.draw(graphics2dBar, rectangle2dBar);
        pieChart.draw(graphics2dPie, rectangle2dPie);
        graphics2dBar.dispose();
        graphics2dPie.dispose();

        float leftMargin = 0.15f * pageWidth;
        float bottomMargin = 0.07f * pageHeight;
        if (barList.size() <= ONE_PAGE_MAX) {
            contentByte.addTemplate(templateBarChart, leftMargin, pieHeight + 2 * bottomMargin);
        } else {
            contentByte.addTemplate(templateBarChart, leftMargin, bottomMargin);
            document.newPage();
            bottomMargin = 0.29f * pageHeight;
        }
        if (!wrapperList.isEmpty()) {
            contentByte.addTemplate(templatePieChart, leftMargin, bottomMargin);
        }
    }

    private String getDateAsString(Locale currentLocale) {
        DateFormat formatter = DateFormat.getDateInstance(DateFormat.DEFAULT,
                currentLocale);
        Date today = new Date();
        String output = formatter.format(today);
        return output;
    }

    /**
     * Generates bar chart.
     *
     * @param rates  list of rates shown on the chart
     * @param header chart header
     * @param set    chart set description
     * @param xAxis  chart x axis description
     * @param yAxis  chart y axis description
     * @return generated chart
     */
    public static JFreeChart generateBarChart(List<FeatureRateWrapper> rates, String header, String set, String xAxis, String yAxis) {
        Paint[] paints = ChartColor.createDefaultPaintArray();
        //create dataset
        DefaultCategoryDataset dataSet = new DefaultCategoryDataset();
        Integer i = 1;
        for (FeatureRateWrapper frw : rates) {
            dataSet.setValue(frw.getValue(), "set", i);
            i++;
        }

        //create chart
        JFreeChart chart = ChartFactory.createBarChart(
                header, null, null,
                dataSet, PlotOrientation.VERTICAL, false, false, false);


        final CategoryPlot plot = chart.getCategoryPlot();
        plot.setNoDataMessage("NO DATA!");
        plot.setBackgroundPaint(getBgColor());
        System.out.println(chart.getPadding());
        //set custom color bar renderer
        final CategoryItemRenderer renderer = new ColorBarRenderer(paints);
        plot.setRenderer(renderer);
        plot.setInsets((new RectangleInsets(15, 5, 5, 5)));

        // change the margin at the top of the range axis... (??)
        final ValueAxis rangeAxis = plot.getRangeAxis();
        rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
        rangeAxis.setLowerMargin(0.15);
        rangeAxis.setUpperMargin(0.15);
        //adjust upper bound
        rangeAxis.setUpperBound(100);
        //add legend
        LegendTitle legend = getLegend(rates, false);
        chart.addLegend(legend);

        return chart;
    }

    /**
     * Generates pie chart.
     *
     * @param wrapperList list of rates to show on chart
     * @param header      chart header
     * @return generated chart
     */
    public static JFreeChart generatePieChart(java.util.List<FeatureRateWrapper> wrapperList, String header) {
        //create dataset
        DefaultPieDataset dataSet = new DefaultPieDataset();
        Integer i = 1;
        for (FeatureRateWrapper frw : wrapperList) {
            dataSet.setValue(i, frw.getValue());
            i++;
        }

        //create chart
        JFreeChart chart = ChartFactory.createPieChart(
                header, dataSet, false, false, false);


        //adjust margins
        Plot plot = chart.getPlot();
        plot.setInsets((new RectangleInsets(15, 5, 5, 5)));
        plot.setBackgroundPaint(getBgColor());
        //add legend
        LegendTitle legend = getLegend(wrapperList,true);
        chart.addLegend(legend);

        return chart;
    }

    /**
     * Geneates legend for plots.
     *
     * @param featureRateList list of features to generate legend for, must be sorted
     * @return the legend
     */
    private static LegendTitle getLegend(List<FeatureRateWrapper> featureRateList,boolean hints) {
        LegendTitle legend = new FeatureChartLegendTitle(new FeatureChartLegendSource(featureRateList,hints));
        legend.setMargin(new RectangleInsets(1.0, 1.0, 1.0, 1.0));
        //  legend.setFrame(new LineBorder());
        legend.setBackgroundPaint(Color.white);
        legend.setPosition(RectangleEdge.BOTTOM);
        return legend;
    }

    /**
     * Gets background color for all charts.
     *
     * @return background color
     */
    private static Paint getBgColor() {
        //  return new Color(213, 213, 213);
        return new Color(242, 242, 242);

    }


}