package cz.muni.fi.web;

import au.com.bytecode.opencsv.CSVReader;
import cz.muni.fi.domain.*;
import cz.muni.fi.exception.AlreadyPresentException;
import cz.muni.fi.service.FeatureManager;
import cz.muni.fi.service.FormManager;
import cz.muni.fi.service.OrganizationManager;
import cz.muni.fi.service.ParticipantManager;
import cz.muni.fi.util.FeatureEditor;
import cz.muni.fi.util.FeatureParentComparator;
import cz.muni.fi.util.ParticipantEditor;
import cz.muni.fi.validation.FileWrapperValidator;
import cz.muni.fi.validation.FormPairFileWrapperValidator;
import cz.muni.fi.validation.FormPairValidator;
import cz.muni.fi.wrapper.FormPairFileWrapper;
import cz.muni.fi.wrapper.FormPairWrapper;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import cz.muni.fi.wrapper.UploadFileWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.multipart.support.ByteArrayMultipartFileEditor;
import org.springframework.web.multipart.support.StringMultipartFileEditor;

import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 3.3.12
 */

@Controller
@RequestMapping("/forms/bulk")
@SessionAttributes(types = FormPairFileWrapper.class)
public class BulkGeneratePairForm {
    private final FormManager formManager;

    private final ParticipantManager participantManager;

    private final FeatureManager featureManager;

    private final OrganizationManager organizationManager;

    protected final Log logger = LogFactory.getLog(getClass());


    @Autowired
    public BulkGeneratePairForm(FormManager manager, ParticipantManager participantManager, FeatureManager featureManager, OrganizationManager organizationManager) {
        this.formManager = manager;
        this.participantManager = participantManager;
        this.featureManager = featureManager;
        this.organizationManager = organizationManager;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        //  dataBinder.registerCustomEditor(Participant.class, new ParticipantEditor(participantManager));
        dataBinder.registerCustomEditor(Feature.class, new FeatureEditor(featureManager));
        //dataBinder.setBindEmptyMultipartFiles(false);
        //dataBinder.registerCustomEditor(String.class, new StringMultipartFileEditor());
    }

    @RequestMapping(method = RequestMethod.GET)
    public String setupForm(HttpSession session, Model model) {
        logger.debug("Rendering bulk generate pair form.");
        session.removeAttribute("sessionMessage");
        FormPairFileWrapper pairWrapper = new FormPairFileWrapper();
        pairWrapper.setFeatureList(featureManager.getAllFeatures());
        model.addAttribute("formPairFileWrapper", pairWrapper);
        return "forms/bulk";
    }

    @ModelAttribute("allFeatures")
    public List<Feature> populateFeatures() {
        return featureManager.getAllRootFeatures();
    }

    @RequestMapping(method = RequestMethod.POST)
    public String processSubmit(@ModelAttribute FormPairFileWrapper formPairFileWrapper, BindingResult result, SessionStatus status, HttpSession session) {
        logger.debug("Saving generate bulk pair form.");
        session.removeAttribute("sessionMessage");
        new FormPairFileWrapperValidator().validate(formPairFileWrapper, result);
        if (result.hasErrors()) {
            logger.debug("Form validation failed");
            return "forms/bulk";
        } else {
            Period currentPeriod = (Period) session.getAttribute("sessionPeriod");
            if (currentPeriod == null) {
                logger.error("Current period is null, form pair can't be generated.");
                return "error";
            }
            List<Feature> featureList = new ArrayList<Feature>();
            for (Feature f : formPairFileWrapper.getFeatureList()) {
                if (f.getParent() != null) {
                    featureList.add(f);
                } else {
                    if (f.getSubFeatures().isEmpty()) {
                        featureList.add(f);
                    } else {
                        if (!containsAny(formPairFileWrapper.getFeatureList(), f.getSubFeatures())) {
                            featureList.add(f);
                        }
                    }
                }
            }

            Collections.sort(featureList, new FeatureParentComparator());
            status.setComplete();
            int ignored = 0;
            int errors = 0;
            int added = 0;
            try {
                CSVReader reader = new CSVReader(new InputStreamReader(formPairFileWrapper.getFile().getInputStream(), "utf-8"), ';');
                //skip the header
                String[] nextLine = reader.readNext();
                while ((nextLine = reader.readNext()) != null) {
                    if (nextLine.length == 2) {
                        Participant student = participantManager.getByUco(nextLine[0]);
                        if (student == null) {
                            errors++;
                        } else {
                            Organization org = organizationManager.getByIdNumber(nextLine[1]);
                            if (org == null) {
                                errors++;
                            } else {
                                List<Participant> managers = participantManager.getAllManagersFor(org);
                                if (managers.size() < 1) {
                                    errors++;
                                } else {
                                    this.formManager.createNewPair(student, managers.get(0), featureList, currentPeriod);
                                    added++;
                                }
                            }
                        }
                    } else {
                        logger.warn("CSV Line does not contain expected items, line:\n" + nextLine + " was ignored.");
                        errors++;
                    }
                }
            } catch (IOException e) {
                logger.error("CSV parsing failed.");
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "csv.parse.error"));
            }
            logger.debug("Generated forms: " + added + ", ignored: " + ignored + ", errors: " + errors);
            if (errors == 0 && ignored == 0) {
                SessionMessageWrapper mes = new SessionMessageWrapper("okMessage", "successfully.imported");
                String[] params = new String[1];
                params[0] = String.valueOf(added);
                mes.setParam(params);
                session.setAttribute("sessionMessage", mes);
            } else {
                SessionMessageWrapper mes = new SessionMessageWrapper("warningMessage", "successfully.imported.warning");
                String[] params = new String[3];
                params[0] = String.valueOf(added);
                params[1] = String.valueOf(errors);
                params[2] = String.valueOf(ignored);
                mes.setParam(params);
                session.setAttribute("sessionMessage", mes);
            }

            return "redirect:/forms/list";
        }
    }

    /**
     * Checks if one list of features contains any of feature from other list.
     *
     * @param hay     list of features where to look for
     * @param needles list of features to find
     * @return true if it contains some feature, false otherwise
     */
    private boolean containsAny(List<Feature> hay, List<Feature> needles) {
        for (Feature feature : needles) {
            if (hay.contains(feature)) {
                return true;
            }
        }
        return false;
    }

}
