package cz.muni.fi.web;

import cz.muni.fi.domain.Manager;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Participant;
import cz.muni.fi.domain.Student;
import cz.muni.fi.exception.AlreadyPresentException;
import cz.muni.fi.service.OrganizationManager;
import cz.muni.fi.service.ParticipantManager;
import cz.muni.fi.util.OrganizationEditor;
import cz.muni.fi.validation.ParticipantValidator;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.view.RedirectView;

import javax.servlet.http.HttpSession;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 2.3.12
 */

@Controller
@RequestMapping("/managers/edit/{manId}")
@SessionAttributes(types = Manager.class, value = {"canEditOrg","newManager"})
public class EditManagerForm {
    private final ParticipantManager participantManager;

    private final OrganizationManager organizationManager;


    protected final Log logger = LogFactory.getLog(getClass());

    @Autowired
    public EditManagerForm(ParticipantManager manager, OrganizationManager organizationManager) {
        this.participantManager = manager;
        this.organizationManager = organizationManager;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        dataBinder.setDisallowedFields("role", "id");
        dataBinder.registerCustomEditor(Organization.class, new OrganizationEditor(organizationManager));

    }

    @RequestMapping(method = RequestMethod.GET)
    public Object setupForm(HttpSession session, @PathVariable("manId") Long manId, Model model) {
        logger.debug("Rendering edit manager form");
        session.removeAttribute("sessionMessage");
        Participant manager = participantManager.getById(manId);
        logger.info("Going to delete manager " + manId);
        if (manager == null) {
            logger.error("Requested user " + manId + " was not found.");
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "user.not.found"));
            return redirect("/organizations/list");
        }
        model.addAttribute(manager);
        boolean canEditOrg = organizationManager.canBeDeleted(((Manager) manager).getOrganization());
        model.addAttribute("canEditOrg", canEditOrg);
        model.addAttribute("newManager",false);
        return "organizations/manager";
    }

    @RequestMapping(method = RequestMethod.PUT)
    public Object processSubmit(HttpSession session, @ModelAttribute Manager manager, BindingResult result, SessionStatus status) {
        logger.debug("Saving edit manager form");
        new ParticipantValidator().validate(manager, result);
        if (result.hasErrors()) {
            logger.debug("Form validation failed");
            return "organizations/manager";
        } else {
            try {
                this.participantManager.updateParticipant(manager);
            } catch (AlreadyPresentException e) {
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "participant.already.present"));
            }
            status.setComplete();
            if (session.getAttribute("sessionMessage") == null) {
                session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.saved"));
            }
            return redirect("/organizations/list");
        }
    }

    @ModelAttribute("allOrganizations")
    public List<Organization> populateOrganizations() {
        return organizationManager.getAllActiveOrganizations();
    }

    private RedirectView redirect(String url) {
        RedirectView view = new RedirectView(url, true);
        view.setExposeModelAttributes(false);
        return view;
    }

}
