package cz.muni.fi.web;

import cz.muni.fi.domain.Organization;
import cz.muni.fi.exception.AlreadyPresentException;
import cz.muni.fi.service.OrganizationManager;
import cz.muni.fi.validation.OrganizationValidator;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.view.RedirectView;

import javax.servlet.http.HttpSession;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 1.3.12
 */

@Controller
@RequestMapping("/organizations/{organizationId}/edit")
@SessionAttributes(types = Organization.class, value = "newOrganization")
public class EditOrganizationForm {
    private final OrganizationManager organizationManager;

    protected final Log logger = LogFactory.getLog(getClass());


    @Autowired
    public EditOrganizationForm(OrganizationManager organizationManager) {
        this.organizationManager = organizationManager;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        dataBinder.setDisallowedFields("id");
    }

    @RequestMapping(method = RequestMethod.GET)
    public Object setupForm(Model model, @PathVariable("organizationId") Long organizationId, HttpSession session) {
        logger.debug("Rendering edit organization form");
        session.removeAttribute("sessionMessage");
        Organization organization = organizationManager.getById(organizationId);
        if (organization == null) {
            logger.info("No such organization " + organizationId);
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "organization.not.found"));
            return redirect("/organizations/list");
        }
        model.addAttribute(organization);
        model.addAttribute("newOrganization", false);
        return "organizations/form";
    }

    @RequestMapping(method = RequestMethod.PUT)
    public Object processSubmit(HttpSession session, @ModelAttribute Organization organization, BindingResult result, SessionStatus status) {
        logger.debug("Saving edit organization form");
        new OrganizationValidator().validate(organization, result);
        if (result.hasErrors()) {
            logger.debug("Form validation failed");
            return "organizations/form";
        } else {
            organization.setActive(true);
            try {
                this.organizationManager.updateOrganization(organization);
            } catch (AlreadyPresentException e) {
                logger.info("Organization " + organization + " already present.");
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "organization.already.present"));
                status.setComplete();
                return redirect("/organizations/list");
            }
            status.setComplete();
            session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.saved"));
            return redirect("/organizations/list");
        }
    }

    private RedirectView redirect(String url) {
        RedirectView view = new RedirectView(url, true);
        view.setExposeModelAttributes(false);
        return view;
    }
}
