package cz.muni.fi.web;

import cz.muni.fi.domain.Authority;
import cz.muni.fi.domain.User;
import cz.muni.fi.service.UserManager;
import cz.muni.fi.util.HashGenerator;
import cz.muni.fi.validation.PasswordValidator;
import cz.muni.fi.validation.UserValidator;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import cz.muni.fi.wrapper.UserAuthorityWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.view.RedirectView;

import javax.servlet.http.HttpSession;
import java.security.NoSuchAlgorithmException;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 28.10.11
 */
@Controller

@SessionAttributes(types = UserAuthorityWrapper.class, value = "newUser")
public class EditUserForm {
    private UserManager userManager;

    protected final Log logger = LogFactory.getLog(getClass());

    @Autowired
    public EditUserForm(UserManager userManager) {
        this.userManager = userManager;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        dataBinder.setDisallowedFields("user.userName");
    }

    @RequestMapping(value = "/admin/{userId}", method = RequestMethod.GET)
    public String setupForm(HttpSession session, @PathVariable("userId") String userId, Model model) {
        logger.debug("Rendering form for edit of user " + userId);
        session.removeAttribute("sessionMessage");
        UserAuthorityWrapper uaw = new UserAuthorityWrapper();
        User user = userManager.getById(userId);
        if (user == null) {
            logger.warn("User " + userId + " was not found.");
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "user.not.found"));
            return "redirect:/admin";
        } else {
            uaw.setAvailableRoles(populateRoles(user));
        }
        uaw.setUser(user);
        uaw.setAuthority(new Authority());
        model.addAttribute("newUser", false);
        model.addAttribute("userAuthorityWrapper", uaw);
        return "admin/form";
    }

    @RequestMapping(value = "/admin/{userId}", method = RequestMethod.PUT)
    public Object processSubmit(HttpSession session, @ModelAttribute UserAuthorityWrapper userAuthorityWrapper, BindingResult result, SessionStatus status) {
        logger.debug("Saving form for edit user");
        session.removeAttribute("sessionMessage");
        if (!"NONE".equals(userAuthorityWrapper.getAuthority().getRole())) {
            //we are adding role
            logger.debug("Adding role.");
            User updated = userManager.addUserAuthority(userAuthorityWrapper.getUser(), userAuthorityWrapper.getAuthority());
            userAuthorityWrapper.setUser(updated);
            userAuthorityWrapper.setAuthority(new Authority());
            userAuthorityWrapper.setAvailableRoles(populateRoles(updated));
            logger.debug("Role added for user " + updated.getUserName() + ", new value is " + updated.getAuthorities());
            return "admin/form";
        }
        new UserValidator().validate(userAuthorityWrapper.getUser(), result);
        if (result.hasErrors()) {
            logger.debug("Form validation failed.");
            return "admin/form";
        }
        //if password is filled, then validate it
        if (StringUtils.hasLength(userAuthorityWrapper.getPassword())) {
            new PasswordValidator().validate(userAuthorityWrapper.getPassword(), userAuthorityWrapper.getPasswordConfirm(), result);
            if (result.hasErrors()) {
                logger.debug("Form validation failed.");
                return "admin/form";
            } else {
                try {
                    userAuthorityWrapper.getUser().setPassword(HashGenerator.computeHash(userAuthorityWrapper.getPassword()));
                } catch (NoSuchAlgorithmException e) {
                    logger.error("En exception happened while trying to generate password hash.", e);
                    return "error";
                }
            }
        }
        this.userManager.updateUser(userAuthorityWrapper.getUser());
        session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.saved"));
        status.setComplete();
        return redirect("/admin");
    }


    @RequestMapping("/admin/delete/role/{roleId}")
    public Object deleteRole(HttpSession session, @PathVariable("roleId") Long roleId, @ModelAttribute UserAuthorityWrapper userAuthorityWrapper) {
        session.removeAttribute("sessionMessage");
        logger.debug("Deleting user role " + roleId);
        Authority aut = userManager.getAuthorityById(roleId);
        if (aut == null) {
            logger.warn("Role " + roleId + " was not found.");
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "user.not.found"));
            return "redirect:/admin";
        }
        User updated = userManager.removeUserAuthority(userAuthorityWrapper.getUser(), aut);
        userAuthorityWrapper.setUser(updated);
        userAuthorityWrapper.setAvailableRoles(populateRoles(updated));
        return redirect("/admin/" + userAuthorityWrapper.getUser().getUserName());
    }

    private List<String> populateRoles(User user) {
        List<Authority> userRoles = user.getAuthorities();
        List<String> roles = userManager.getAvailableRoles();
        for (Authority aut : userRoles) {
            if (roles.contains(aut.getRole())) {
                roles.remove(aut.getRole());
            }
        }
        return roles;
    }

    private RedirectView redirect(String url) {
        RedirectView view = new RedirectView(url, true);
        view.setExposeModelAttributes(false);
        return view;
    }


}
