package cz.muni.fi.web;

import cz.muni.fi.domain.*;
import cz.muni.fi.exception.DataIntegrityViolationException;
import cz.muni.fi.service.*;
import cz.muni.fi.util.GraphBar;
import cz.muni.fi.util.OrganizationEditor;
import cz.muni.fi.util.PeriodEditor;
import cz.muni.fi.wrapper.FeatureRateWrapper;
import cz.muni.fi.wrapper.PeriodOrganizationWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.ServletException;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.*;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 16.8.11
 */
@Controller
@SessionAttributes(types = PeriodOrganizationWrapper.class)
public class FeatureRateController {

    protected final Log logger = LogFactory.getLog(getClass());

    private final PlainFeatureRateManager plainFeatureRateManager;

    private final AggregatedFeatureRateManager aggregatedFeatureRateManager;

    private final FormManager formManager;

    private final PeriodManager periodManager;

    private final OrganizationManager organizationManager;

    private final FeatureManager featureManager;


    @Autowired
    public FeatureRateController(PlainFeatureRateManager manager, PeriodManager periodManager, OrganizationManager organizationManager, FormManager formManager, FeatureManager featureManager, AggregatedFeatureRateManager aggregatedFeatureRateManager) {
        this.plainFeatureRateManager = manager;
        this.periodManager = periodManager;
        this.organizationManager = organizationManager;
        this.formManager = formManager;
        this.featureManager = featureManager;
        this.aggregatedFeatureRateManager = aggregatedFeatureRateManager;
    }

    @InitBinder
    public void setEditors(WebDataBinder dataBinder) {
        dataBinder.registerCustomEditor(Period.class, new PeriodEditor(periodManager));
        dataBinder.registerCustomEditor(Organization.class, new OrganizationEditor(organizationManager));


    }

    @RequestMapping("/pdf")
    public ModelAndView pdfHandler(@ModelAttribute("periodOrganizationWrapper") PeriodOrganizationWrapper periodOrganizationWrapper, Model model, HttpSession session) {
        Period period = periodOrganizationWrapper.getPeriod();
        Organization organization = periodOrganizationWrapper.getOrganization();
        model.addAllAttributes(prepareData(period, organization, periodOrganizationWrapper.isAggregate()));
        logger.info("Rendering pdf report for period " + period + " for organization " + organization);
        return new ModelAndView("PdfRateReport", "model", model);
    }

    @RequestMapping("/excel")
    public ModelAndView excelHandler(@ModelAttribute("periodOrganizationWrapper") PeriodOrganizationWrapper periodOrganizationWrapper, Model model, HttpSession session) {
        Period period = periodOrganizationWrapper.getPeriod();
        Organization organization = periodOrganizationWrapper.getOrganization();
        model.addAllAttributes(prepareData(period, organization, periodOrganizationWrapper.isAggregate()));
        List<FormPair> pairs = formManager.getAllFormPairs(period, organization);
        model.addAttribute("formPairs", pairs);
        List<Feature> features = featureManager.getAllFeatures();
        List<Feature> used = new ArrayList<Feature>();
        for (FormPair pair : pairs) {
            if ("completed".equals(pair.getState())) {
                for (FeatureRating rating : pair.getManagersForm().getFeatureRatingList()) {
                    if (!used.contains(rating.getFeature())) {
                        used.add(rating.getFeature());
                    }
                }
            }
        }
        features.retainAll(used);
        model.addAttribute("features", features);
        logger.info("Rendering excel report for period " + period + " for organization " + organization);
        return new ModelAndView("ExcelRateReport", "model", model);
    }

    @RequestMapping(value = "/rates/list", method = RequestMethod.GET)
    public ModelAndView htmlHandler(HttpSession session)
            throws ServletException, IOException {
        logger.info("Rendering rates list using GET.");
        Map<String, Object> myModel = new HashMap<String, Object>();
        Period period = (Period) session.getAttribute("sessionPeriod");
        if (period == null) {
            try {
                period = periodManager.getCurrentPeriod();
            } catch (DataIntegrityViolationException e) {
                return new ModelAndView("error", myModel);
            }
        }
        PeriodOrganizationWrapper wrapper = new PeriodOrganizationWrapper();
        wrapper.setPeriod(period);
        wrapper.setOrganization(null);
        wrapper.setAggregate(true);
        myModel.put("periodOrganizationWrapper", wrapper);
        myModel.putAll(prepareData(period, null, true));

        return new ModelAndView("rates/list", myModel);
    }

    @RequestMapping(value = "/rates/list", method = RequestMethod.POST)
    public String htmlHandlerPost(HttpSession session, Model model, @ModelAttribute("periodOrganizationWrapper") PeriodOrganizationWrapper periodOrganizationWrapper)
            throws ServletException, IOException {
        logger.info("Rendering rates list using POST - setting period/organization.");
        Period statsPeriod = periodOrganizationWrapper.getPeriod();
        Organization statsOrg = periodOrganizationWrapper.getOrganization();
        model.addAllAttributes(prepareData(statsPeriod, statsOrg, periodOrganizationWrapper.isAggregate()));
        return "rates/list";
    }

    @ModelAttribute
    public List<Period> populatePeriods() {
        return periodManager.getAllPeriods();
    }

    @ModelAttribute
    public List<Organization> populateOrganizations() {
        return organizationManager.getAllActiveOrganizations();
    }

    /**
     * Prepares data for rendering graphs.
     *
     * @return model and view for rendering graphs
     */
    private Map<String, Object> prepareData(Period period, Organization organization, boolean agg) {
        FeatureRateManager featureRateManager = agg ?
                this.aggregatedFeatureRateManager :
                this.plainFeatureRateManager;
        System.out.println(agg ? "Agreguje se" : "neni agregace, puvodni");
        Map<String, Object> myModel = new HashMap<String, Object>();
        List<FeatureRate> rates = null;
        if (period == null) {
            if (organization == null) {
                //global among periods and orgs.
                rates = featureRateManager.getAggregatedFeatureRates();
            } else {
                //global among periods but for certain org
                rates = featureRateManager.getAggregatedFeatureRates(organization);
            }
        } else {
            if (organization == null) {
                //period is given, organizations are global
                rates = featureRateManager.getAggregatedFeatureRates(period);
            } else {
                //period and organization is given
                rates = featureRateManager.getAllFeatureRates(period, organization);
            }
        }
        List<FeatureRateWrapper> wrapperList = new ArrayList<FeatureRateWrapper>();
        int average = featureRateManager.getAverage(period, organization);
        int totalPosImp = featureRateManager.getTotalImprovementCount(true, period, organization);
        int totalNegImp = featureRateManager.getTotalImprovementCount(false, period, organization);
        for (FeatureRate rate : rates) {
            if (rate.getResponses() > 0) {
                FeatureRateWrapper wrapper = new FeatureRateWrapper();
                wrapper.setFeature(rate.getFeature());
                float rateVal = rate.getValue();
                float diff = average - rateVal;
                float imp = 0;
                if (diff > 0) {
                    imp = (diff / (totalPosImp != 0 ? totalPosImp : 1)) * 100;
                    wrapper.setValue(Math.round(imp));
                    wrapperList.add(wrapper);
                }
            }
        }
        Collections.sort(wrapperList);
        //list of aspects that need improvement
        //rendered as pie graph
        myModel.put("rates", wrapperList);

        List<FeatureRateWrapper> wrapperTotalList = new ArrayList<FeatureRateWrapper>();
        List<GraphBar> bars = new ArrayList<GraphBar>();
        for (FeatureRate rate : rates) {
            if (rate.getResponses() > 0) {
                FeatureRateWrapper wrapper = new FeatureRateWrapper();
                wrapper.setFeature(rate.getFeature());
                wrapper.setValue(rate.getValue());
                wrapperTotalList.add(wrapper);
                GraphBar bar = new GraphBar();
                bar.setColor(wrapper.getColor());
                bar.setValue(wrapper.getValue());
                bars.add(bar);
            }
        }
        Collections.sort(wrapperTotalList);
        Collections.sort(bars);
        int i = 0;
        for (GraphBar bar : bars) {
            bar.setBarName("bar" + i);
            bar.setOrdNumber(i);
            i++;
        }

        myModel.put("totalRates", wrapperTotalList);
        myModel.put("totalRatesBars", bars);

        String year = (period == null) ? null : period.toString();

        myModel.put("year", year);
        myModel.put("organization", organization);
        return myModel;
    }
}
