package cz.muni.fi.web;

import cz.muni.fi.domain.*;
import cz.muni.fi.exception.DataIntegrityViolationException;
import cz.muni.fi.exception.EmailSendException;
import cz.muni.fi.mail.MailCommons;
import cz.muni.fi.service.FormManager;
import cz.muni.fi.service.UserManager;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.security.Principal;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 12.8.11
 */
@Controller
public class FormsController {

    protected final Log logger = LogFactory.getLog(getClass());

    private final FormManager formManager;

    private final UserManager userManager;

    private final MailCommons mailCommons;

    @Autowired
    public FormsController(FormManager manager, UserManager userManager, MailCommons mailCommons) {
        this.formManager = manager;
        this.userManager = userManager;
        this.mailCommons = mailCommons;
    }


    @RequestMapping("/forms/list")
    public ModelAndView formsListHandler(HttpSession session) throws ServletException, IOException {
        SessionMessageWrapper atr = (SessionMessageWrapper) session.getAttribute("sessionMessage");
        if (atr != null) {
            if (atr.isSeen()) {
                session.removeAttribute("sessionMessage");
            } else {
                atr.setSeen(true);
            }
        }
        logger.debug("Rendering list of form pairs");
        Map<String, Object> myModel = new HashMap<String, Object>();
        Period period = (Period) session.getAttribute("sessionPeriod");
        if (period == null) {
            myModel.put("pairs", this.formManager.getAllFormPairs());
        } else {
            myModel.put("pairs", this.formManager.getAllFormPairs(period));
        }
        return new ModelAndView("forms/list", "model", myModel);

    }

    @RequestMapping("/public/forms/thanks")
    public String thanksHandler() {
        logger.debug("Form was saved, showing thanks message");
        return "forms/thanks";
    }

    @RequestMapping("/public/forms/timeout")
    public String timeOutHandler() {
        logger.debug("Form timeout handler.");
        return "forms/timeout";
    }

    @RequestMapping("/public/forms/notfound")
    public String notFoundHandler() {
        logger.debug("Form notfound handler.");
        return "forms/notfound";
    }

    @RequestMapping("/forms/sendmail/{formPairId}")
    public String mailHandler(HttpSession session, @PathVariable("formPairId") Long id, HttpServletRequest request, Principal principal) {
        logger.debug("Requsting send mail of " + id);
        session.removeAttribute("sessionMessage");
        FormPair pair = formManager.getFormPairById(id);
        if (pair == null) {
            logger.error("Can't find form pair with id " + id);
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "pair.not.found"));
        } else {
            try {
                String myId = principal.getName();
                User me = userManager.getById(myId);
                String myName = me.getName();
                String myMail = me.getEmail();
                mailCommons.sendMailAboutForm(pair, true, request, myName, myMail);
                mailCommons.sendMailAboutForm(pair, false, request, myName, myMail);
                formManager.setFormPairTimestamp(pair);
                logger.info("Mails were successfully sent.");
                session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "mail.was.sent"));
            } catch (EmailSendException e) {
                logger.error("Error while sending mail, mails were not sent.", e);
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "mail.error"));
            }
        }
        return "redirect:/forms/list";

    }

    @RequestMapping("/forms/preview/{formHash}")
    public String viewPreview(HttpSession session, @PathVariable("formHash") String hash, Model model) {
        logger.debug("Rendering form preview.");
        session.removeAttribute("sessionMessage");
        Form form = this.formManager.getFormByHash(hash);
        if (form == null) {
            logger.warn("Requested form not found: " + hash);
            return "error";
        }
        for (FeatureRating rating : form.getFeatureRatingList()) {
            rating.setValue(0);
        }
        model.addAttribute("form", form);
        model.addAttribute("preview", true);
        FormPair pair = formManager.findFormPair(form);
        Participant other;
        if ("student".equals(form.getOwner().getRole())) {
            other = pair.getManagersForm().getOwner();
        } else {
            other = pair.getStudentsForm().getOwner();
        }
        model.addAttribute("other",other);
        return "forms/rate";
    }

    @RequestMapping("/forms/delete/{formPairId}")
    public String deleteHandler(HttpSession session, @PathVariable("formPairId") Long id) {
        logger.info("Going to delete form pair " + id);
        FormPair pair = formManager.getFormPairById(id);
        if (pair == null) {
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "pair.not.found"));
            logger.error("Form pair " + id + " was not found.");
        } else {
            try {
                formManager.deleteFormPair(pair);
                session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.deleted"));
                logger.debug("Deleted.");
            } catch (DataIntegrityViolationException e) {
                logger.error("Form pair can't be deleted.", e);
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "pair.cannot.delete"));
                logger.error("Form pair " + id + " cannot be deleted.");
            }
        }
        return "redirect:/forms/list";
    }

}
