package cz.muni.fi.web;

import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.Manager;
import cz.muni.fi.domain.Participant;
import cz.muni.fi.domain.Period;
import cz.muni.fi.service.FeatureManager;
import cz.muni.fi.service.FormManager;
import cz.muni.fi.service.ParticipantManager;
import cz.muni.fi.util.FeatureEditor;
import cz.muni.fi.util.FeatureParentComparator;
import cz.muni.fi.util.ParticipantEditor;
import cz.muni.fi.validation.FormPairValidator;
import cz.muni.fi.wrapper.FormPairWrapper;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;

import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 12.8.11
 */
@Controller
@RequestMapping("/forms/new")
@SessionAttributes(types = FormPairWrapper.class)
public class GeneratePairForm {
    private final FormManager formManager;

    private final ParticipantManager participantManager;

    private final FeatureManager featureManager;


    protected final Log logger = LogFactory.getLog(getClass());


    @Autowired
    public GeneratePairForm(FormManager manager, ParticipantManager participantManager, FeatureManager featureManager) {
        this.formManager = manager;
        this.participantManager = participantManager;
        this.featureManager = featureManager;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        dataBinder.registerCustomEditor(Participant.class, new ParticipantEditor(participantManager));
        dataBinder.registerCustomEditor(Feature.class, new FeatureEditor(featureManager));

    }

    @RequestMapping(method = RequestMethod.GET)
    public String setupForm(HttpSession session, Model model) {
        logger.debug("Rendering generate pair form.");
        session.removeAttribute("sessionMessage");
        FormPairWrapper pairWrapper = new FormPairWrapper();
        pairWrapper.setFeatureList(featureManager.getAllFeatures());
        model.addAttribute("formPairWrapper", pairWrapper);
        return "forms/form";
    }

    @ModelAttribute("studentList")
    public List<Participant> populateStudents() {
        return participantManager.getAllStudents();
    }

    @ModelAttribute("managerList")
    public List<Participant> populateManagers() {
        List<Participant> allMans = participantManager.getAllManagers();
        List<Participant> result = new ArrayList<Participant>();
        for (Participant man : allMans) {
            if (((Manager) man).getOrganization().isActive()) {
                result.add(man);
            }
        }
        return result;
    }

    @ModelAttribute("allFeatures")
    public List<Feature> populateFeatures() {
        return featureManager.getAllRootFeatures();
    }

    @RequestMapping(method = RequestMethod.POST)
    public String processSubmit(@ModelAttribute FormPairWrapper pairWrapper, BindingResult result, SessionStatus status, HttpSession session) {
        logger.debug("Saving generate pair form.");
        session.removeAttribute("sessionMessage");
        new FormPairValidator().validate(pairWrapper, result);
        if (result.hasErrors()) {
            logger.debug("Form validation failed");
            return "forms/form";
        } else {
            Period currentPeriod = (Period) session.getAttribute("sessionPeriod");
            if (currentPeriod == null) {
                logger.error("Current period is null, form pair can't be generated.");
                return "error";
            }
            List<Feature> featureList = new ArrayList<Feature>();
            for (Feature f : pairWrapper.getFeatureList()) {
                if (f.getParent() != null) {
                    featureList.add(f);
                } else {
                    if (f.getSubFeatures().isEmpty()) {
                        featureList.add(f);
                    } else {
                        if (!containsAny(pairWrapper.getFeatureList(),f.getSubFeatures())) {
                            featureList.add(f);
                        }
                    }
                }
            }

            Collections.sort(featureList, new FeatureParentComparator());
            this.formManager.createNewPair(pairWrapper.getStudent(), pairWrapper.getManager(), featureList, currentPeriod);
            status.setComplete();
            session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.saved"));
            return "redirect:/forms/list";
        }
    }

    /**
     * Checks if one list of features contains any of feature from other list.
     *
     * @param hay list of features where to look for
     * @param needles list of features to find
     * @return true if it contains some feature, false otherwise
     */
    private boolean containsAny(List<Feature> hay, List<Feature> needles) {
        for (Feature feature : needles) {
            if (hay.contains(feature)) {
                return true;
            }
        }
        return false;
    }

}

