package cz.muni.fi.web;

import au.com.bytecode.opencsv.CSVReader;
import cz.muni.fi.domain.Manager;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Participant;
import cz.muni.fi.domain.Student;
import cz.muni.fi.exception.AlreadyPresentException;
import cz.muni.fi.exception.DataIntegrityViolationException;
import cz.muni.fi.service.OrganizationManager;
import cz.muni.fi.service.ParticipantManager;
import cz.muni.fi.validation.FileWrapperValidator;
import cz.muni.fi.wrapper.OrganizationWrapper;
import cz.muni.fi.wrapper.ParticipantWrapper;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import cz.muni.fi.wrapper.UploadFileWrapper;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;

import javax.servlet.ServletException;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 4.11.11
 */
@Controller
@SessionAttributes(value = {"uploadFileWrapper", "organizations", "managers"})
public class OrganizationsController {

    protected final Log logger = LogFactory.getLog(getClass());

    private final ParticipantManager participantManager;

    private final OrganizationManager organizationManager;

    @Autowired
    public OrganizationsController(ParticipantManager manager, OrganizationManager organizationManager) {
        this.participantManager = manager;
        this.organizationManager = organizationManager;
    }

    @RequestMapping(value = "/organizations/list", method = RequestMethod.GET)
    public String organizationsListHandler(HttpSession session, Model model)
            throws ServletException, IOException {
        logger.debug("Rendering organizations and managers list view.");
        SessionMessageWrapper atr = (SessionMessageWrapper) session.getAttribute("sessionMessage");
        if (atr != null) {
            if (atr.isSeen()) {
                session.removeAttribute("sessionMessage");
            } else {
                atr.setSeen(true);
            }
        }
        constructOrganizations(model);
        constructManagers(model);
        model.addAttribute("uploadFileWrapper", new UploadFileWrapper());
        return "organizations/list";
    }

    @RequestMapping(value = "/organizations/list", method = RequestMethod.POST)
    public String saveFile(HttpSession session, @ModelAttribute UploadFileWrapper uploadFileWrapper, BindingResult result, SessionStatus status)
            throws ServletException, IOException {
        logger.debug("Saving managers and organizations list - importing file");
        session.removeAttribute("sessionMessage");
        new FileWrapperValidator().validate(uploadFileWrapper, result);
        if (result.hasErrors()) {
            logger.debug("Form validation failed");
            return "organizations/list";
        }
        logger.info("Uploaded file: " + uploadFileWrapper.getFile().getOriginalFilename());
        status.setComplete();

        int ignored = 0;
        int errors = 0;
        int added = 0;
        try {
            CSVReader reader = new CSVReader(new InputStreamReader(uploadFileWrapper.getFile().getInputStream(), "utf-8"), ';');
            //skip the header
            String[] nextLine = reader.readNext();
            while ((nextLine = reader.readNext()) != null) {
                if (nextLine.length == 4) {
                    Manager part = new Manager();
                    Organization org = new Organization();
                    org.setActive(true);
                    String[] nameParts = nextLine[2].split(",");
                    if (nameParts.length < 2) {
                        part.setSurname(nextLine[2].trim());
                    } else {
                        part.setSurname(nameParts[0].trim());
                        part.setName(nameParts[1].trim());
                    }
                    part.setMail(nextLine[3]);
                    org.setName(nextLine[1]);
                    org.setIdNumber(nextLine[0]);
                    part.setRole("manager");
                    try {
                        organizationManager.addOrganization(org);
                        part.setOrganization(org);
                        try {
                            participantManager.addParticipant(part);
                            added++;
                        } catch (AlreadyPresentException ex) {
                            organizationManager.deleteOrganization(org);
                            throw ex;
                        } catch (IllegalArgumentException ex) {
                            organizationManager.deleteOrganization(org);
                            throw ex;
                        }
                    } catch (AlreadyPresentException e) {
                        ignored++;
                        logger.warn("Organization " + org + " already present, was ignored.");
                    }  catch (IllegalArgumentException ex) {
                        errors  ++;
                        logger.warn("Exception occured while adding organization: " + ex.getMessage());
                    }
                } else {
                    logger.warn("CSV Line does not contain expected items, line:\n" + nextLine + " was ignored.");
                    errors++;
                }
            }
        } catch (IOException e) {
            logger.error("CSV parsing failed.");
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "csv.parse.error"));

        }
        logger.debug("Added organizations and managers: " + added + ", ignored: " + ignored + ", errors: " + errors);
        if (errors == 0 && ignored == 0) {
            SessionMessageWrapper mes = new SessionMessageWrapper("okMessage", "successfully.imported");
            String[] params = new String[1];
            params[0] = String.valueOf(added);
            mes.setParam(params);
            session.setAttribute("sessionMessage", mes);
        } else {
            SessionMessageWrapper mes = new SessionMessageWrapper("warningMessage", "successfully.imported.warning");
            String[] params = new String[3];
            params[0] = String.valueOf(added);
            params[1] = String.valueOf(errors);
            params[2] = String.valueOf(ignored);
            mes.setParam(params);
            session.setAttribute("sessionMessage", mes);
        }

        return "redirect:/organizations/list";
    }

    private void constructOrganizations(Model model) {
        List<OrganizationWrapper> orgs = new ArrayList<OrganizationWrapper>();
        for (Organization org : organizationManager.getAllOrganizations()) {
            OrganizationWrapper wrapper = new OrganizationWrapper();
            wrapper.setOrganization(org);
            wrapper.setCanBeDeleted(organizationManager.canBeDeleted(org));
            orgs.add(wrapper);
        }
        model.addAttribute("organizations", orgs);
    }

    private void constructManagers(Model model) {
        List<Participant> list = this.participantManager.getAllManagers();
        List<ParticipantWrapper> wrapperList = new ArrayList<ParticipantWrapper>();
        for (Participant part : list) {
            ParticipantWrapper pw = new ParticipantWrapper();
            pw.setParticipant(part);
            pw.setCanBeDeleted(participantManager.canParticipantBeDeleted(part));
            pw.setRating(participantManager.getDisagreementList(part).size());
            wrapperList.add(pw);
        }
        model.addAttribute("managers", wrapperList);
    }

    @RequestMapping("/organizations/enable/{orgId}")
    public String userDeleteHandler(HttpSession session, @PathVariable("orgId") Long orgId, Model model)
            throws ServletException, IOException {
        logger.debug("Going to enable organization " + orgId);
        session.removeAttribute("sessionMessage");
        Organization org = organizationManager.getById(orgId);
        if (org == null) {
            logger.error("Organization " + orgId + " was not found.");
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "organization.not.found"));
        } else {
            org.setActive(true);
            try {
                organizationManager.updateOrganization(org);
            } catch (AlreadyPresentException e) {
                logger.error("Duplicate Organization ID", e);
                return "error";
            }
            logger.info("Organization " + org + "activated.");
            session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.saved"));

        }
        constructManagers(model);
        constructOrganizations(model);
        return "redirect:/organizations/list";

    }


    @RequestMapping("/organizations/disable/{orgId}")
    public String orgDisableHandler(HttpSession session, @PathVariable("orgId") Long orgId, Model model)
            throws ServletException, IOException {
        logger.debug("Going to disable organization " + orgId);
        session.removeAttribute("sessionMessage");
        Organization org = organizationManager.getById(orgId);
        if (org == null) {
            logger.error("Organization " + orgId + " was not found.");
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "organization.not.found"));
        } else {
            org.setActive(false);
            try {
                organizationManager.updateOrganization(org);
            } catch (AlreadyPresentException e) {
                logger.error("Duplicate Organization ID", e);
                return "error";
            }
            logger.info("Organization " + org + "disabled.");
            session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.saved"));

        }
        constructManagers(model);
        constructOrganizations(model);
        return "redirect:/organizations/list";
    }

    @RequestMapping("/managers/delete/{manId}")
    public String managerDeleteHandler(HttpSession session, @PathVariable("manId") Long manId, Model model)
            throws ServletException, IOException {
        session.removeAttribute("sessionMessage");
        logger.info("Going to delete manager " + manId);
        Participant user = participantManager.getById(manId);
        if (user == null) {
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "user.not.found"));
            logger.error("Manager " + manId + " was not found.");
        } else {
            try {
                participantManager.deleteParticipant(user);
            } catch (DataIntegrityViolationException e) {
                logger.error("User can't be deleted.", e);
                return "error";
            }
            logger.info("Deleted " + manId);
            session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.deleted"));
        }
        constructManagers(model);
        constructOrganizations(model);
        return "redirect:/organizations/list";

    }

    @RequestMapping("/organizations/delete/{orgId}")
    public String orgDeleteHandler(HttpSession session, @PathVariable("orgId") Long orgId, Model model)
            throws ServletException, IOException {
        logger.debug("Going to delete organization " + orgId);
        session.removeAttribute("sessionMessage");
        Organization org = organizationManager.getById(orgId);
        if (org == null) {
            logger.error("Organization " + orgId + " was not found.");
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "organization.not.found"));
        } else {
            if (!organizationManager.canBeDeleted(org)) {
                logger.warn("Organization " + org + " cannot be deleted.");
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "organization.cannot.delete"));
            } else if (participantManager.getAllManagersFor(org).size() > 0) {
                logger.warn("Organization has some managers, can't be deleted.");
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "organization.cannot.delete.manager"));

            } else {
                organizationManager.deleteOrganization(org);
                logger.info("Organization " + org + "deleted.");
                session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.deleted"));
            }
        }
        constructManagers(model);
        constructOrganizations(model);
        return "redirect:/organizations/list";
    }
}
