package cz.muni.fi.web;

import cz.muni.fi.domain.Period;
import cz.muni.fi.domain.User;
import cz.muni.fi.exception.DataIntegrityViolationException;
import cz.muni.fi.service.PeriodManager;
import cz.muni.fi.service.UserManager;
import cz.muni.fi.util.PeriodEditor;
import cz.muni.fi.wrapper.PeriodDetail;
import cz.muni.fi.wrapper.PeriodWrapper;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.Errors;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.view.RedirectView;

import javax.servlet.http.HttpSession;
import java.security.Principal;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 24.11.11
 */
@Controller
@SessionAttributes(value = {"periods", "periodWrapper"})
public class PeriodsController {

    protected final Log logger = LogFactory.getLog(getClass());

    private final PeriodManager periodManager;

    private final UserManager userManager;


    @Autowired
    public PeriodsController(PeriodManager periodManager, UserManager userManager) {
        this.periodManager = periodManager;
        this.userManager = userManager;
    }

    @InitBinder
    public void setEditors(WebDataBinder dataBinder) {
        dataBinder.registerCustomEditor(Period.class, new PeriodEditor(periodManager));

    }

    @RequestMapping(value = "/periods/list", method = RequestMethod.GET)
    public String getAllPeriodsHandler(HttpSession session, Model model) {
        logger.debug("Rendering list of periods");
        constructPeriods(model);
        Period period = (Period) session.getAttribute("sessionPeriod");
        SessionMessageWrapper atr = (SessionMessageWrapper) session.getAttribute("sessionMessage");
        if (atr != null) {
            if (atr.isSeen()) {
                session.removeAttribute("sessionMessage");
            } else {
                atr.setSeen(true);
            }
        }
        if (period == null) {
            try {
                period = periodManager.getCurrentPeriod();
            } catch (DataIntegrityViolationException e) {
                logger.error("Internal error, no period found.", e);
                return "error";
            }
        }
        PeriodWrapper periodWrapper = new PeriodWrapper();
        periodWrapper.setPeriod(period);
        model.addAttribute(periodWrapper);
        return "periods/list";
    }

    private void constructPeriods(Model model) {
        List<PeriodWrapper> periodList = new ArrayList<PeriodWrapper>();
        for (Period p : periodManager.getAllPeriods()) {
            PeriodWrapper pw = new PeriodWrapper();
            pw.setPeriod(p);
            pw.setCanBeDeleted(periodManager.canPeriodBeDeleted(p));
            periodList.add(pw);
        }
        if (periodList.size() == 1) {
            periodList.get(0).setCanBeDeleted(false);
            logger.debug("Period is last, can't be deleted, reseting to false");
        }
        model.addAttribute("periods", periodList);

    }

    @RequestMapping(value = "/periods/list", method = RequestMethod.POST)
    public String changePeriodHandler(Principal principal, @ModelAttribute PeriodWrapper periodWrapper, Model model, HttpSession session) {
        logger.debug("Changing period");
        Period period = (Period) session.getAttribute("sessionPeriod");
        session.removeAttribute("sessionMessage");
        PeriodWrapper periodSession = new PeriodWrapper();
        periodSession.setPeriod(period);
        if (!periodWrapper.equals(periodSession)) {
            session.setAttribute("sessionPeriod", periodWrapper.getPeriod());
            String name = principal.getName();
            User user = userManager.getById(name);
            userManager.changeSelectedPeriod(user, periodWrapper.getPeriod());
        }
        constructPeriods(model);
        return "periods/list";
    }

    @RequestMapping("/periods/{periodId}")
    public String periodDetailHandler(HttpSession session, @PathVariable("periodId") Long periodId, Model model) {
        logger.debug("Rendering period detail for " + periodId);
        Period period = periodManager.getPeriodById(periodId);
        session.removeAttribute("sessionMessage");
        if (period == null) {
            logger.warn("No such period " + periodId);
            return "error";
        }
        PeriodDetail detail = new PeriodDetail();
        detail.setPeriod(period);
        detail.setNumberOfForms(periodManager.getNumberOfFormsForPeriod(period));
        model.addAttribute("detail", detail);
        return "periods/show";
    }

    @RequestMapping("/periods/delete/{periodId}")
    public Object periodDeleteHandler(HttpSession session, @PathVariable("periodId") Long periodId, Model model, @ModelAttribute PeriodWrapper periodWrapper, Errors errors) {
        logger.debug("Going to delete period " + periodId);
        session.removeAttribute("sessionMessage");
        Period period = periodManager.getPeriodById(periodId);
        if (period == null) {
            logger.warn("No such period " + periodId);
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "period.not.found"));

        } else {
            if (periodManager.getAllPeriods().size() < 2) {
                logger.warn("Last period, can't be deleted.");
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "period.cannot.delete"));
            } else {
                periodManager.deletePeriod(period);
                logger.debug("Period was deleted.");
                session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.deleted"));
                Period periodC = null;
                try {
                    logger.debug("Going to get current period.");
                    periodC = periodManager.getCurrentPeriod();
                } catch (DataIntegrityViolationException e) {
                    logger.error("Can't find current period.", e);
                    return "error";
                }
                //if the deleted period was in session
                if (period.equals(session.getAttribute("sessionPeriod"))) {
                    logger.debug("Re-setting period in session.");
                    session.setAttribute("sessionPeriod", periodC);
                }
            }
        }
        constructPeriods(model);
        return redirect("/periods/list");
    }

    @ModelAttribute("periodsSimple")
    public List<Period> getAllPeriods() {
        return periodManager.getAllPeriods();
    }

    @RequestMapping("/noperiod")
    public String noPeriodHandler() {
        logger.debug("Rendering no period page.");
        return "periods/noperiod";
    }

    private RedirectView redirect(String url) {
        RedirectView view = new RedirectView(url, true);
        view.setExposeModelAttributes(false);
        return view;
    }
}

