package cz.muni.fi.web;

import au.com.bytecode.opencsv.CSVReader;
import cz.muni.fi.domain.Participant;
import cz.muni.fi.domain.Student;
import cz.muni.fi.exception.AlreadyPresentException;
import cz.muni.fi.exception.DataIntegrityViolationException;
import cz.muni.fi.service.ParticipantManager;
import cz.muni.fi.validation.FileWrapperValidator;
import cz.muni.fi.wrapper.ParticipantWrapper;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import cz.muni.fi.wrapper.UploadFileWrapper;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;

import javax.servlet.ServletException;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 3.11.11
 */
@Controller
@SessionAttributes(value = {"uploadFileWrapper", "students"})
public class StudentsController {

    protected final Log logger = LogFactory.getLog(getClass());

    private final ParticipantManager participantManager;

    @Autowired
    public StudentsController(ParticipantManager manager) {
        this.participantManager = manager;
    }

    @RequestMapping(value = "/students/list", method = RequestMethod.GET)
    public String studentsListHandler(HttpSession session, Model model)
            throws ServletException, IOException {
        SessionMessageWrapper atr = (SessionMessageWrapper) session.getAttribute("sessionMessage");
        if (atr != null) {
            if (atr.isSeen()) {
                session.removeAttribute("sessionMessage");
            } else {
                atr.setSeen(true);
            }
        }
        logger.info("Rendering list of students");
        constructStudents(model);
        model.addAttribute("uploadFileWrapper", new UploadFileWrapper());
        return "students/list";

    }

    private void constructStudents(Model model) {
        List<Participant> list = this.participantManager.getAllStudents();
        List<ParticipantWrapper> wrapperList = new ArrayList<ParticipantWrapper>();
        for (Participant part : list) {
            ParticipantWrapper pw = new ParticipantWrapper();
            pw.setParticipant(part);
            pw.setCanBeDeleted(participantManager.canParticipantBeDeleted(part));
            pw.setRating(participantManager.getDisagreementList(part).size());
            wrapperList.add(pw);
        }
        model.addAttribute("students", wrapperList);
    }

    @RequestMapping("/students/delete/{userId}")
    public String userDeleteHandler(HttpSession session, @PathVariable("userId") Long userId, Model model)
            throws ServletException, IOException {
        session.removeAttribute("sessionMessage");
        Participant user = participantManager.getById(userId);
        logger.info("Going to delete student " + userId);
        if (user == null) {
            logger.error("Requested user " + userId + " was not found.");
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "user.not.found"));
        } else {
            try {
                participantManager.deleteParticipant(user);
                logger.info("Deleted " + userId);
                session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.deleted"));
            } catch (DataIntegrityViolationException e) {
                logger.error("User can't be deleted.", e);
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "user.cannot.delete"));
            }
        }
        constructStudents(model);
        return "redirect:/students/list";

    }

    @RequestMapping(value = "/students/list", method = RequestMethod.POST)
    public String create(HttpSession session, @ModelAttribute UploadFileWrapper uploadFileWrapper, BindingResult result, SessionStatus status) {
        logger.debug("Saving students list - importing file");
        session.removeAttribute("sessionMessage");
        new FileWrapperValidator().validate(uploadFileWrapper, result);
        if (result.hasErrors()) {
            logger.debug("Form validation failed");
            return "students/list";
        }
        status.setComplete();
        logger.info("Uploaded file: " + uploadFileWrapper.getFile().getOriginalFilename());
        int ignored = 0;
        int errors = 0;
        int added = 0;
        //skip the header
        try {
            CSVReader reader = new CSVReader(new InputStreamReader(uploadFileWrapper.getFile().getInputStream(), "utf-8"), ';');
            String[] nextLine = reader.readNext();
            while ((nextLine = reader.readNext()) != null) {
                if (nextLine.length == 3) {
                    Student part = new Student();
                    String[] nameParts = nextLine[2].split(",");
                    if (nameParts.length < 2) {
                        part.setSurname(nextLine[2].trim());
                    } else {
                        part.setSurname(nameParts[0].trim());
                        part.setName(nameParts[1].trim());
                    }
                    part.setUco(nextLine[1].trim());
                    part.setMail(nextLine[1].trim() + "@mail.muni.cz");
                    part.setRole("student");
                    try {
                        participantManager.addParticipant(part);
                        added++;
                    } catch (AlreadyPresentException e) {
                        ignored++;
                        logger.warn("Participant " + part + " already present, was ignored.");
                    } catch (IllegalArgumentException ex) {
                        errors++;
                        logger.warn("Exception occured while adding organization: " + ex.getMessage());
                    }
                    
                } else {
                    logger.warn("CSV Line does not contain expected items, line:\n" + nextLine + " was ignored.");
                    errors++;
                }
            }
        } catch (IOException e) {
            logger.error("CSV parsing failed.");
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "csv.parse.error"));

        }
        logger.debug("Added students: " + added + ", ignored: " + ignored + ", errors: " + errors);
        if (errors == 0 && ignored == 0) {
            SessionMessageWrapper mes = new SessionMessageWrapper("okMessage", "successfully.imported");
            String[] params = new String[1];
            params[0] = String.valueOf(added);
            mes.setParam(params);
            session.setAttribute("sessionMessage", mes);
        } else {
            SessionMessageWrapper mes = new SessionMessageWrapper("warningMessage", "successfully.imported.warning");
            String[] params = new String[3];
            params[0] = String.valueOf(added);
            params[1] = String.valueOf(errors);
            params[2] = String.valueOf(ignored);
            mes.setParam(params);
            session.setAttribute("sessionMessage", mes);
        }
        return "redirect:/students/list";
    }
}
