package cz.muni.fi.test;

import cz.muni.fi.dao.FeatureRateDao;
import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.FeatureRate;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Period;
import cz.muni.fi.service.*;
import junit.framework.Assert;
import org.junit.After;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertNull;
import static junit.framework.Assert.fail;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 15.8.11
 */

@RunWith(SpringJUnit4ClassRunner.class)
// ApplicationContext will be loaded from "/applicationContext.xml" and "/applicationContext-test.xml"
// in the root of the classpath
@ContextConfiguration(locations = {"/applicationContext-test.xml"})
public class FeatureManagerTest {

    @Autowired
    private FeatureManager featureManager;

    @Autowired
    private PlainFeatureRateManager plainFeatureRateManager;

    @Autowired
    private PeriodManager periodManager;

    @Autowired
    private OrganizationManager organizationManager;

    @Autowired
    private FeatureRateDao featureRateDao;

    @After
    public void tearDown() {
        assertEquals(0, featureManager.getAllFeatures().size());
        assertEquals(0, organizationManager.getAllOrganizations().size());
    }

    @Test
    public void testDeleteFeature() {
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        Feature feature = createSampleFeature();
        try {
            featureManager.deleteFeature(feature);
            fail("deleting non present feature");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        featureManager.addFeature(feature);
        Assert.assertNotNull(feature.getId());
        List<Feature> result = featureManager.getAllFeatures();
        Assert.assertEquals(1, result.size());
        assertTrue(result.contains(feature));
        featureManager.deleteFeature(feature);
        result = featureManager.getAllFeatures();
        Assert.assertEquals(0, result.size());
        feature.setId(null);
        featureManager.addFeature(feature);
        Feature feature2 = createSampleFeature();
        feature2.setParent(feature);
        featureManager.addFeature(feature2);
        result = featureManager.getAllFeatures();
        Assert.assertEquals(2, result.size());
        featureManager.deleteFeature(feature);
        result = featureManager.getAllFeatures();
        Assert.assertEquals(0, result.size());
        feature.setId(null);
        feature2.setId(null);
        feature.setSubFeatures(null);
        featureManager.addFeature(feature);
        feature2.setParent(feature);
        featureManager.addFeature(feature2);
        Feature found = featureManager.getFeatureById(feature.getId());
        assertEquals(1, found.getSubFeatures().size());
        featureManager.deleteFeature(feature2);
        found = featureManager.getFeatureById(feature.getId());
        assertEquals(0, found.getSubFeatures().size());
        featureManager.deleteFeature(feature);
        periodManager.deletePeriod(period);
    }

    @Test
    public void testUpdateFeature() {
        try {
            featureManager.updateFeature(null);
            fail("updating null");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        Feature feature = createSampleFeature();
        try {
            featureManager.updateFeature(feature);
            fail("updating non present feature");
        } catch (IllegalArgumentException ex) {
            //OK
        }

        featureManager.addFeature(feature);
        feature.setName("name new is");
        featureManager.updateFeature(feature);
        Feature result = featureManager.getFeatureById(feature.getId());
        assertEquals("name new is", result.getName());
        feature.setHint("hintiiik");
        featureManager.updateFeature(feature);
        result = featureManager.getFeatureById(feature.getId());
        assertEquals("hintiiik", result.getHint());
        Feature feature2 = createSampleFeature();
        feature2.setParent(feature);
        featureManager.addFeature(feature2);
        result = featureManager.getFeatureById(feature.getId());
        assertEquals(1, result.getSubFeatures().size());
        assertNull(result.getParent());
        result = featureManager.getFeatureById(feature2.getId());
        assertEquals(0, result.getSubFeatures().size());
        assertEquals(feature, result.getParent());
        feature2.setParent(null);
        featureManager.updateFeature(feature2);
        result = featureManager.getFeatureById(feature.getId());
        assertEquals(0, result.getSubFeatures().size());
        assertNull(result.getParent());
        result = featureManager.getFeatureById(feature2.getId());
        assertEquals(0, result.getSubFeatures().size());
        assertNull(result.getParent());
        feature2.setParent(feature);
        featureManager.updateFeature(feature2);
        result = featureManager.getFeatureById(feature.getId());
        assertEquals(1, result.getSubFeatures().size());
        assertNull(result.getParent());
        result = featureManager.getFeatureById(feature2.getId());
        assertEquals(0, result.getSubFeatures().size());
        assertEquals(feature, result.getParent());

        Feature feature3 = createSampleFeature();
        featureManager.addFeature(feature3);
        feature2.setParent(feature3);
        featureManager.updateFeature(feature2);
        result = featureManager.getFeatureById(feature.getId());
        assertEquals(0, result.getSubFeatures().size());
        assertNull(result.getParent());
        result = featureManager.getFeatureById(feature3.getId());
        assertEquals(1, result.getSubFeatures().size());
        assertNull(result.getParent());
        result = featureManager.getFeatureById(feature2.getId());
        assertEquals(0, result.getSubFeatures().size());
        assertEquals(feature3, result.getParent());
        feature.setParent(feature);
        //38:55 -  40:30
        try {
            featureManager.updateFeature(feature);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        featureManager.deleteFeature(feature);
        featureManager.deleteFeature(feature3);
        periodManager.deletePeriod(period);

    }

    @Test
    public void testGetAllFeatures() {
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        Feature feature = createSampleFeature();
        Feature f2 = createSampleFeature();
        Feature f3 = createSampleFeature();
        List<Feature> result = featureManager.getAllFeatures();
        assertEquals(0, result.size());
        featureManager.addFeature(feature);
        result = featureManager.getAllFeatures();
        assertEquals(1, result.size());
        assertTrue(result.contains(feature));
        featureManager.addFeature(f2);
        result = featureManager.getAllFeatures();
        assertEquals(2, result.size());
        assertTrue(result.contains(feature));
        assertTrue(result.contains(f2));
        featureManager.addFeature(f3);
        result = featureManager.getAllFeatures();
        assertEquals(3, result.size());
        assertTrue(result.contains(feature));
        assertTrue(result.contains(f2));
        assertTrue(result.contains(f3));
        featureManager.deleteFeature(feature);
        featureManager.deleteFeature(f2);
        featureManager.deleteFeature(f3);
        periodManager.deletePeriod(period);
    }

    @Test
    public void testGetAllRootFeatures() {
        Feature feature = createSampleFeature();
        Feature f2 = createSampleFeature();
        Feature f3 = createSampleFeature();
        List<Feature> result = featureManager.getAllRootFeatures();
        assertEquals(0, result.size());
        featureManager.addFeature(feature);
        result = featureManager.getAllRootFeatures();
        assertEquals(1, result.size());
        f2.setParent(feature);
        featureManager.addFeature(f2);
        result = featureManager.getAllRootFeatures();
        assertEquals(1, result.size());
        f3.setParent(feature);
        featureManager.addFeature(f3);
        result = featureManager.getAllRootFeatures();
        assertEquals(1, result.size());
        Feature found = featureManager.getFeatureById(feature.getId());
        assertEquals(2, found.getSubFeatures().size());
        featureManager.deleteFeature(f3);
        f3.setParent(null);
        f3.setId(null);
        featureManager.addFeature(f3);
        result = featureManager.getAllRootFeatures();
        assertEquals(2, result.size());
        assertTrue(result.contains(feature));
        assertTrue(result.contains(f3));
        featureManager.deleteFeature(feature);
        featureManager.deleteFeature(f3);
        result = featureManager.getAllFeatures();
        assertEquals(0, result.size());
        Feature f5 = createSampleFeature();
        Feature f4 = createSampleFeature();
        Feature f6 = createSampleFeature();
        featureManager.addFeature(f4);
        f5.setParent(f4);
        featureManager.addFeature(f5);
        f6.setParent(f5);
        featureManager.addFeature(f6);
        result = featureManager.getAllRootFeatures();
        assertEquals(1, result.size());
        assertEquals(1, result.get(0).getSubFeatures().size());
        assertEquals(1, result.get(0).getSubFeatures().get(0).getSubFeatures().size());
        assertEquals(f4, result.get(0));
        assertEquals(f5, result.get(0).getSubFeatures().get(0));
        assertEquals(f6, result.get(0).getSubFeatures().get(0).getSubFeatures().get(0));
        featureManager.deleteFeature(f5);
        result = featureManager.getAllFeatures();
        assertEquals(1, result.size());
        assertTrue(result.get(0).getSubFeatures().isEmpty());
        featureManager.deleteFeature(f4);

    }

    @Test
    public void testGetById() {
        try {
            featureManager.getFeatureById(null);
            fail("id null");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        Feature result = featureManager.getFeatureById(34L);
        assertNull(result);
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        Feature feature = createSampleFeature();
        featureManager.addFeature(feature);
        result = featureManager.getFeatureById(feature.getId());
        assertEquals(result, feature);
        assertEquals(result.getHint(), feature.getHint());
        assertEquals(result.getManagerQuestion(), feature.getManagerQuestion());
        assertEquals(result.getStudentQuestion(), feature.getStudentQuestion());
        assertEquals(result.getName(), feature.getName());
        featureManager.deleteFeature(feature);
        periodManager.deletePeriod(period);

    }

    @Test
    public void testAddFeature() throws Exception {
        try {
            featureManager.addFeature(null);
            fail("adding null");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        Feature feature = createSampleFeature();
        Period period = createSamplePeriod();
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        assertNotNull(feature.getId());
        Feature result = featureManager.getFeatureById(feature.getId());
        assertEquals(feature, result);
        assertEquals(feature.getHint(), result.getHint());
        assertEquals(feature.getName(), result.getName());
        assertEquals(feature.getManagerQuestion(), result.getManagerQuestion());
        assertEquals(feature.getStudentQuestion(), result.getStudentQuestion());

        FeatureRate rate = plainFeatureRateManager.getRateForFeature(feature, period);
        assertNotNull(rate);
        assertNotNull(rate.getPeriod());
        assertEquals(periodManager.getCurrentPeriod(), rate.getPeriod());
        assertEquals(0, rate.getResponses());
        assertEquals(0, rate.getSum());
        assertEquals(feature, rate.getFeature());

        rate = plainFeatureRateManager.getRateForFeature(feature, period, org);
        assertNotNull(rate);
        assertEquals(0, rate.getValue());

        Feature feature2 = createSampleFeature();
        feature2.setParent(feature2);
        try {
            featureManager.addFeature(feature2);
            fail();
        } catch (Exception ex) {
            //OK
        }

        feature2.setParent(feature);
        feature2.setSubFeatures(new ArrayList<Feature>(Arrays.asList(feature)));
        try {
            featureManager.addFeature(feature2);
            fail();
        } catch (Exception ex) {
            //OK
        }
        feature2.setSubFeatures(null);
        featureManager.addFeature(feature2);
        result = featureManager.getFeatureById(feature2.getId());
        assertEquals(feature2, result);
        assertEquals(feature, result.getParent());
        assertEquals(0, result.getSubFeatures().size());
        result = featureManager.getFeatureById(feature.getId());
        assertTrue(result.getSubFeatures().contains(feature2));

        featureManager.deleteFeature(feature2);
        featureManager.deleteFeature(feature);
        periodManager.deletePeriod(period);
        organizationManager.deleteOrganization(org);
    }

    @Test
    public void testCanFeatureBeDeleted() throws Exception{
        try {
            featureManager.canFeatureBeDeleted(null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        Feature feature = createSampleFeature();
        Period period = createSamplePeriod();
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        periodManager.addPeriod(period);
        assertEquals(0, featureManager.getAllFeatures().size());
        featureManager.addFeature(feature);
        assertEquals(1, featureManager.getAllFeatures().size());
        assertTrue(featureManager.canFeatureBeDeleted(feature));
        Feature feature2 = createSampleFeature();
        feature2.setId(34L);
        assertFalse(featureManager.canFeatureBeDeleted(feature2));
        FeatureRate rate = featureRateDao.findByFeature(feature, period, org);
        rate.setResponses(2);
        rate.setSum(20);
        featureRateDao.update(rate);
        assertFalse(featureManager.canFeatureBeDeleted(feature));
        rate.setResponses(0);
        rate.setSum(0);
        featureRateDao.update(rate);
        assertTrue(featureManager.canFeatureBeDeleted(feature));
        featureManager.deleteFeature(feature);
        assertEquals(0, featureManager.getAllFeatures().size());
        Feature feature3 = createSampleFeature();
        Feature f2 = createSampleFeature();
        featureManager.addFeature(feature3);
        assertEquals(1, featureManager.getAllFeatures().size());
        f2.setParent(feature3);
        featureManager.addFeature(f2);
        assertEquals(2, featureManager.getAllFeatures().size());
        rate = featureRateDao.findByFeature(f2, period, org);
        rate.setResponses(2);
        rate.setSum(20);
        featureRateDao.update(rate);
        assertFalse(featureManager.canFeatureBeDeleted(feature3));
        assertFalse(featureManager.canFeatureBeDeleted(f2));
        rate.setResponses(0);
        rate.setSum(0);
        featureRateDao.update(rate);
        assertTrue(featureManager.canFeatureBeDeleted(feature3));
        assertTrue(featureManager.canFeatureBeDeleted(f2));
        featureManager.deleteFeature(feature3);
        assertEquals(0, featureManager.getAllFeatures().size());
        periodManager.deletePeriod(period);
        organizationManager.deleteOrganization(org);

    }

    private Feature createSampleFeature() {
        Feature feature = new Feature();
        feature.setHint("hint");
        feature.setManagerQuestion("manager");
        feature.setStudentQuestion("student");
        feature.setName("name");
        return feature;
    }

    private static Period createSamplePeriod() {
        Period period = new Period();
        period.setDescription("decription");
        period.setNote("note");
        period.setYear(2011);
        return period;
    }
}
