package cz.muni.fi.test;

import cz.muni.fi.dao.FeatureRateDao;
import cz.muni.fi.domain.*;
import cz.muni.fi.service.*;
import org.junit.After;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.ArrayList;
import java.util.List;

import static junit.framework.Assert.assertNull;
import static junit.framework.Assert.fail;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 16.8.11
 */

@RunWith(SpringJUnit4ClassRunner.class)
// ApplicationContext will be loaded from "/applicationContext.xml" and "/applicationContext-test.xml"
// in the root of the classpath
@ContextConfiguration(locations = {"/applicationContext-test.xml"})
public class FeatureRateManagerTest {
    @After
    public void tearDown() {
        assertEquals(0, featureManager.getAllFeatures().size());
        assertEquals(0, organizationManager.getAllOrganizations().size());
    }

    @Autowired
    private PlainFeatureRateManager plainFeatureRateManager;

    @Autowired
    private FeatureManager featureManager;

    @Autowired
    private FormManager formManager;

    @Autowired
    private ParticipantManager participantManager;

    @Autowired
    private OrganizationManager organizationManager;

    @Autowired
    private PeriodManager periodManager;

    @Autowired
    private FeatureRateDao featureRateDao;

    @Test
    public void testGetAggregatedFeatureRates() throws Exception {
        try {
            plainFeatureRateManager.getAggregatedFeatureRates((Period) null);
            fail("period is null");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        Period period = createSamplePeriod();
        try {
            plainFeatureRateManager.getAggregatedFeatureRates(period);
            fail("period is null");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        periodManager.addPeriod(period);
        List<FeatureRate> list = plainFeatureRateManager.getAggregatedFeatureRates(period);
        assertEquals(0, list.size());
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Feature feature = createSampleFeature();
        featureManager.addFeature(feature);
        list = plainFeatureRateManager.getAggregatedFeatureRates(period);
        assertEquals(1, list.size());
        list = plainFeatureRateManager.getAggregatedFeatureRates(org);
        assertEquals(1, list.size());
        Feature feature2 = createSampleFeature();
        featureManager.addFeature(feature2);

        list = plainFeatureRateManager.getAggregatedFeatureRates(period);
        assertEquals(2, list.size());
        list = plainFeatureRateManager.getAggregatedFeatureRates(org);
        assertEquals(2, list.size());

        Organization org1 = new Organization();
        org1.setName("org1");
        org1.setIdNumber("3441");
        organizationManager.addOrganization(org1);

        list = plainFeatureRateManager.getAggregatedFeatureRates(period);
        assertEquals(2, list.size());
        list = plainFeatureRateManager.getAggregatedFeatureRates(org);
        assertEquals(2, list.size());

        featureManager.deleteFeature(feature2);

        FeatureRate rate = featureRateDao.findByFeature(feature, period, org);
        rate.setSum(20);
        rate.setResponses(4);
        featureRateDao.update(rate);

        list = plainFeatureRateManager.getAggregatedFeatureRates(period);
        assertEquals(1, list.size());
        assertEquals(20, list.get(0).getSum());
        assertEquals(4, list.get(0).getResponses());

        list = plainFeatureRateManager.getAggregatedFeatureRates(org);
        assertEquals(1, list.size());
        assertEquals(20, list.get(0).getSum());
        assertEquals(4, list.get(0).getResponses());

        list = plainFeatureRateManager.getAggregatedFeatureRates(org1);
        assertEquals(1, list.size());
        assertEquals(0, list.get(0).getSum());
        assertEquals(0, list.get(0).getResponses());

        rate = featureRateDao.findByFeature(feature, period, org1);
        rate.setSum(30);
        rate.setResponses(5);
        featureRateDao.update(rate);

        list = plainFeatureRateManager.getAggregatedFeatureRates(period);
        assertEquals(1, list.size());
        assertEquals(50, list.get(0).getSum());
        assertEquals(9, list.get(0).getResponses());

        list = plainFeatureRateManager.getAggregatedFeatureRates(org);
        assertEquals(1, list.size());
        assertEquals(20, list.get(0).getSum());
        assertEquals(4, list.get(0).getResponses());

        list = plainFeatureRateManager.getAggregatedFeatureRates(org1);
        assertEquals(1, list.size());
        assertEquals(30, list.get(0).getSum());
        assertEquals(5, list.get(0).getResponses());

        featureManager.deleteFeature(feature);
        periodManager.deletePeriod(period);
        organizationManager.deleteOrganization(org);
        organizationManager.deleteOrganization(org1);
    }


    @Test
    public void testGetRateForFeature() throws Exception {
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        Feature feature = createSampleFeature();
        featureManager.addFeature(feature);
        //global
        FeatureRate result = plainFeatureRateManager.getRateForFeature(feature);
        assertEquals(result.getFeature(), feature);
        assertEquals(0, result.getResponses());
        assertEquals(0, result.getSum());
        assertEquals(0, result.getValue());

        //no organization, no data
        result = plainFeatureRateManager.getRateForFeature(feature, period);
        assertNull(result);

        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);

        //per period
        result = plainFeatureRateManager.getRateForFeature(feature, period);
        assertEquals(result.getPeriod(), period);
        assertEquals(result.getFeature(), feature);
        assertEquals(0, result.getResponses());
        assertEquals(0, result.getSum());
        assertEquals(0, result.getValue());

        Feature feature2 = createSampleFeature();
        featureManager.addFeature(feature2);


        Manager manager = new Manager();
        manager.setMail("man@org.com");
        manager.setName("manager");
        manager.setRole("manager");
        manager.setOrganization(org);
        participantManager.addParticipant(manager);
        Student student = new Student();
        student.setRole("student");
        student.setMail("student@org.com");
        student.setName("student");
        student.setUco("3456");
        participantManager.addParticipant(student);
        List<Feature> list = new ArrayList<Feature>();
        list.add(feature);
        list.add(feature2);
        FormPair pair = formManager.createNewPair(student, manager, list, period);
        assertNotNull(pair);
        Form rated = pair.getStudentsForm();
        int val = 5;
        rated.getFeatureRatingList().get(0).setValue(val);
        rated.getFeatureRatingList().get(1).setValue(val + 3);
        formManager.saveForm(rated);
        Form resultForm = formManager.getFormByHash(rated.getHash());
        assertEquals(val, resultForm.getFeatureRatingList().get(0).getValue());
        FormPair result1 = formManager.findFormPair(rated);
        assertEquals(pair.getStudentsForm().getOwner().getRole(), result1.getState());

        rated = pair.getManagersForm();
        int val1 = 8;
        rated.getFeatureRatingList().get(0).setValue(val1);
        rated.getFeatureRatingList().get(1).setValue(val1 - 2);
        formManager.saveForm(rated);
        result1 = formManager.findFormPair(rated);
        assertEquals("completed", result1.getState());
        result = plainFeatureRateManager.getRateForFeature(feature);
        assertEquals(1, result.getResponses());
        assertEquals(65, result.getSum());
        assertEquals(65, result.getValue());

        result = plainFeatureRateManager.getRateForFeature(feature, period);
        assertEquals(1, result.getResponses());
        assertEquals(65, result.getSum());
        assertEquals(65, result.getValue());

        //second feature in different period is added
        Period period2 = createSamplePeriod();
        periodManager.addPeriod(period2);
        formManager.forceDeleteFormPair(pair);
        pair = formManager.createNewPair(student, manager, list, period2);
        assertNotNull(pair);
        rated = pair.getStudentsForm();
        val = 5;
        rated.getFeatureRatingList().get(0).setValue(val);
        rated.getFeatureRatingList().get(1).setValue(val + 3);
        formManager.saveForm(rated);
        resultForm = formManager.getFormByHash(rated.getHash());
        assertEquals(val, resultForm.getFeatureRatingList().get(0).getValue());
        result1 = formManager.findFormPair(rated);
        assertEquals(pair.getStudentsForm().getOwner().getRole(), result1.getState());

        rated = pair.getManagersForm();
        val1 = 8;
        rated.getFeatureRatingList().get(0).setValue(val1);
        rated.getFeatureRatingList().get(1).setValue(val1 - 2);
        formManager.saveForm(rated);
        result1 = formManager.findFormPair(rated);
        assertEquals("completed", result1.getState());

        result = plainFeatureRateManager.getRateForFeature(feature);
        assertEquals(2, result.getResponses());
        assertEquals(130, result.getSum());
        assertEquals(65, result.getValue());

        result = plainFeatureRateManager.getRateForFeature(feature, period);
        assertEquals(1, result.getResponses());
        assertEquals(65, result.getSum());
        assertEquals(65, result.getValue());

        //and organizations check
        result = plainFeatureRateManager.getRateForFeature(feature, org);
        assertEquals(2, result.getResponses());
        assertEquals(130, result.getSum());
        assertEquals(65, result.getValue());

        result = plainFeatureRateManager.getRateForFeature(feature, period, org);
        assertEquals(1, result.getResponses());
        assertEquals(65, result.getSum());
        assertEquals(65, result.getValue());

        Organization org2 = new Organization();
        org2.setName("org2");
        org2.setIdNumber("3434");
        organizationManager.addOrganization(org2);

        FeatureRate rate = featureRateDao.findByFeature(feature, period, org2);
        rate.setResponses(2);
        rate.setSum(10);
        featureRateDao.update(rate);

        result = plainFeatureRateManager.getRateForFeature(feature, org);
        assertEquals(2, result.getResponses());
        assertEquals(130, result.getSum());
        assertEquals(65, result.getValue());

        result = plainFeatureRateManager.getRateForFeature(feature, period, org);
        assertEquals(1, result.getResponses());
        assertEquals(65, result.getSum());
        assertEquals(65, result.getValue());

        result = plainFeatureRateManager.getRateForFeature(feature, period);
        assertEquals(3, result.getResponses());
        assertEquals(75, result.getSum());

        result = plainFeatureRateManager.getRateForFeature(feature, period, org2);
        assertEquals(2, result.getResponses());
        assertEquals(10, result.getSum());

        formManager.forceDeleteFormPair(pair);
        featureManager.deleteFeature(feature);
        featureManager.deleteFeature(feature2);
        participantManager.deleteParticipant(student);
        participantManager.deleteParticipant(manager);
        organizationManager.deleteOrganization(org);
        organizationManager.deleteOrganization(org2);
        periodManager.deletePeriod(period);
        periodManager.deletePeriod(period2);
    }

    @Test
    public void testGetAverage() throws Exception {
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        Feature feature = createSampleFeature();
        Feature feature2 = createSampleFeature();
        featureManager.addFeature(feature);
        featureManager.addFeature(feature2);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Manager manager = new Manager();
        manager.setMail("man@org.com");
        manager.setName("manager");
        manager.setRole("manager");
        manager.setOrganization(org);
        participantManager.addParticipant(manager);
        Student student = new Student();
        student.setRole("student");
        student.setMail("student@org.com");
        student.setName("student");
        student.setUco("345");
        participantManager.addParticipant(student);
        List<Feature> list = new ArrayList<Feature>();
        list.add(feature);
        list.add(feature2);
        FormPair pair = formManager.createNewPair(student, manager, list, period);
        assertNotNull(pair);
        Form rated = pair.getStudentsForm();
        int val = 5;
        rated.getFeatureRatingList().get(0).setValue(val);
        rated.getFeatureRatingList().get(1).setValue(val + 3);
        formManager.saveForm(rated);
        Form result = formManager.getFormByHash(rated.getHash());
        assertEquals(val, result.getFeatureRatingList().get(0).getValue());
        FormPair result1 = formManager.findFormPair(rated);
        assertEquals(pair.getStudentsForm().getOwner().getRole(), result1.getState());

        rated = pair.getManagersForm();
        int val1 = 8;
        rated.getFeatureRatingList().get(0).setValue(val1);
        rated.getFeatureRatingList().get(1).setValue(val1 - 2);
        formManager.saveForm(rated);
        result1 = formManager.findFormPair(rated);
        assertEquals("completed", result1.getState());
        FeatureRate rate = plainFeatureRateManager.getRateForFeature(feature);
        assertEquals(1, rate.getResponses());
        assertEquals(65, rate.getSum());
        assertEquals(65, rate.getValue());
        int average = plainFeatureRateManager.getAverage();
        assertEquals(68, average);
        average = plainFeatureRateManager.getAverage(period);
        assertEquals(68, average);
        average = plainFeatureRateManager.getAverage((Period) null);
        assertEquals(68, average);

        formManager.forceDeleteFormPair(pair);
        Period period2 = createSamplePeriod();
        periodManager.addPeriod(period2);
        pair = formManager.createNewPair(student, manager, list, period2);
        assertNotNull(pair);
        rated = pair.getStudentsForm();
        val = 6;
        rated.getFeatureRatingList().get(0).setValue(val);
        rated.getFeatureRatingList().get(1).setValue(val + 3);
        formManager.saveForm(rated);
        Form resultForm = formManager.getFormByHash(rated.getHash());
        assertEquals(val, resultForm.getFeatureRatingList().get(0).getValue());
        result1 = formManager.findFormPair(rated);
        assertEquals(pair.getStudentsForm().getOwner().getRole(), result1.getState());

        rated = pair.getManagersForm();
        val1 = 8;
        rated.getFeatureRatingList().get(0).setValue(val1);
        rated.getFeatureRatingList().get(1).setValue(val1 - 2);
        formManager.saveForm(rated);
        result1 = formManager.findFormPair(rated);
        assertEquals("completed", result1.getState());

        FeatureRate resultRate = plainFeatureRateManager.getRateForFeature(feature);
        assertEquals(2, resultRate.getResponses());
        assertEquals(135, resultRate.getSum());
        assertEquals(68, resultRate.getValue());

        resultRate = plainFeatureRateManager.getRateForFeature(feature, period);
        assertEquals(1, resultRate.getResponses());
        assertEquals(65, resultRate.getSum());
        assertEquals(65, resultRate.getValue());

        average = plainFeatureRateManager.getAverage();
        assertEquals(71, average);
        average = plainFeatureRateManager.getAverage(period);
        assertEquals(68, average);
        average = plainFeatureRateManager.getAverage((Period) null);
        assertEquals(71, average);

        average = plainFeatureRateManager.getAverage(org);
        assertEquals(71, average);
        average = plainFeatureRateManager.getAverage(period, org);
        assertEquals(68, average);

        Organization org2 = new Organization();
        org2.setName("org2");
        org2.setIdNumber("3444");
        organizationManager.addOrganization(org2);

        rate = featureRateDao.findByFeature(feature, period, org2);
        rate.setResponses(2);
        rate.setSum(20);
        featureRateDao.update(rate);

        average = plainFeatureRateManager.getAverage();
        assertEquals(56, average);
        average = plainFeatureRateManager.getAverage(period);
        assertEquals(49, average);
        average = plainFeatureRateManager.getAverage((Period) null);
        assertEquals(56, average);

        average = plainFeatureRateManager.getAverage(org);
        assertEquals(71, average);
        average = plainFeatureRateManager.getAverage(period, org);
        assertEquals(68, average);

        average = plainFeatureRateManager.getAverage(org2);
        assertEquals(10, average);
        average = plainFeatureRateManager.getAverage(period, org2);
        assertEquals(10, average);
        average = plainFeatureRateManager.getAverage(period2, org2);
        assertEquals(0, average);


        formManager.forceDeleteFormPair(pair);
        featureManager.deleteFeature(feature);
        featureManager.deleteFeature(feature2);
        participantManager.deleteParticipant(student);
        participantManager.deleteParticipant(manager);
        organizationManager.deleteOrganization(org);
        organizationManager.deleteOrganization(org2);
        periodManager.deletePeriod(period);
        periodManager.deletePeriod(period2);
    }

    @Test
    public void testGetTotalImprovementCount() throws Exception {
        Feature feature = createSampleFeature();
        Feature feature2 = createSampleFeature();
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        featureManager.addFeature(feature2);
        Feature feature3 = createSampleFeature();
        featureManager.addFeature(feature3);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Manager manager = new Manager();
        manager.setMail("man@org.com");
        manager.setName("manager");
        manager.setRole("manager");
        manager.setOrganization(org);
        participantManager.addParticipant(manager);
        Student student = new Student();
        student.setRole("student");
        student.setMail("student@org.com");
        student.setName("student");
        student.setUco("3343");
        participantManager.addParticipant(student);
        List<Feature> list = new ArrayList<Feature>();
        list.add(feature);
        list.add(feature2);
        list.add(feature3);

        FormPair pair = formManager.createNewPair(student, manager, list, period);
        assertNotNull(pair);
        Form rated = pair.getStudentsForm();
        int val = 5;
        rated.getFeatureRatingList().get(0).setValue(val);
        rated.getFeatureRatingList().get(1).setValue(val + 3);
        rated.getFeatureRatingList().get(2).setValue(val);
        formManager.saveForm(rated);
        Form result = formManager.getFormByHash(rated.getHash());
        assertEquals(val, result.getFeatureRatingList().get(0).getValue());
        FormPair result1 = formManager.findFormPair(rated);
        assertEquals(pair.getStudentsForm().getOwner().getRole(), result1.getState());

        rated = pair.getManagersForm();
        int val1 = 8;
        rated.getFeatureRatingList().get(0).setValue(val1);
        rated.getFeatureRatingList().get(1).setValue(val1 - 2);
        rated.getFeatureRatingList().get(2).setValue(val1);
        formManager.saveForm(rated);
        result1 = formManager.findFormPair(rated);
        assertEquals("completed", result1.getState());
        FeatureRate rate = plainFeatureRateManager.getRateForFeature(feature);
        assertEquals(1, rate.getResponses());
        assertEquals(65, rate.getSum());
        assertEquals(65, rate.getValue());
        int average = plainFeatureRateManager.getAverage();
        assertEquals(67, average);
        int count = plainFeatureRateManager.getTotalImprovementCount(true);
        assertEquals(4, count);
        count = plainFeatureRateManager.getTotalImprovementCount(true, period);
        assertEquals(4, count);
        count = plainFeatureRateManager.getTotalImprovementCount(true, (Period) null);
        assertEquals(4, count);


        formManager.forceDeleteFormPair(pair);
        Period period2 = createSamplePeriod();
        periodManager.addPeriod(period2);
        pair = formManager.createNewPair(student, manager, list, period2);
        assertNotNull(pair);
        rated = pair.getStudentsForm();
        val = 6;
        rated.getFeatureRatingList().get(0).setValue(val);
        rated.getFeatureRatingList().get(1).setValue(val + 3);
        rated.getFeatureRatingList().get(2).setValue(val);

        formManager.saveForm(rated);
        Form resultForm = formManager.getFormByHash(rated.getHash());
        assertEquals(val, resultForm.getFeatureRatingList().get(0).getValue());
        result1 = formManager.findFormPair(rated);
        assertEquals(pair.getStudentsForm().getOwner().getRole(), result1.getState());

        rated = pair.getManagersForm();
        val1 = 8;
        rated.getFeatureRatingList().get(0).setValue(val1);
        rated.getFeatureRatingList().get(1).setValue(val1 - 2);
        rated.getFeatureRatingList().get(2).setValue(val1);

        formManager.saveForm(rated);
        result1 = formManager.findFormPair(rated);
        assertEquals("completed", result1.getState());

        FeatureRate resultRate = plainFeatureRateManager.getRateForFeature(feature);
        assertEquals(2, resultRate.getResponses());
        assertEquals(135, resultRate.getSum());
        assertEquals(68, resultRate.getValue());

        resultRate = plainFeatureRateManager.getRateForFeature(feature, period);
        assertEquals(1, resultRate.getResponses());
        assertEquals(65, resultRate.getSum());
        assertEquals(65, resultRate.getValue());

        count = plainFeatureRateManager.getTotalImprovementCount(true);
        assertEquals(4, count);
        count = plainFeatureRateManager.getTotalImprovementCount(true, period);
        assertEquals(4, count);
        count = plainFeatureRateManager.getTotalImprovementCount(true, (Period) null);
        assertEquals(4, count);

        count = plainFeatureRateManager.getTotalImprovementCount(true, org);
        assertEquals(4, count);
        count = plainFeatureRateManager.getTotalImprovementCount(true, period, org);
        assertEquals(4, count);


        Organization org2 = new Organization();
        org2.setName("org2");
        org2.setIdNumber("3443");
        organizationManager.addOrganization(org2);

        rate = featureRateDao.findByFeature(feature, period, org2);
        rate.setResponses(2);
        rate.setSum(20);
        featureRateDao.update(rate);

        average = plainFeatureRateManager.getAverage();
        assertEquals(60, average);
        count = plainFeatureRateManager.getTotalImprovementCount(true);
        assertEquals(21, count);
        count = plainFeatureRateManager.getTotalImprovementCount(true, period);
        assertEquals(26, count);
        count = plainFeatureRateManager.getTotalImprovementCount(true, (Period) null);
        assertEquals(21, count);

        count = plainFeatureRateManager.getTotalImprovementCount(true, org);
        assertEquals(4, count);
        count = plainFeatureRateManager.getTotalImprovementCount(true, org2);
        assertEquals(0, count);
        count = plainFeatureRateManager.getTotalImprovementCount(true, period, org);
        assertEquals(4, count);


        formManager.forceDeleteFormPair(pair);
        featureManager.deleteFeature(feature);
        featureManager.deleteFeature(feature2);
        featureManager.deleteFeature(feature3);
        participantManager.deleteParticipant(student);
        participantManager.deleteParticipant(manager);
        organizationManager.deleteOrganization(org);
        organizationManager.deleteOrganization(org2);
        periodManager.deletePeriod(period);
        periodManager.deletePeriod(period2);
    }


    @Test
    public void testGetAggregatedFeatureRatesGlobal() throws Exception {
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        Feature feature = createSampleFeature();
        featureManager.addFeature(feature);

        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);

        List<FeatureRate> result = plainFeatureRateManager.getAggregatedFeatureRates();
        assertEquals(1, result.size());
        assertEquals(feature, result.get(0).getFeature());
        assertEquals(0, result.get(0).getSum());
        assertEquals(0, result.get(0).getValue());
        assertEquals(0, result.get(0).getResponses());

        FeatureRate rate = featureRateDao.findByFeature(feature, period).get(0);
        rate.setResponses(1);
        rate.setSum(5);
        featureRateDao.update(rate);

        result = plainFeatureRateManager.getAggregatedFeatureRates();
        assertEquals(1, result.size());
        assertEquals(feature, result.get(0).getFeature());
        assertEquals(5, result.get(0).getSum());
        assertEquals(5, result.get(0).getValue());
        assertEquals(1, result.get(0).getResponses());

        Period period2 = createSamplePeriod();
        periodManager.addPeriod(period2);

        rate = featureRateDao.findByFeature(feature, period2).get(0);
        rate.setResponses(1);
        rate.setSum(7);
        featureRateDao.update(rate);
        result = plainFeatureRateManager.getAggregatedFeatureRates();

        assertEquals(1, result.size());
        assertEquals(feature, result.get(0).getFeature());
        assertEquals(12, result.get(0).getSum());
        assertEquals(6, result.get(0).getValue());
        assertEquals(2, result.get(0).getResponses());

        result = plainFeatureRateManager.getAggregatedFeatureRates(org);

        assertEquals(1, result.size());
        assertEquals(feature, result.get(0).getFeature());
        assertEquals(12, result.get(0).getSum());
        assertEquals(6, result.get(0).getValue());
        assertEquals(2, result.get(0).getResponses());

        featureManager.deleteFeature(feature);
        periodManager.deletePeriod(period);
        periodManager.deletePeriod(period2);
        organizationManager.deleteOrganization(org);
    }


    @Test
    public void testGetAllFeatureRates() throws Exception {
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        Feature feature = createSampleFeature();
        featureManager.addFeature(feature);

        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);

        List<FeatureRate> result = plainFeatureRateManager.getAllFeatureRates(period, org);
        assertEquals(1, result.size());
        assertEquals(feature, result.get(0).getFeature());
        assertEquals(0, result.get(0).getSum());
        assertEquals(0, result.get(0).getValue());
        assertEquals(0, result.get(0).getResponses());

        FeatureRate rate = featureRateDao.findByFeature(feature, period).get(0);
        rate.setResponses(1);
        rate.setSum(5);
        featureRateDao.update(rate);

        result = plainFeatureRateManager.getAllFeatureRates(period, org);
        assertEquals(1, result.size());
        assertEquals(feature, result.get(0).getFeature());
        assertEquals(5, result.get(0).getSum());
        assertEquals(5, result.get(0).getValue());
        assertEquals(1, result.get(0).getResponses());

        Period period2 = createSamplePeriod();
        periodManager.addPeriod(period2);

        rate = featureRateDao.findByFeature(feature, period2).get(0);
        rate.setResponses(1);
        rate.setSum(7);
        featureRateDao.update(rate);
        result = plainFeatureRateManager.getAllFeatureRates(period2, org);

        assertEquals(1, result.size());
        assertEquals(feature, result.get(0).getFeature());
        assertEquals(7, result.get(0).getSum());
        assertEquals(7, result.get(0).getValue());
        assertEquals(1, result.get(0).getResponses());

        result = plainFeatureRateManager.getAllFeatureRates(period, org);

        assertEquals(1, result.size());
        assertEquals(feature, result.get(0).getFeature());
        assertEquals(5, result.get(0).getSum());
        assertEquals(5, result.get(0).getValue());
        assertEquals(1, result.get(0).getResponses());

        Feature feature2 = createSampleFeature();
        featureManager.addFeature(feature2);
        result = plainFeatureRateManager.getAllFeatureRates(period, org);
        assertEquals(2, result.size());

        result = plainFeatureRateManager.getAllFeatureRates(period2, org);
        assertEquals(2, result.size());

        featureManager.deleteFeature(feature);
        featureManager.deleteFeature(feature2);
        periodManager.deletePeriod(period);
        periodManager.deletePeriod(period2);
        organizationManager.deleteOrganization(org);
    }

    private static Feature createSampleFeature() {
        Feature feature = new Feature();
        feature.setHint("hint");
        feature.setManagerQuestion("manager");
        feature.setStudentQuestion("student");
        feature.setName("name");
        return feature;
    }

    private static Period createSamplePeriod() {
        Period period = new Period();
        period.setDescription("decription");
        period.setNote("note");
        period.setYear(2011);
        return period;
    }
}
