package cz.muni.fi.test;

import cz.muni.fi.domain.*;
import cz.muni.fi.exception.DataIntegrityViolationException;
import cz.muni.fi.service.*;
import org.junit.After;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.ArrayList;
import java.util.List;

import static junit.framework.Assert.assertFalse;
import static junit.framework.Assert.assertNull;
import static junit.framework.Assert.fail;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 12.8.11
 */

@RunWith(SpringJUnit4ClassRunner.class)
// ApplicationContext will be loaded from "/applicationContext.xml" and "/applicationContext-test.xml"
// in the root of the classpath
@ContextConfiguration(locations = {"/applicationContext-test.xml"})
public class FormManagerTest {

    @Autowired
    private FormManager formManager;

    @Autowired
    private FeatureManager featureManager;

    @Autowired
    private OrganizationManager organizationManager;

    @Autowired
    private ParticipantManager participantManager;

    @Autowired
    private PlainFeatureRateManager plainFeatureRateManager;

    @Autowired
    private PeriodManager periodManager;

    @After
    public void tearDown() {
        List<Organization> list = organizationManager.getAllOrganizations();
        assertTrue(list.isEmpty());
        assertEquals(0, featureManager.getAllFeatures().size());

    }

    @Test
    public void testCreateNewPair() throws Exception {
        try {
            formManager.createNewPair(null, null, null, null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        Feature feature = createSampleFeature();
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Manager manager = new Manager();
        manager.setMail("man@org.com");
        manager.setName("manager");
        manager.setRole("manager");
        manager.setOrganization(org);
        participantManager.addParticipant(manager);
        try {
            formManager.createNewPair(manager, null, null, null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        Student student = new Student();
        student.setUco("54434");
        student.setRole("student");
        student.setMail("student@org.com");
        student.setName("student");
        participantManager.addParticipant(student);
        try {
            formManager.createNewPair(manager, student, null, null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        try {
            formManager.createNewPair(manager, student, new ArrayList<Feature>(), null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        List<Feature> list = new ArrayList<Feature>();
        list.add(feature);
        try {
            formManager.createNewPair(student, manager, list, null);
            fail("period is null");
        } catch (IllegalArgumentException ex) {
            //OK
        }

        FormPair pair = formManager.createNewPair(student, manager, list, period);
        assertNotNull(pair);
        assertNotNull(pair.getManagersForm());
        assertEquals(manager, pair.getManagersForm().getOwner());
        assertNotNull(pair.getManagersForm().getHash());
        assertEquals(1, pair.getManagersForm().getFeatureRatingList().size());
        assertEquals(0, pair.getManagersForm().getFeatureRatingList().get(0).getValue());
        assertEquals(feature, pair.getManagersForm().getFeatureRatingList().get(0).getFeature());
        assertNotNull(pair.getStudentsForm());
        assertEquals(student, pair.getStudentsForm().getOwner());
        assertNotNull(pair.getStudentsForm().getHash());
        assertEquals(1, pair.getStudentsForm().getFeatureRatingList().size());
        assertEquals(0, pair.getStudentsForm().getFeatureRatingList().get(0).getValue());
        assertEquals(feature, pair.getStudentsForm().getFeatureRatingList().get(0).getFeature());
        assertEquals("new", pair.getState());
        assertEquals("new", pair.getStudentsForm().getState());
        assertEquals("new", pair.getManagersForm().getState());
        assertNotNull(pair.getStudentsForm().getTimestamp());
        assertNotNull(pair.getManagersForm().getTimestamp());
        assertNull(pair.getMailTimestamp());
        assertNotNull(pair.getManagersForm().getPeriod());
        assertNotNull(pair.getStudentsForm().getPeriod());
        assertFalse(participantManager.canParticipantBeDeleted(student));
        assertFalse(participantManager.canParticipantBeDeleted(manager));

        formManager.forceDeleteFormPair(pair);
        participantManager.deleteParticipant(student);
        try {
            formManager.createNewPair(manager, student, list, period);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        participantManager.deleteParticipant(manager);
        try {
            formManager.createNewPair(manager, student, list, period);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        featureManager.deleteFeature(feature);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);

    }

    private Feature createSampleFeature() {
        Feature feature = new Feature();
        feature.setHint("hint");
        feature.setManagerQuestion("manager");
        feature.setStudentQuestion("student");
        feature.setName("name");
        return feature;
    }

    @Test
    public void testSaveForm() throws Exception {
        Feature feature = createSampleFeature();
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Manager manager = new Manager();
        manager.setMail("man@org.com");
        manager.setName("manager");
        manager.setRole("manager");
        manager.setOrganization(org);
        participantManager.addParticipant(manager);
        Student student = new Student();
        student.setUco("54434");
        student.setRole("student");
        student.setMail("student@org.com");
        student.setName("student");
        participantManager.addParticipant(student);
        List<Feature> list = new ArrayList<Feature>();
        list.add(feature);

        FormPair pair = formManager.createNewPair(student, manager, list, period);
        assertNotNull(pair);
        Form rated = pair.getStudentsForm();
        int val = 5;
        rated.getFeatureRatingList().get(0).setValue(val);
        formManager.saveForm(rated);
        Form result = formManager.getFormByHash(rated.getHash());
        assertEquals(val, result.getFeatureRatingList().get(0).getValue());
        FormPair result1 = formManager.findFormPair(rated);
        assertEquals(pair.getStudentsForm().getOwner().getRole(), result1.getState());
        assertEquals("saved", pair.getStudentsForm().getState());
        assertNotNull(pair.getStudentsForm().getTimestamp());

        rated = pair.getManagersForm();
        int val1 = 8;
        rated.getFeatureRatingList().get(0).setValue(val1);
        formManager.saveForm(rated);
        result1 = formManager.findFormPair(rated);
        assertEquals("saved", result1.getManagersForm().getState());
        assertNotNull(result1.getManagersForm().getTimestamp());

        assertEquals("completed", result1.getState());
        FeatureRate rate = plainFeatureRateManager.getRateForFeature(feature);
        assertEquals(1, rate.getResponses());
        assertEquals(65, rate.getSum());

        formManager.forceDeleteFormPair(pair);
        featureManager.deleteFeature(feature);
        participantManager.deleteParticipant(student);
        participantManager.deleteParticipant(manager);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);

    }

    @Test
    public void testFindFormPair() throws Exception {
        try {
            formManager.findFormPair(null);
            fail("finding null");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        Form notFound = new Form();
        notFound.setId(233L);
        assertNull(formManager.findFormPair(notFound));

        Feature feature = createSampleFeature();
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Manager manager = new Manager();
        manager.setMail("man@org.com");
        manager.setName("manager");
        manager.setRole("manager");
        manager.setOrganization(org);
        participantManager.addParticipant(manager);
        Student student = new Student();
        student.setUco("54434");
        student.setRole("student");
        student.setMail("student@org.com");
        student.setName("student");
        participantManager.addParticipant(student);
        List<Feature> list = new ArrayList<Feature>();
        list.add(feature);


        FormPair pair = formManager.createNewPair(student, manager, list, period);
        assertNotNull(pair);
        FormPair result = formManager.findFormPair(pair.getStudentsForm());
        assertNotNull(result.getManagersForm());
        assertEquals(manager, result.getManagersForm().getOwner());
        assertNotNull(result.getManagersForm().getHash());
        assertEquals(1, result.getManagersForm().getFeatureRatingList().size());
        assertEquals(0, result.getManagersForm().getFeatureRatingList().get(0).getValue());
        assertEquals(feature, result.getManagersForm().getFeatureRatingList().get(0).getFeature());
        assertNotNull(result.getStudentsForm());
        assertEquals(student, result.getStudentsForm().getOwner());
        assertNotNull(result.getStudentsForm().getHash());
        assertEquals(1, result.getStudentsForm().getFeatureRatingList().size());
        assertEquals(0, result.getStudentsForm().getFeatureRatingList().get(0).getValue());
        assertEquals(feature, result.getStudentsForm().getFeatureRatingList().get(0).getFeature());
        assertEquals("new", result.getState());
        assertEquals("new", result.getStudentsForm().getState());
        assertEquals("new", result.getManagersForm().getState());
        assertNotNull(result.getStudentsForm().getTimestamp());
        assertNotNull(result.getManagersForm().getTimestamp());
        assertNull(result.getMailTimestamp());

        result = formManager.findFormPair(pair.getManagersForm());
        assertNotNull(result.getManagersForm());
        assertEquals(manager, result.getManagersForm().getOwner());
        assertNotNull(result.getManagersForm().getHash());
        assertEquals(1, result.getManagersForm().getFeatureRatingList().size());
        assertEquals(0, result.getManagersForm().getFeatureRatingList().get(0).getValue());
        assertEquals(feature, result.getManagersForm().getFeatureRatingList().get(0).getFeature());
        assertNotNull(result.getStudentsForm());
        assertEquals(student, result.getStudentsForm().getOwner());
        assertNotNull(result.getStudentsForm().getHash());
        assertEquals(1, result.getStudentsForm().getFeatureRatingList().size());
        assertEquals(0, result.getStudentsForm().getFeatureRatingList().get(0).getValue());
        assertEquals(feature, result.getStudentsForm().getFeatureRatingList().get(0).getFeature());
        assertEquals("new", result.getState());
        assertEquals("new", result.getStudentsForm().getState());
        assertEquals("new", result.getManagersForm().getState());
        assertNotNull(result.getStudentsForm().getTimestamp());
        assertNotNull(result.getManagersForm().getTimestamp());
        assertNull(result.getMailTimestamp());

        formManager.forceDeleteFormPair(pair);
        participantManager.deleteParticipant(student);
        participantManager.deleteParticipant(manager);
        featureManager.deleteFeature(feature);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);


    }

    @Test
    public void testGetFormByHash() throws Exception {
        Feature feature = createSampleFeature();
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Manager manager = new Manager();
        manager.setMail("man1@org.com");
        manager.setName("manager");
        manager.setRole("manager");
        manager.setOrganization(org);
        participantManager.addParticipant(manager);
        Student student = new Student();
        student.setUco("54434");
        student.setRole("student");
        student.setMail("student1@org.com");
        student.setName("student");
        participantManager.addParticipant(student);
        List<Feature> list = new ArrayList<Feature>();
        list.add(feature);

        FormPair pair = formManager.createNewPair(student, manager, list, period);

        String hash = pair.getStudentsForm().getHash();
        Form form = formManager.getFormByHash(hash);
        assertEquals(pair.getStudentsForm(), form);
        assertEquals(pair.getStudentsForm().getOwner(), form.getOwner());
        assertEquals(pair.getStudentsForm().getId(), form.getId());
        assertEquals(pair.getStudentsForm().getFeatureRatingList().size(), form.getFeatureRatingList().size());
        assertEquals(pair.getStudentsForm().getFeatureRatingList().get(0), form.getFeatureRatingList().get(0));

        formManager.forceDeleteFormPair(pair);
        featureManager.deleteFeature(feature);
        participantManager.deleteParticipant(student);
        participantManager.deleteParticipant(manager);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);
    }

    @Test
    public void testCanBeAnswered() throws Exception {
        Feature feature = createSampleFeature();
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Manager manager = new Manager();
        manager.setMail("man@org.com");
        manager.setName("manager");
        manager.setRole("manager");
        manager.setOrganization(org);
        participantManager.addParticipant(manager);
        Student student = new Student();
        student.setUco("54434");
        student.setRole("student");
        student.setMail("student@org.com");
        student.setName("student");
        participantManager.addParticipant(student);
        List<Feature> list = new ArrayList<Feature>();
        list.add(feature);

        FormPair pair = formManager.createNewPair(student, manager, list, period);
        assertNotNull(pair);

        assertTrue(formManager.canBeAnswered(pair.getManagersForm()));
        assertTrue(formManager.canBeAnswered(pair.getStudentsForm()));

        formManager.openForm(pair.getManagersForm());

        pair = formManager.getFormPairById(pair.getId());
        assertTrue(formManager.canBeAnswered(pair.getManagersForm()));
        assertTrue(formManager.canBeAnswered(pair.getStudentsForm()));
        Form rated = pair.getStudentsForm();
        int val = 5;
        rated.getFeatureRatingList().get(0).setValue(val);
        formManager.saveForm(rated);
        Form result = formManager.getFormByHash(rated.getHash());
        assertEquals(val, result.getFeatureRatingList().get(0).getValue());
        FormPair result1 = formManager.findFormPair(rated);
        assertEquals(pair.getStudentsForm().getOwner().getRole(), result1.getState());
        assertEquals("saved", pair.getStudentsForm().getState());
        assertNotNull(pair.getStudentsForm().getTimestamp());

        rated = pair.getManagersForm();
        int val1 = 8;
        rated.getFeatureRatingList().get(0).setValue(val1);
        formManager.saveForm(rated);
        result1 = formManager.findFormPair(rated);
        assertEquals("saved", result1.getManagersForm().getState());
        assertNotNull(result1.getManagersForm().getTimestamp());

        assertEquals("completed", result1.getState());
        FeatureRate rate = plainFeatureRateManager.getRateForFeature(feature);
        assertEquals(1, rate.getResponses());
        assertEquals(65, rate.getSum());

        assertFalse(formManager.canBeAnswered(pair.getManagersForm()));
        assertFalse(formManager.canBeAnswered(pair.getStudentsForm()));

        try {
            formManager.openForm(pair.getManagersForm());
            fail("form opening of closed form");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        formManager.forceDeleteFormPair(pair);
        featureManager.deleteFeature(feature);
        participantManager.deleteParticipant(student);
        participantManager.deleteParticipant(manager);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);

    }

    @Test
    public void testEnableFormReopen() throws Exception {
        Feature feature = createSampleFeature();
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Manager manager = new Manager();
        manager.setMail("man@org.com");
        manager.setName("manager");
        manager.setRole("manager");
        manager.setOrganization(org);
        participantManager.addParticipant(manager);
        Student student = new Student();
        student.setUco("54434");
        student.setRole("student");
        student.setMail("student@org.com");
        student.setName("student");
        participantManager.addParticipant(student);
        List<Feature> list = new ArrayList<Feature>();
        list.add(feature);

        FormPair pair = formManager.createNewPair(student, manager, list, period);
        assertNotNull(pair);

        try {
            formManager.enableFormReopen(pair.getManagersForm());
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        formManager.openForm(pair.getManagersForm());


        pair = formManager.getFormPairById(pair.getId());
        Form rated = pair.getStudentsForm();
        int val = 5;
        rated.getFeatureRatingList().get(0).setValue(val);
        formManager.saveForm(rated);
        Form result = formManager.getFormByHash(rated.getHash());
        assertEquals(val, result.getFeatureRatingList().get(0).getValue());
        FormPair result1 = formManager.findFormPair(rated);
        assertEquals(pair.getStudentsForm().getOwner().getRole(), result1.getState());
        assertEquals("saved", pair.getStudentsForm().getState());
        assertNotNull(pair.getStudentsForm().getTimestamp());

        rated = pair.getManagersForm();
        int val1 = 8;
        rated.getFeatureRatingList().get(0).setValue(val1);
        formManager.saveForm(rated);
        result1 = formManager.findFormPair(rated);
        assertEquals("saved", result1.getManagersForm().getState());
        assertNotNull(result1.getManagersForm().getTimestamp());

        assertEquals("completed", result1.getState());
        FeatureRate rate = plainFeatureRateManager.getRateForFeature(feature);
        assertEquals(1, rate.getResponses());
        assertEquals(65, rate.getSum());


        try {
            formManager.enableFormReopen(pair.getManagersForm());
            fail("form reopening of closed form");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        formManager.forceDeleteFormPair(pair);
        featureManager.deleteFeature(feature);
        participantManager.deleteParticipant(student);
        participantManager.deleteParticipant(manager);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);
    }

    @Test
    public void testSetFormPairTimestamp() throws Exception {
        Feature feature = createSampleFeature();
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Manager manager = new Manager();
        manager.setMail("man@org.com");
        manager.setName("manager");
        manager.setRole("manager");
        manager.setOrganization(org);
        participantManager.addParticipant(manager);
        Student student = new Student();
        student.setUco("54434");
        student.setRole("student");
        student.setMail("student@org.com");
        student.setName("student");
        participantManager.addParticipant(student);
        List<Feature> list = new ArrayList<Feature>();
        list.add(feature);


        FormPair pair = formManager.createNewPair(student, manager, list, period);
        assertNotNull(pair);
        assertNotNull(pair.getStudentsForm().getTimestamp());
        assertNotNull(pair.getManagersForm().getTimestamp());
        assertNull(pair.getMailTimestamp());
        formManager.setFormPairTimestamp(pair);
        FormPair result = formManager.getFormPairById(pair.getId());
        assertNotNull(result.getMailTimestamp());
        formManager.forceDeleteFormPair(pair);
        participantManager.deleteParticipant(student);
        participantManager.deleteParticipant(manager);
        featureManager.deleteFeature(feature);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);
    }

    private static Period createSamplePeriod() {
        Period period = new Period();
        period.setDescription("decription");
        period.setNote("note");
        period.setYear(2011);
        return period;
    }

    @Test
    public void testGetAllFormPairs() throws Exception {
        try {
            formManager.getAllFormPairs(null);
            fail("period is null");
        } catch (IllegalArgumentException ex) {
            //OK
        }

        Feature feature = createSampleFeature();
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Manager manager = new Manager();
        manager.setMail("man@org.com");
        manager.setName("manager");
        manager.setRole("manager");
        manager.setOrganization(org);
        participantManager.addParticipant(manager);
        Student student = new Student();
        student.setUco("54434");
        student.setRole("student");
        student.setMail("student@org.com");
        student.setName("student");
        participantManager.addParticipant(student);
        List<Feature> list = new ArrayList<Feature>();
        list.add(feature);

        FormPair pair = formManager.createNewPair(student, manager, list, period);
        assertNotNull(pair);
        List<FormPair> result = formManager.getAllFormPairs();
        assertEquals(1, result.size());
        assertTrue(result.contains(pair));
        result = formManager.getAllFormPairs(period);
        assertEquals(1, result.size());
        assertTrue(result.contains(pair));
        result = formManager.getAllFormPairs(period, org);
        assertEquals(1, result.size());
        assertTrue(result.contains(pair));
        Period period2 = createSamplePeriod();
        periodManager.addPeriod(period2);
        FormPair pair2 = formManager.createNewPair(student, manager, list, period2);
        assertNotNull(pair2);
        result = formManager.getAllFormPairs();
        assertEquals(2, result.size());
        assertTrue(result.contains(pair));
        assertTrue(result.contains(pair2));
        result = formManager.getAllFormPairs(period2, org);
        assertEquals(1, result.size());
        assertTrue(result.contains(pair2));
        result = formManager.getAllFormPairs(period);
        assertEquals(1, result.size());
        assertTrue(result.contains(pair));
        result = formManager.getAllFormPairs(period, org);
        assertEquals(1, result.size());
        assertTrue(result.contains(pair));
        result = formManager.getAllFormPairs(period, null);
        assertEquals(1, result.size());
        assertTrue(result.contains(pair));
        result = formManager.getAllFormPairs(null, org);
        assertEquals(2, result.size());
        assertTrue(result.contains(pair));
        assertTrue(result.contains(pair2));
        result = formManager.getAllFormPairs(null, null);
        assertEquals(2, result.size());
        assertTrue(result.contains(pair));
        assertTrue(result.contains(pair2));

        formManager.forceDeleteFormPair(pair);
        formManager.forceDeleteFormPair(pair2);
        featureManager.deleteFeature(feature);
        participantManager.deleteParticipant(student);
        participantManager.deleteParticipant(manager);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);
        periodManager.deletePeriod(period2);
    }

    @Test
    public void testDeleteFormPair() throws Exception {
        Feature feature = createSampleFeature();
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Manager manager = new Manager();
        manager.setMail("man@org.com");
        manager.setName("manager");
        manager.setRole("manager");
        manager.setOrganization(org);
        participantManager.addParticipant(manager);
        Student student = new Student();
        student.setUco("54434");
        student.setRole("student");
        student.setMail("student@org.com");
        student.setName("student");
        participantManager.addParticipant(student);
        List<Feature> list = new ArrayList<Feature>();
        list.add(feature);
        FormPair pair = formManager.createNewPair(student, manager, list, period);
        assertNotNull(pair);

        try {
            formManager.deleteFormPair(null);
            fail();
        } catch (Exception ex) {
            //OK
        }

        pair = formManager.getFormPairById(pair.getId());
        Form rated = pair.getStudentsForm();
        int val = 5;
        rated.getFeatureRatingList().get(0).setValue(val);
        formManager.saveForm(rated);
        Form result = formManager.getFormByHash(rated.getHash());
        assertEquals(val, result.getFeatureRatingList().get(0).getValue());
        FormPair result1 = formManager.findFormPair(rated);
        assertEquals(pair.getStudentsForm().getOwner().getRole(), result1.getState());
        assertEquals("saved", pair.getStudentsForm().getState());
        assertNotNull(pair.getStudentsForm().getTimestamp());
        rated = pair.getManagersForm();
        int val1 = 8;
        rated.getFeatureRatingList().get(0).setValue(val1);
        formManager.saveForm(rated);
        try {
            formManager.deleteFormPair(pair);
            fail();
        } catch (DataIntegrityViolationException ex) {
            //OK
        }
        formManager.forceDeleteFormPair(pair);
        FormPair resultPair = formManager.getFormPairById(pair.getId());
        assertNull(resultPair);
        pair = formManager.createNewPair(student, manager, list, period);
        assertNotNull(pair);
        formManager.deleteFormPair(pair);
        resultPair = formManager.getFormPairById(pair.getId());
        assertNull(resultPair);
        assertEquals(0, formManager.getAllFormPairs().size());
        featureManager.deleteFeature(feature);
        participantManager.deleteParticipant(student);
        participantManager.deleteParticipant(manager);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);
    }

}

