package cz.muni.fi.test;

import cz.muni.fi.dao.FeatureDao;
import cz.muni.fi.dao.FeatureRateDao;
import cz.muni.fi.dao.PeriodDao;
import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.FeatureRate;
import cz.muni.fi.domain.Period;
import org.junit.After;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.List;

import static org.junit.Assert.*;
import static org.junit.Assert.assertEquals;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 20.8.2011
 */
@RunWith(SpringJUnit4ClassRunner.class)
// ApplicationContext will be loaded from "/applicationContext.xml" and "/applicationContext-test.xml"
// in the root of the classpath
@ContextConfiguration(locations = {"/applicationContext-test.xml"})
public class JpaFeatureDaoTest {

    @After
    public void tearDown() {
        assertEquals(0, featureDao.getAllFeatures().size());
    }

    @Autowired
    private FeatureDao featureDao;

    @Autowired
    private FeatureRateDao featureRateDao;

    @Autowired
    private PeriodDao periodDao;

    @Test
    public void testFindById() throws Exception {
        Feature feature = createSampleFeature();
        featureDao.save(feature);
        assertNotNull(feature.getId());
        assertNotNull(featureDao.findById(feature.getId()));
        featureDao.delete(feature);
    }

    private Feature createSampleFeature() {
        Feature feature = new Feature();
        feature.setHint("hint");
        feature.setManagerQuestion("manager");
        feature.setStudentQuestion("student");
        feature.setName("name");
        return feature;
    }

    @Test
    public void testGetAllFeaturesWithoutRatingInPeriod() {
        Feature feature = createSampleFeature();
        featureDao.save(feature);
        Period period = createSamplePeriod();
        periodDao.save(period);
        FeatureRate rate = new FeatureRate();
        rate.setFeature(feature);
        rate.setPeriod(period);
        featureRateDao.save(rate);
        List<Feature> featureList = featureDao.getAllFeaturesWithoutRateInPeriod(period);
        assertTrue(featureList.isEmpty());
        Feature feature2 = createSampleFeature();
        featureDao.save(feature2);
        featureList = featureDao.getAllFeaturesWithoutRateInPeriod(period);
        assertEquals(1, featureList.size());
        assertTrue(featureList.contains(feature2));
        Feature feature3 = createSampleFeature();
        featureDao.save(feature3);
        featureList = featureDao.getAllFeaturesWithoutRateInPeriod(period);
        assertEquals(2, featureList.size());
        assertTrue(featureList.contains(feature2));
        assertTrue(featureList.contains(feature3));
        featureRateDao.delete(rate);
        featureDao.delete(feature);
        featureDao.delete(feature2);
        featureDao.delete(feature3);
        periodDao.delete(period);
    }

    @Test
    public void testSave() throws Exception {
        Feature feature = createSampleFeature();
        featureDao.save(feature);
        assertNotNull(feature.getId());
        Feature result = featureDao.findById(feature.getId());
        assertEquals(feature, result);
        assertEquals(feature.getHint(), result.getHint());
        assertEquals(feature.getManagerQuestion(), result.getManagerQuestion());
        assertEquals(feature.getStudentQuestion(), result.getStudentQuestion());
        assertEquals(feature.getName(), result.getName());

        featureDao.delete(feature);

    }

    @Test
    public void testUpdate() throws Exception {
        Feature feature = createSampleFeature();
        featureDao.save(feature);
        assertNotNull(feature.getId());
        Feature result = featureDao.findById(feature.getId());
        assertEquals(feature, result);
        feature.setHint("newH");
        feature.setManagerQuestion("newM");
        feature.setStudentQuestion("newS");
        feature.setName("newNAme");
        featureDao.update(feature);
        result = featureDao.findById(feature.getId());
        assertEquals(feature, result);
        assertEquals(feature.getHint(), result.getHint());
        assertEquals(feature.getManagerQuestion(), result.getManagerQuestion());
        assertEquals(feature.getStudentQuestion(), result.getStudentQuestion());
        assertEquals(feature.getName(), result.getName());
        featureDao.delete(feature);
    }

    @Test
    public void testDelete() throws Exception {
        Feature feature = createSampleFeature();
        featureDao.save(feature);
        assertNotNull(feature.getId());
        Feature feature2 = createSampleFeature();
        featureDao.save(feature2);
        assertEquals(2, featureDao.getAllFeatures().size());
        Feature result = featureDao.findById(feature.getId());
        assertNotNull(result);
        featureDao.delete(feature);
        result = featureDao.findById(feature.getId());
        assertNull(result);
        assertEquals(1, featureDao.getAllFeatures().size());
        result = featureDao.findById(feature2.getId());
        assertNotNull(result);
        featureDao.delete(feature2);
        result = featureDao.findById(feature2.getId());
        assertNull(result);
        assertEquals(0, featureDao.getAllFeatures().size());

    }

    @Test
    public void testGetAllFeatures() throws Exception {
        Feature feature = createSampleFeature();
        featureDao.save(feature);
        Feature feature2 = createSampleFeature();
        featureDao.save(feature2);
        List<Feature> result = featureDao.getAllFeatures();
        assertEquals(2, result.size());
        assertTrue(result.contains(feature));
        assertTrue(result.contains(feature2));
        assertEquals(feature, result.get(0));
        assertEquals(feature2, result.get(1));
        assertEquals(feature.getHint(), result.get(0).getHint());
        assertEquals(feature.getName(), result.get(0).getName());
        assertEquals(feature.getManagerQuestion(), result.get(0).getManagerQuestion());
        assertEquals(feature.getStudentQuestion(), result.get(0).getStudentQuestion());
        featureDao.delete(feature);
        featureDao.delete(feature2);
    }

    private static Period createSamplePeriod() {
        Period period = new Period();
        period.setDescription("decription");
        period.setNote("note");
        period.setYear(2011);
        return period;
    }
}
