package cz.muni.fi.test;

import cz.muni.fi.dao.FeatureRateDao;
import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.FeatureRate;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Period;
import cz.muni.fi.service.*;
import org.junit.After;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.List;

import static junit.framework.Assert.*;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 7.11.11
 */

@RunWith(SpringJUnit4ClassRunner.class)
// ApplicationContext will be loaded from "/applicationContext.xml" and "/applicationContext-test.xml"
// in the root of the classpath
@ContextConfiguration(locations = {"/applicationContext-test.xml"})
public class OrganizationManagerTest {

    @Autowired
    private OrganizationManager organizationManager;

    @Autowired
    private FeatureManager featureManager;

    @Autowired
    private PeriodManager periodManager;

    @Autowired
    private PlainFeatureRateManager plainFeatureRateManager;

    @Autowired
    private FeatureRateDao featureRateDao;


    @After
    public void tearDown() {
        assertEquals(0, featureManager.getAllFeatures().size());
        assertEquals(0,organizationManager.getAllOrganizations().size());
    }

    @Test
    public void testGetAllActiveOrganizations() throws Exception {
        Organization org = new Organization();
        org.setName("abc");
        org.setActive(true);
        org.setIdNumber("1");
        List<Organization> result = organizationManager.getAllActiveOrganizations();
        assertEquals(0, result.size());
        organizationManager.addOrganization(org);
        result = organizationManager.getAllActiveOrganizations();
        assertEquals(1, result.size());
        Organization org2 = new Organization();
        org2.setName("dabc");
        org2.setIdNumber("2");
        organizationManager.addOrganization(org2);
        result = organizationManager.getAllActiveOrganizations();
        assertEquals(1, result.size());
        Organization org3 = new Organization();
        org3.setName("dabc");
        org3.setActive(true);
        org3.setIdNumber("3");
        organizationManager.addOrganization(org3);
        result = organizationManager.getAllActiveOrganizations();
        assertEquals(2, result.size());
        organizationManager.deleteOrganization(org);
        organizationManager.deleteOrganization(org2);
        organizationManager.deleteOrganization(org3);
    }

    @Test
    public void testGetAllOrganizations() throws Exception {
        Organization org = new Organization();
        org.setName("abc");
        org.setActive(true);
        org.setIdNumber("1");
        List<Organization> result = organizationManager.getAllOrganizations();
        assertEquals(0, result.size());
        organizationManager.addOrganization(org);
        result = organizationManager.getAllOrganizations();
        assertEquals(1, result.size());
        Organization org2 = new Organization();
        org2.setName("dabc");
        org2.setIdNumber("14");
        organizationManager.addOrganization(org2);
        result = organizationManager.getAllOrganizations();
        assertEquals(2, result.size());
        Organization org3 = new Organization();
        org3.setName("dabc");
        org3.setActive(true);
        org3.setIdNumber("143");
        organizationManager.addOrganization(org3);
        result = organizationManager.getAllOrganizations();
        assertEquals(3, result.size());
        organizationManager.deleteOrganization(org);
        organizationManager.deleteOrganization(org2);
        organizationManager.deleteOrganization(org3);
    }

    @Test
    public void testAddOrganization() throws Exception{
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("1");
        Period period = createSamplePeriod();
        Feature feature = createSampleFeature();
        try {
            organizationManager.addOrganization(null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        org.setIdNumber(null);
        try {
            organizationManager.addOrganization(org);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        org.setIdNumber("1");
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        organizationManager.addOrganization(org);
        assertNotNull(org.getId());
        Organization result = organizationManager.getById(org.getId());
        assertEquals(org.getName(), result.getName());
        assertEquals(org, result);
        List<FeatureRate> list = plainFeatureRateManager.getAggregatedFeatureRates(period);
        assertEquals(1, list.size());
        list = plainFeatureRateManager.getAllFeatureRates(period, org);
        assertEquals(1, list.size());
        FeatureRate rate = plainFeatureRateManager.getRateForFeature(feature);
        assertNotNull(rate);
        assertEquals(0, rate.getValue());
        featureManager.deleteFeature(feature);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);
    }

    @Test
    public void testDeleteOrganization() throws Exception {
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("1");
        Period period = createSamplePeriod();
        Feature feature = createSampleFeature();

        try {
            organizationManager.deleteOrganization(null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        try {
            organizationManager.deleteOrganization(org);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        organizationManager.addOrganization(org);
        Organization result = organizationManager.getById(org.getId());
        assertNotNull(result);
        List<Organization> orgs = organizationManager.getAllOrganizations();
        assertEquals(1, orgs.size());
        FeatureRate rate = plainFeatureRateManager.getRateForFeature(feature, org);
        assertNotNull(rate);
        organizationManager.deleteOrganization(org);
        result = organizationManager.getById(org.getId());
        assertNull(result);
        rate = plainFeatureRateManager.getRateForFeature(feature, org);
        assertNull(rate);
        orgs = organizationManager.getAllOrganizations();
        assertEquals(0, orgs.size());
        periodManager.deletePeriod(period);
        featureManager.deleteFeature(feature);

    }

    @Test
    public void testGetByIdNumber() throws Exception{
        try {
            organizationManager.getByIdNumber(null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("1");
        Period period = createSamplePeriod();
        Feature feature = createSampleFeature();
        Organization result = organizationManager.getByIdNumber("45");
        assertNull(result);
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        organizationManager.addOrganization(org);
        result = organizationManager.getByIdNumber(org.getIdNumber());
        assertNotNull(result);
        assertEquals(org.getName(), result.getName());
        featureManager.deleteFeature(feature);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);
    }

    @Test
    public void testGetById() throws Exception{
        try {
            organizationManager.getById(null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("1");
        Period period = createSamplePeriod();
        Feature feature = createSampleFeature();
        Organization result = organizationManager.getById(34L);
        assertNull(result);
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        organizationManager.addOrganization(org);
        assertNotNull(org.getId());
        result = organizationManager.getById(org.getId());
        assertNotNull(result);
        assertEquals(org.getName(), result.getName());
        featureManager.deleteFeature(feature);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);
    }


    @Test
    public void testUpdate() throws Exception{
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("1");
        Period period = createSamplePeriod();
        Feature feature = createSampleFeature();
        try {
            organizationManager.updateOrganization(null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        try {
            organizationManager.updateOrganization(org);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        organizationManager.addOrganization(org);
        assertNotNull(org.getId());
        Organization result = organizationManager.getById(org.getId());

        org.setName("newOrg");
        org.setIdNumber("155");
        organizationManager.updateOrganization(org);

        result = organizationManager.getById(org.getId());
        assertEquals(org.getName(), result.getName());
        assertEquals(org.getIdNumber(), result.getIdNumber());
        assertEquals(org, result);

        featureManager.deleteFeature(feature);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);

    }

    @Test
    public void testCanBeDeleted() throws Exception{
        Feature feature = createSampleFeature();
        Period period = createSamplePeriod();
        Organization org = new Organization();
        org.setName("Org");
        org.setIdNumber("1");
        boolean result = organizationManager.canBeDeleted(null);
        assertFalse(result);
        result = organizationManager.canBeDeleted(org);
        assertFalse(result);
        org.setId(20L);
        result = organizationManager.canBeDeleted(org);
        assertFalse(result);
        org.setId(null);
        organizationManager.addOrganization(org);
        featureManager.addFeature(feature);
        result = organizationManager.canBeDeleted(org);
        assertTrue(result);
        periodManager.addPeriod(period);
        result = organizationManager.canBeDeleted(org);
        assertTrue(result);
        FeatureRate rate = featureRateDao.findByFeature(feature, period, org);
        rate.setResponses(2);
        featureRateDao.update(rate);
        result = organizationManager.canBeDeleted(org);
        assertFalse(result);
        featureManager.deleteFeature(feature);
        result = organizationManager.canBeDeleted(org);
        assertTrue(result);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);


    }

    private static Feature createSampleFeature() {
        Feature feature = new Feature();
        feature.setHint("hint");
        feature.setManagerQuestion("manager");
        feature.setStudentQuestion("student");
        feature.setName("name");
        return feature;
    }

    private static Period createSamplePeriod() {
        Period period = new Period();
        period.setDescription("decription");
        period.setNote("note");
        period.setYear(2011);
        return period;
    }

}
