package cz.muni.fi.test;

import cz.muni.fi.dao.DisagreedFeatureRateDao;
import cz.muni.fi.domain.*;
import cz.muni.fi.exception.AlreadyPresentException;
import cz.muni.fi.service.FeatureManager;
import cz.muni.fi.service.OrganizationManager;
import cz.muni.fi.service.ParticipantManager;
import org.junit.After;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.List;

import static junit.framework.Assert.*;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 14.11.11
 */
@RunWith(SpringJUnit4ClassRunner.class)
// ApplicationContext will be loaded from "/applicationContext.xml" and "/applicationContext-test.xml"
// in the root of the classpath
@ContextConfiguration(locations = {"/applicationContext-test.xml"})
public class ParticipantManagerTest {

    @Autowired
    private ParticipantManager participantManager;
    @Autowired
    private OrganizationManager organizationManager;

    @Autowired
    private DisagreedFeatureRateDao disagreedFeatureRateDao;

    @Autowired
    private FeatureManager featureManager;

    @After
    public void tearDown() {
        List<Organization> list = organizationManager.getAllOrganizations();
        assertTrue(list.isEmpty());
    }

    @Test
    public void testGetAllStudents() throws Exception {
        List<Participant> result = participantManager.getAllStudents();
        assertTrue(result.isEmpty());
        Participant student1 = createSampleStudent(1);
        participantManager.addParticipant(student1);
        result = participantManager.getAllStudents();
        assertEquals(1, result.size());
        assertTrue(result.contains(student1));
        Participant student2 = createSampleStudent(2);
        participantManager.addParticipant(student2);
        result = participantManager.getAllStudents();
        assertEquals(2, result.size());
        assertTrue(result.contains(student1));
        assertTrue(result.contains(student2));

        participantManager.deleteParticipant(student1);
        participantManager.deleteParticipant(student2);

    }

    @Test
    public void testGetAllManagers() throws Exception {
        List<Participant> result = participantManager.getAllManagers();
        assertTrue(result.isEmpty());
        Participant manager1 = createSampleManager(1);
        organizationManager.addOrganization(((Manager) manager1).getOrganization());
        participantManager.addParticipant(manager1);
        result = participantManager.getAllManagers();
        assertEquals(1, result.size());
        assertTrue(result.contains(manager1));
        Participant manager2 = createSampleManager(2);
        organizationManager.addOrganization(((Manager) manager2).getOrganization());

        participantManager.addParticipant(manager2);
        result = participantManager.getAllManagers();
        assertEquals(2, result.size());
        assertTrue(result.contains(manager1));
        assertTrue(result.contains(manager2));

        participantManager.deleteParticipant(manager1);
        participantManager.deleteParticipant(manager2);
        organizationManager.deleteOrganization(((Manager) manager1).getOrganization());
        organizationManager.deleteOrganization(((Manager) manager2).getOrganization());

    }

    @Test
    public void testGetById() throws Exception {
        Participant part = createSampleStudent(1);
        try {
            participantManager.getById(null);
            fail("getting null");
        } catch (IllegalArgumentException ex) {
            //OK
        }

        Participant result = participantManager.getById(45L);
        assertNull(result);
        participantManager.addParticipant(part);
        assertNotNull(part.getId());
        result = participantManager.getById(part.getId());
        assertEquals(part, result);
        assertEquals(part.getName(), result.getName());
        assertEquals(part.getRole(), result.getRole());
        assertEquals(part.getSurname(), result.getSurname());
        participantManager.deleteParticipant(part);
    }

    @Test
    public void testAddParticipant() throws Exception {
        Participant part = createSampleStudent(1);
        try {
            participantManager.addParticipant(null);
            fail("adding null");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        participantManager.addParticipant(part);
        Participant result = participantManager.getById(part.getId());
        assertEquals(part, result);
        assertEquals(part.getName(), result.getName());
        assertEquals(part.getSurname(), result.getSurname());
        assertEquals(part.getRole(), result.getRole());
        assertTrue(result instanceof Student);
        Participant part1 = createSampleStudent(1);
        try {
            participantManager.addParticipant(part1);
            fail();
        } catch (AlreadyPresentException ex) {
            //OK
        }
        String ucoPresent = ((Student) part1).getUco();
        part1 = createSampleManager(1);
        try {
            participantManager.addParticipant(part1);
            fail("organization not known");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        organizationManager.addOrganization(((Manager) part1).getOrganization());
        participantManager.addParticipant(part1);
        result = participantManager.getById(part1.getId());
        assertEquals(part1, result);
        assertEquals(part1.getName(), result.getName());
        assertEquals(part1.getSurname(), result.getSurname());
        assertEquals(part1.getRole(), result.getRole());
        assertTrue(result instanceof Manager);
        Manager manager = (Manager) result;
        assertEquals(((Manager) part1).getOrganization(), manager.getOrganization());
        assertEquals(((Manager) part1).getOrganization().getName(), manager.getOrganization().getName());
        assertEquals(((Manager) part1).getOrganization().isActive(), manager.getOrganization().isActive());
        Participant part3 = createSampleStudent(30);
        ((Student) part3).setUco(ucoPresent);
        try {
            participantManager.addParticipant(part3);
            fail();
        } catch (AlreadyPresentException ex) {
            //OK
        }
        ((Student) part3).setUco("1234");
        part3.setMail(part.getMail());
        try {
            participantManager.addParticipant(part3);
            fail();
        } catch (AlreadyPresentException ex) {
            //OK
        }

        participantManager.deleteParticipant(manager);
        organizationManager.deleteOrganization(manager.getOrganization());
        participantManager.deleteParticipant(part);
    }

    @Test
    public void testDeleteParticipant() throws Exception {
        Participant part = createSampleStudent(1);
        participantManager.addParticipant(part);
        List<Participant> list = participantManager.getAllStudents();
        assertEquals(1, list.size());
        participantManager.deleteParticipant(part);
        list = participantManager.getAllStudents();
        assertEquals(0, list.size());
        list = participantManager.getAllManagers();
        assertEquals(0, list.size());
        Manager manager = createSampleManager(1);
        organizationManager.addOrganization(manager.getOrganization());
        participantManager.addParticipant(manager);
        list = participantManager.getAllManagers();
        assertEquals(1, list.size());
        participantManager.deleteParticipant(manager);
        list = participantManager.getAllManagers();
        assertEquals(0, list.size());
        assertNotNull(organizationManager.getById(manager.getOrganization().getId()));
        organizationManager.deleteOrganization(manager.getOrganization());
    }

    @Test
    public void testCanParticipantBeDeleted() throws Exception {
        Participant partip = createSampleStudent(1);
        participantManager.addParticipant(partip);
        assertTrue(participantManager.canParticipantBeDeleted(partip));
        participantManager.deleteParticipant(partip);
        //other beahavior tested in formManagerTest
    }

    @Test
    public void testUpdateParticipant() throws Exception {
        Participant part = createSampleStudent(1);
        try {
            participantManager.updateParticipant(null);
            fail("updating null");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        try {
            participantManager.updateParticipant(part);
            fail("updating not present");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        participantManager.addParticipant(part);
        part.setName("new name");
        part.setSurname("new surname");
        participantManager.updateParticipant(part);
        Participant result = participantManager.getById(part.getId());
        assertEquals(part, result);
        assertEquals(part.getName(), result.getName());
        assertEquals(part.getSurname(), result.getSurname());
        assertEquals(part.getRole(), result.getRole());
        assertTrue(result instanceof Student);
        Participant part1 = createSampleStudent(2);
        participantManager.addParticipant(part1);
        ((Student) part1).setUco(((Student) part).getUco());
        try {
            participantManager.updateParticipant(part1);
            fail();
        } catch (AlreadyPresentException ex) {
            //OK
        }
        participantManager.deleteParticipant(part);
        participantManager.deleteParticipant(part1);
        part1 = createSampleManager(1);
        Organization org = ((Manager) part1).getOrganization();
        organizationManager.addOrganization(((Manager) part1).getOrganization());
        participantManager.addParticipant(part1);
        Organization org2 = new Organization();
        org2.setName("nameOrg");
        org2.setIdNumber("34567");
        org2.setActive(true);
        ((Manager) part1).setOrganization(org2);
        try {
            participantManager.updateParticipant(part);
            fail("updating not present new organization");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        organizationManager.addOrganization(org2);
        part1.setName("new name EE");
        participantManager.updateParticipant(part1);
        result = participantManager.getById(part1.getId());
        assertEquals(part1, result);
        assertEquals(part1.getName(), result.getName());
        assertEquals(part1.getSurname(), result.getSurname());
        assertEquals(part1.getRole(), result.getRole());
        assertTrue(result instanceof Manager);
        Manager manager = (Manager) result;
        assertEquals(org2, manager.getOrganization());
        assertEquals(org2.getName(), manager.getOrganization().getName());
        assertEquals(org2.isActive(), manager.getOrganization().isActive());
        assertEquals(org2.getIdNumber(), manager.getOrganization().getIdNumber());

        participantManager.deleteParticipant(manager);
        organizationManager.deleteOrganization(manager.getOrganization());
        organizationManager.deleteOrganization(org);
    }

    @Test
    public void testGetByUco() throws Exception {
        Participant student = createSampleStudent(1);
        try {
            participantManager.getByUco(null);
        } catch (IllegalArgumentException ex) {
            //OK
        }
        Participant result = participantManager.getByUco("3456");
        assertNull(result);
        String uco = ((Student) student).getUco();
        participantManager.addParticipant(student);
        result = participantManager.getByUco(uco);
        assertEquals(student, result);
        assertEquals(student.getMail(), result.getMail());
        assertEquals(student.getFullName(), result.getFullName());
        assertEquals(student.getName(), result.getName());
        assertEquals(student.getSurname(), result.getSurname());
        participantManager.deleteParticipant(student);


    }

    @Test
    public void testGetByEmail() throws Exception {
        Participant student = createSampleStudent(1);
        try {
            participantManager.getByEmail(null);
        } catch (IllegalArgumentException ex) {
            //OK
        }
        Participant result = participantManager.getByEmail("3456");
        assertNull(result);
        String mail = student.getMail();
        participantManager.addParticipant(student);
        result = participantManager.getByEmail(mail);
        assertEquals(student, result);
        assertEquals(student.getMail(), result.getMail());
        assertEquals(student.getFullName(), result.getFullName());
        assertEquals(student.getName(), result.getName());
        assertEquals(student.getSurname(), result.getSurname());
        participantManager.deleteParticipant(student);
    }

    private Student createSampleStudent(int i) {
        Student student = new Student();
        student.setMail(i + "studentQ@org.com");
        student.setRole("student");
        student.setName("student");
        student.setSurname("green");
        student.setUco("23344" + i);
        return student;
    }

    @Test
    public void testGetAllManagersFor() throws Exception {
        Manager man1 = createSampleManager(1);
        Organization org = man1.getOrganization();
        organizationManager.addOrganization(org);
        participantManager.addParticipant(man1);
        List<Participant> result = participantManager.getAllManagersFor(org);
        assertEquals(1, result.size());
        assertTrue(result.contains(man1));
        Manager man2 = createSampleManager(12);
        man2.setOrganization(org);
        participantManager.addParticipant(man2);
        result = participantManager.getAllManagersFor(org);
        assertEquals(2, result.size());
        assertTrue(result.contains(man1));
        assertTrue(result.contains(man2));
        Manager man3 = createSampleManager(3);
        organizationManager.addOrganization(man3.getOrganization());
        participantManager.addParticipant(man3);
        result = participantManager.getAllManagersFor(org);
        assertEquals(2, result.size());
        assertTrue(result.contains(man1));
        assertTrue(result.contains(man2));
        participantManager.deleteParticipant(man1);
        participantManager.deleteParticipant(man2);
        participantManager.deleteParticipant(man3);
        organizationManager.deleteOrganization(org);
        organizationManager.deleteOrganization(man3.getOrganization());
    }

    @Test
    public void testGetDisagreementList() throws Exception {
        Participant part = createSampleStudent(1);
        Feature feature = createSampleFeature();
        try {
            featureManager.getDisagreementList((Feature) null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        try {
            participantManager.getDisagreementList((Participant) null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        try {
            featureManager.getDisagreementList(feature);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        try {
            participantManager.getDisagreementList(part);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        featureManager.addFeature(feature);
        participantManager.addParticipant(part);
        List<DisagreedFeatureRate> result = participantManager.getDisagreementList(part);
        assertEquals(0, result.size());
        result = featureManager.getDisagreementList(feature);
        assertEquals(0, result.size());
        DisagreedFeatureRate fr = new DisagreedFeatureRate();
        fr.setFeature(feature);
        fr.setStudent(part);
        disagreedFeatureRateDao.save(fr);
        result = participantManager.getDisagreementList(part);
        assertEquals(1, result.size());
        result = featureManager.getDisagreementList(feature);
        assertEquals(1, result.size());
        Feature feature1 = createSampleFeature();
        featureManager.addFeature(feature1);
        DisagreedFeatureRate fr1 = new DisagreedFeatureRate();
        fr1.setFeature(feature1);
        fr1.setStudent(part);
        disagreedFeatureRateDao.save(fr1);
        result = participantManager.getDisagreementList(part);
        assertEquals(2, result.size());
        result = featureManager.getDisagreementList(feature);
        assertEquals(1, result.size());
        disagreedFeatureRateDao.delete(fr);
        featureManager.deleteFeature(feature);
        try {
            featureManager.getDisagreementList(feature);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        DisagreedFeatureRate fr2 = new DisagreedFeatureRate();
        fr2.setFeature(feature1);
        Participant manager = createSampleManager(1);
        organizationManager.addOrganization(((Manager)manager).getOrganization());
        participantManager.addParticipant(manager);
        fr2.setManager(manager);
        fr2.setStudent(part);
        disagreedFeatureRateDao.save(fr2);
        result = participantManager.getDisagreementList(part);
        assertEquals(2, result.size());
        result = featureManager.getDisagreementList(feature1);
        assertEquals(2, result.size());

        disagreedFeatureRateDao.delete(fr1);
        disagreedFeatureRateDao.delete(fr2);
        featureManager.deleteFeature(feature1);
        participantManager.deleteParticipant(part);
        participantManager.deleteParticipant(manager);
        organizationManager.deleteOrganization(((Manager)manager).getOrganization());
    }

    private Manager createSampleManager(int i) {
        Manager manager = new Manager();
        Organization org = new Organization();
        org.setName("org");
        org.setActive(true);
        org.setIdNumber("344" + i);
        manager.setMail(i + "managerQ@org.com");
        manager.setName("manager");
        manager.setSurname("surname");
        manager.setRole("manager");
        manager.setOrganization(org);
        return manager;
    }

    private static Feature createSampleFeature() {
        Feature feature = new Feature();
        feature.setHint("hint");
        feature.setManagerQuestion("manager");
        feature.setStudentQuestion("student");
        feature.setName("name");
        return feature;
    }
}
