package cz.muni.fi.test;

import cz.muni.fi.dao.FeatureRateDao;
import cz.muni.fi.domain.*;
import cz.muni.fi.exception.DataIntegrityViolationException;
import cz.muni.fi.service.*;
import org.junit.After;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.ArrayList;
import java.util.List;

import static junit.framework.Assert.assertFalse;
import static junit.framework.Assert.assertNull;
import static junit.framework.Assert.fail;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 21.11.11
 */
@RunWith(SpringJUnit4ClassRunner.class)
// ApplicationContext will be loaded from "/applicationContext.xml" and "/applicationContext-test.xml"
// in the root of the classpath
@ContextConfiguration(locations = {"/applicationContext-test.xml"})
public class PeriodManagerTest {

    @Autowired
    private PeriodManager periodManager;

    @Autowired
    private PlainFeatureRateManager plainFeatureRateManager;

    @Autowired
    private FeatureManager featureManager;

    @Autowired
    private UserManager userManager;

    @Autowired
    private FeatureRateDao featureRateDao;

    @Autowired
    private OrganizationManager organizationManager;

    @Autowired
    private FormManager formManager;

    @Autowired
    private ParticipantManager participantManager;

    @After
    public void tearDown() {
        assertEquals(0, featureManager.getAllFeatures().size());
        assertEquals(0, organizationManager.getAllOrganizations().size());
    }

    @Test
    public void testAddPeriod() throws Exception {
        Period period = createSamplePeriod();
        Feature feature = createSampleFeature();
        try {
            periodManager.addPeriod(null);
            fail("can't add null");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        periodManager.addPeriod(period);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        assertNotNull(period.getId());
        Period result = periodManager.getPeriodById(period.getId());
        assertEquals(period, result);
        assertEquals(period.getDescription(), result.getDescription());
        assertEquals(period.getNote(), result.getNote());
        assertEquals(period.getYear(), result.getYear());
        Period period2 = createSamplePeriod();
        featureManager.addFeature(feature);
        periodManager.addPeriod(period2);
        FeatureRate rate = plainFeatureRateManager.getRateForFeature(feature, period);
        assertNotNull(rate);
        assertEquals(feature, rate.getFeature());
        assertEquals(period, rate.getPeriod());
        assertEquals(0, rate.getResponses());
        assertEquals(0, rate.getSum());
        assertEquals(0, rate.getValue());

        rate = plainFeatureRateManager.getRateForFeature(feature, period, org);
        assertNotNull(rate);
        assertEquals(feature, rate.getFeature());
        assertEquals(period, rate.getPeriod());
        assertEquals(0, rate.getResponses());
        assertEquals(0, rate.getSum());
        assertEquals(0, rate.getValue());

        rate = plainFeatureRateManager.getRateForFeature(feature, period2);
        assertNotNull(rate);
        assertEquals(feature, rate.getFeature());
        assertEquals(period2, rate.getPeriod());
        assertEquals(0, rate.getResponses());
        assertEquals(0, rate.getSum());
        assertEquals(0, rate.getValue());

        rate = plainFeatureRateManager.getRateForFeature(feature, period2, org);
        assertNotNull(rate);
        assertEquals(feature, rate.getFeature());
        assertEquals(period2, rate.getPeriod());
        assertEquals(0, rate.getResponses());
        assertEquals(0, rate.getSum());
        assertEquals(0, rate.getValue());

        featureManager.deleteFeature(feature);
        periodManager.deletePeriod(period);
        periodManager.deletePeriod(period2);
        organizationManager.deleteOrganization(org);
    }

    @Test
    public void testUpdatePeriod() throws Exception {
        try {
            periodManager.updatePeriod(null);
            fail("update null");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        Period period = createSamplePeriod();
        try {
            periodManager.updatePeriod(period);
            fail("can't update non present period");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        periodManager.addPeriod(period);
        period.setDescription("newDescription");
        periodManager.updatePeriod(period);
        Period result = periodManager.getPeriodById(period.getId());
        assertEquals("newDescription", result.getDescription());
        assertEquals(period, result);
        assertEquals(period.getDescription(), result.getDescription());
        assertEquals(period.getNote(), result.getNote());
        assertEquals(period.getYear(), result.getYear());
        periodManager.deletePeriod(period);
    }

    @Test
    public void testGetAllPeriods() throws Exception {
        Period period = createSamplePeriod();
        List<Period> result = periodManager.getAllPeriods();
        assertEquals(0, result.size());
        periodManager.addPeriod(period);
        result = periodManager.getAllPeriods();
        assertEquals(1, result.size());
        assertTrue(result.contains(period));
        Period period2 = createSamplePeriod();
        periodManager.addPeriod(period2);
        result = periodManager.getAllPeriods();
        assertEquals(2, result.size());
        assertTrue(result.contains(period));
        assertTrue(result.contains(period2));
        Period period3 = createSamplePeriod();
        periodManager.addPeriod(period3);
        result = periodManager.getAllPeriods();
        assertEquals(3, result.size());
        assertTrue(result.contains(period));
        assertTrue(result.contains(period2));
        assertTrue(result.contains(period3));
        periodManager.deletePeriod(period);
        periodManager.deletePeriod(period2);
        periodManager.deletePeriod(period3);

    }

    @Test
    public void testGetCurrentPeriod() throws Exception {
        try {
            periodManager.getCurrentPeriod();
            fail("no current period");
        } catch (DataIntegrityViolationException ex) {
            //OK
        }
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        Period result = periodManager.getCurrentPeriod();
        assertEquals(period, result);
        assertEquals(period.getDescription(), result.getDescription());
        assertEquals(period.getNote(), result.getNote());
        assertEquals(period.getYear(), result.getYear());
        Period period2 = createSamplePeriod();
        period2.setDescription("period 2");
        periodManager.addPeriod(period2);
        result = periodManager.getCurrentPeriod();
        assertEquals(period2, result);
        assertEquals(period2.getDescription(), result.getDescription());
        assertEquals(period2.getNote(), result.getNote());
        assertEquals(period2.getYear(), result.getYear());
        Period period3 = createSamplePeriod();
        period2.setDescription("period 3");
        periodManager.addPeriod(period3);
        result = periodManager.getCurrentPeriod();
        assertEquals(period3, result);
        periodManager.deletePeriod(period);
        periodManager.deletePeriod(period2);
        periodManager.deletePeriod(period3);


    }

    @Test
    public void testDeletePeriod() throws Exception {
        try {
            periodManager.deletePeriod(null);
            fail("can't delete null");
        } catch (IllegalArgumentException ex) {
            //OK
        }

        Period period = createSamplePeriod();
        try {
            periodManager.deletePeriod(period);
            fail("can't delete non present period");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        periodManager.addPeriod(period);
        List<Period> result = periodManager.getAllPeriods();
        assertEquals(1, result.size());
        periodManager.deletePeriod(period);
        result = periodManager.getAllPeriods();
        assertEquals(0, result.size());
        period = createSamplePeriod();
        periodManager.addPeriod(period);
        User user = new User();
        user.setEmail("user@mail.com");
        user.setPassword("passw");
        user.setName("User");
        user.setUserName("user");
        user.setSelectedPeriod(period);
        userManager.addUser(user);
        Feature feature = new Feature();
        feature.setName("feature");
        featureManager.addFeature(feature);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        FeatureRate rate = plainFeatureRateManager.getRateForFeature(feature, period);
        assertNotNull(rate);
        periodManager.deletePeriod(period);
        User uResult = userManager.getById(user.getUserName());
        assertNull(uResult.getSelectedPeriod());
        rate = plainFeatureRateManager.getRateForFeature(feature, period);
        assertNull(rate);
        userManager.deleteUser(user);
        featureManager.deleteFeature(feature);
        organizationManager.deleteOrganization(org);
    }

    @Test
    public void testCanBeDeleted() throws Exception {
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        assertTrue(periodManager.canPeriodBeDeleted(period));

        Feature feature = new Feature();
        feature.setName("feature");
        featureManager.addFeature(feature);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        FeatureRate rate = plainFeatureRateManager.getRateForFeature(feature, period, org);
        assertNotNull(rate);
        assertTrue(periodManager.canPeriodBeDeleted(period));
        rate.setResponses(1);
        rate.setSum(2);
        featureRateDao.update(rate);
        assertFalse(periodManager.canPeriodBeDeleted(period));
        rate.setResponses(0);
        rate.setSum(0);
        featureRateDao.update(rate);
        assertTrue(periodManager.canPeriodBeDeleted(period));
        periodManager.deletePeriod(period);
        rate = plainFeatureRateManager.getRateForFeature(feature, period);
        assertNull(rate);
        featureManager.deleteFeature(feature);
        organizationManager.deleteOrganization(org);
    }


    @Test
    public void testGetPeriodById() {
        try {
            periodManager.getPeriodById(null);
            fail("id is null");
        } catch (IllegalArgumentException ex) {
            //OK
        }
        Period result = periodManager.getPeriodById(45L);
        assertNull(result);
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        result = periodManager.getPeriodById(period.getId());
        assertEquals(period, result);
        periodManager.deletePeriod(period);
    }

    @Test
    public void testGetNumberOfFormsForPeriod() throws Exception {
        try {
            periodManager.getNumberOfFormsForPeriod(null);
            fail("period is null");
        } catch (IllegalArgumentException ex) {
            //OK
        }

        Feature feature = createSampleFeature();
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        featureManager.addFeature(feature);
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Manager manager = new Manager();
        manager.setMail("man@org.com");
        manager.setName("manager");
        manager.setRole("manager");
        manager.setOrganization(org);
        participantManager.addParticipant(manager);
        Student student = new Student();
        student.setUco("54434");
        student.setRole("student");
        student.setMail("student@org.com");
        student.setName("student");
        participantManager.addParticipant(student);
        List<Feature> list = new ArrayList<Feature>();
        list.add(feature);

        FormPair pair = formManager.createNewPair(student, manager, list, period);
        assertNotNull(pair);
        assertEquals(2, periodManager.getNumberOfFormsForPeriod(period));
        FormPair pair2 = formManager.createNewPair(student, manager, list, period);
        assertNotNull(pair2);
        assertEquals(4, periodManager.getNumberOfFormsForPeriod(period));

        formManager.deleteFormPair(pair);
        formManager.deleteFormPair(pair2);
        featureManager.deleteFeature(feature);
        participantManager.deleteParticipant(student);
        participantManager.deleteParticipant(manager);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);


    }

    private static Period createSamplePeriod() {
        Period period = new Period();
        period.setDescription("decription");
        period.setNote("note");
        period.setYear(2011);
        return period;
    }

    private static Feature createSampleFeature() {
        Feature feature = new Feature();
        feature.setHint("hint");
        feature.setManagerQuestion("manager");
        feature.setStudentQuestion("student");
        feature.setName("name");
        return feature;
    }
}
