package cz.muni.fi.test;

import cz.muni.fi.domain.Authority;
import cz.muni.fi.domain.Period;
import cz.muni.fi.domain.User;
import cz.muni.fi.service.PeriodManager;
import cz.muni.fi.service.UserManager;
import org.junit.After;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.ArrayList;
import java.util.List;

import static junit.framework.Assert.*;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 26.10.11
 */
@RunWith(SpringJUnit4ClassRunner.class)
// ApplicationContext will be loaded from "/applicationContext.xml" and "/applicationContext-test.xml"
// in the root of the classpath
@ContextConfiguration(locations = {"/applicationContext-test.xml"})
public class UserManagerTest {

    @Autowired
    private UserManager userManager;


    @Autowired
    private PeriodManager periodManager;

    @After
    public void tearDown() {
        List<User> result = userManager.getAllUsers();
        assertTrue(result.isEmpty());
    }

    @Test
    public void testGetAllUsers() throws Exception {
        List<User> result = userManager.getAllUsers();
        assertTrue(result.isEmpty());
        User user = createSampleUser(1);
        userManager.addUser(user);
        result = userManager.getAllUsers();
        assertEquals(1, result.size());
        assertTrue(result.contains(user));
        User user2 = createSampleUser(2);
        userManager.addUser(user2);
        result = userManager.getAllUsers();
        assertEquals(2, result.size());
        assertTrue(result.contains(user));
        assertTrue(result.contains(user2));
        User user3 = createSampleUser(3);
        userManager.addUser(user3);
        result = userManager.getAllUsers();
        assertEquals(3, result.size());
        assertTrue(result.contains(user));
        assertTrue(result.contains(user2));
        assertTrue(result.contains(user3));
        userManager.deleteUser(user);
        userManager.deleteUser(user2);
        userManager.deleteUser(user3);
    }

    @Test
    public void testGetAllUsersWithSelectedPeriod() throws Exception {
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        User user = createSampleUser(1);
        user.setSelectedPeriod(period);
        userManager.addUser(user);
        List<User> result = userManager.getAllUsersWithSelectedPeriod(period);
        assertEquals(1, result.size());
        assertTrue(result.contains(user));
        User user1 = createSampleUser(2);
        userManager.addUser(user1);
        result = userManager.getAllUsersWithSelectedPeriod(period);
        assertEquals(1, result.size());
        assertTrue(result.contains(user));
        User user2 = createSampleUser(3);
        user2.setSelectedPeriod(period);
        userManager.addUser(user2);
        result = userManager.getAllUsersWithSelectedPeriod(period);
        assertEquals(2, result.size());
        assertTrue(result.contains(user));
        assertTrue(result.contains(user2));
        userManager.deleteUser(user1);
        userManager.deleteUser(user2);
        userManager.deleteUser(user);
        periodManager.deletePeriod(period);
    }

    private static Period createSamplePeriod() {
        Period period = new Period();
        period.setDescription("decription");
        period.setNote("note");
        period.setYear(2011);
        return period;
    }

    private User createSampleUser(int i) {
        User user = new User();
        user.setPassword("pass");
        user.setUserName("user" + i);
        user.setEnabled(true);
        user.setName("Uzivatel Zeleny");
        return user;
    }

    @Test
    public void testGetById() throws Exception {
        try {
            userManager.getById(null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        User result = userManager.getById("idik");
        assertNull(result);
        User user = createSampleUser(1);
        userManager.addUser(user);
        result = userManager.getById(user.getUserName());
        assertEquals(user, result);
        assertEquals(user.getPassword(), result.getPassword());
        assertEquals(user.isEnabled(), result.isEnabled());
        assertEquals(user.getName(), result.getName());
        userManager.deleteUser(user);
    }

    @Test
    public void testAddUser() throws Exception {
        try {
            userManager.addUser(null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        User user = createSampleUser(1);
        user.setUserName(null);
        try {
            userManager.addUser(user);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        user.setUserName("");
        try {
            userManager.addUser(user);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        user.setUserName("user1");
        user.setPassword(null);
        try {
            userManager.addUser(user);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        user.setPassword("");
        try {
            userManager.addUser(user);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        user.setPassword("pass");


        Authority authority = new Authority();
        authority.setRole("role");
        user.setAuthorities(new ArrayList<Authority>());
        user.getAuthorities().add(authority);
        userManager.addUser(user);
        User result = userManager.getById(user.getUserName());

        assertEquals(user, result);
        assertEquals(user.getPassword(), result.getPassword());
        assertEquals(user.isEnabled(), result.isEnabled());
        assertEquals(user.getUserName(), result.getUserName());
        assertEquals(user.getAuthorities().size(), result.getAuthorities().size());
        assertEquals(1, result.getAuthorities().size());
        assertEquals(user.getAuthorities().get(0), result.getAuthorities().get(0));
        assertEquals(user.getAuthorities().get(0).getRole(), result.getAuthorities().get(0).getRole());
        User user2 = createSampleUser(2);
        Authority au2 = new Authority();
        au2.setRole("role1");
        Authority au3 = new Authority();
        au3.setRole("role2");
        user2.setAuthorities(new ArrayList<Authority>());
        user2.getAuthorities().add(au2);
        user2.getAuthorities().add(au3);
        userManager.addUser(user2);
        result = userManager.getById(user2.getUserName());
        assertEquals(2, result.getAuthorities().size());
        assertTrue(result.getAuthorities().contains(au2));
        assertTrue(result.getAuthorities().contains(au3));
        userManager.deleteUser(user);
        userManager.deleteUser(user2);
    }

    @Test
    public void testDeleteUser() throws Exception {
        try {
            userManager.deleteUser(null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        User user3 = createSampleUser(3);
        try {
            userManager.deleteUser(user3);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        User user1 = createSampleUser(1);
        userManager.addUser(user1);
        User user2 = createSampleUser(2);
        userManager.addUser(user2);
        List<User> result = userManager.getAllUsers();
        assertEquals(2, result.size());
        userManager.deleteUser(user2);
        result = userManager.getAllUsers();
        assertEquals(1, result.size());
        userManager.deleteUser(user1);
        result = userManager.getAllUsers();
        assertEquals(0, result.size());
    }

    @Test
    public void testAddUserAuthority() {
        try {
            userManager.addUserAuthority(null, null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        User user = createSampleUser(1);
        Authority aut = new Authority();
        try {
            userManager.addUserAuthority(user, aut);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        aut.setRole("");
        try {
            userManager.addUserAuthority(user, aut);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        aut.setRole("role");
        userManager.addUser(user);
        try {
            userManager.addUserAuthority(user, null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        try {
            userManager.addUserAuthority(null, aut);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        userManager.addUserAuthority(user, aut);
        try {
            userManager.addUserAuthority(user, aut);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }

        User result = userManager.getById(user.getUserName());
        assertEquals(1, result.getAuthorities().size());
        assertEquals(aut, result.getAuthorities().get(0));
        Authority aut2 = new Authority();
        aut2.setRole("role2");
        userManager.addUserAuthority(user, aut2);
        result = userManager.getById(user.getUserName());
        assertEquals(2, result.getAuthorities().size());
        assertTrue(result.getAuthorities().contains(aut));
        assertTrue(result.getAuthorities().contains(aut2));
        userManager.deleteUser(user);
    }

    @Test
    public void testRemoveUserAuthority() {
        try {
            userManager.removeUserAuthority(null, null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        User user = createSampleUser(1);
        Authority aut = new Authority();
        aut.setRole("role");
        userManager.addUser(user);
        try {
            userManager.removeUserAuthority(user, null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        try {
            userManager.removeUserAuthority(null, aut);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        userManager.addUserAuthority(user, aut);
        Authority aut2 = new Authority();
        aut2.setRole("role2");
        try {
            userManager.removeUserAuthority(user, aut2);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        userManager.addUserAuthority(user, aut2);
        User result = userManager.getById(user.getUserName());
        assertEquals(2, result.getAuthorities().size());
        userManager.removeUserAuthority(user, aut);
        result = userManager.getById(user.getUserName());
        assertEquals(1, result.getAuthorities().size());
        assertTrue(result.getAuthorities().contains(aut2));
        Authority aut3 = new Authority();
        aut3.setRole("role3");
        userManager.addUserAuthority(user, aut3);
        result = userManager.getById(user.getUserName());
        assertEquals(2, result.getAuthorities().size());
        assertTrue(result.getAuthorities().contains(aut3));
        assertTrue(result.getAuthorities().contains(aut2));
        userManager.removeUserAuthority(user, aut2);
        result = userManager.getById(user.getUserName());
        assertEquals(1, result.getAuthorities().size());
        assertTrue(result.getAuthorities().contains(aut3));
        userManager.removeUserAuthority(user, aut3);
        result = userManager.getById(user.getUserName());
        assertEquals(0, result.getAuthorities().size());
        userManager.deleteUser(user);

    }

    @Test
    public void testUpdateUser() {
        User user = createSampleUser(1);
        try {
            userManager.updateUser(null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        try {
            userManager.updateUser(user);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        userManager.addUser(user);
        user.setName("newName");
        userManager.updateUser(user);
        User result = userManager.getById(user.getUserName());
        assertEquals(user.getName(), result.getName());
        userManager.deleteUser(user);

    }

    @Test
    public void testChangePassword() {
        User user = createSampleUser(1);
        try {
            userManager.changePassword(null, "old", "new");
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        try {
            userManager.changePassword(user.getUserName(), null, "new");
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        try {
            userManager.changePassword(user.getUserName(), "old", null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        try {
            userManager.changePassword("neni", "old", "new");
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }

        userManager.addUser(user);
        try {
            userManager.changePassword(user.getUserName(), "old", "new");
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        userManager.changePassword(user.getUserName(), user.getPassword(), "newPass");
        User result = userManager.getById(user.getUserName());
        assertEquals("newPass", result.getPassword());
        userManager.deleteUser(user);

    }

    @Test
    public void testEnableDisable() {
        User user = createSampleUser(1);
        userManager.addUser(user);
        assertTrue(user.isEnabled());
        userManager.disable(user);
        User result = userManager.getById(user.getUserName());
        assertFalse(result.isEnabled());
        userManager.enable(user);
        result = userManager.getById(user.getUserName());
        assertTrue(result.isEnabled());
        userManager.deleteUser(user);
    }

}


